@@ -1,4 +1,4 @@
-# $Id: balloon.tcl,v 1.15 2005/03/13 21:14:21 aleksey Exp $
+# $Id: balloon.tcl,v 1.16 2005/09/27 23:14:09 aleksey Exp $
 
 
 option add *Balloon.background LightYellow widgetDefault
@@ -60,6 +60,11 @@ proc balloon::show {mx my} {
     variable screenwidth
     variable screenheight
 
+    if {[.balloon.text cget -text] == ""} {
+	balloon::destroy
+	return
+    }
+
     set balloon_showed 1
     set balloon_remove 0
 
