@@ -1,4 +1,4 @@
-# $Id: browser.tcl,v 1.50 2005/08/06 22:13:32 aleksey Exp $
+# $Id: browser.tcl,v 1.52 2006/01/28 20:45:08 aleksey Exp $
 
 option add *JBrowser.fill          Black	widgetDefault
 option add *JBrowser.nscolor       MidnightBlue widgetDefault
@@ -219,48 +219,36 @@ proc browser::process {bw from item level} {
     return $node
 }
 
-set browsericon(group)   [Bitmap::get [pixmap browser conference.gif]]
-set browsericon(user)    [Bitmap::get [pixmap browser user.gif]]
-set browsericon(jud)     [Bitmap::get [pixmap browser jud.gif]]
-set browsericon(aim)     [Bitmap::get [pixmap browser aim.gif]]
-set browsericon(icq)     [Bitmap::get [pixmap browser icq.gif]]
-set browsericon(msn)     [Bitmap::get [pixmap browser msn.gif]]
-set browsericon(gg)      [Bitmap::get [pixmap browser gg.gif]]
-set browsericon(weather) [Bitmap::get [pixmap browser weather.gif]]
-set browsericon(yahoo)   [Bitmap::get [pixmap browser yahoo.gif]]
-set browsericon(message) [Bitmap::get [pixmap browser message.gif]]
 image create photo ""
 
 proc browser::item_icon {category type} {
-    global browsericon
-
     switch -- $category {
 	service {
 	    switch -- $type {
-		jud {return $browsericon(jud)}
-		aim {return $browsericon(aim)}
-		icq {return $browsericon(icq)}
-		msn {return $browsericon(msn)}
-		x-gadugadu {return $browsericon(gg)}
-		x-weather {return $browsericon(weather)}
-		yahoo {return $browsericon(yahoo)}
-		default {return ""}
+		jud        { return browser/jud }
+		aim        { return browser/aim }
+		icq        { return browser/icq }
+		msn        { return browser/msn }
+		x-gadugadu { return browser/gg }
+		x-weather  { return browser/weather }
+		yahoo      { return browser/yahoo }
+		default    { return "" }
 	    }
 	}
 	application {
 	    switch -- $type {
-		x-weather {return $browsericon(weather)}
-		default {return ""}
+		x-weather { return browser/weather }
+		default   { return "" }
 	    }
 	}
 	conference {
-	    return $browsericon(group)
+	    return browser/conference
 	}
 	user {
-	    return $browsericon(user)
+	    return browser/user
 	}
 	headline {
-	    return $browsericon(message)
+	    return browser/message
 	}
 	default {
 	    return ""
@@ -536,8 +524,8 @@ proc browser::textpopup {bw node} {
 	jid {
 	    switch -- [lindex $data 2] {
 		user {
-		    message::subject_menu [set bm .b1popmenu] \
-			   [lindex $data 1] message -connection $browser(connid,$bw)
+		    message::subject_menu [set bm .b1popmenu] $browser(connid,$bw) \
+			   [lindex $data 1] message
 		}
 
 		conference {
