@@ -1,4 +1,4 @@
-# $Id: chats.tcl,v 1.138 2005/08/14 20:44:10 aleksey Exp $
+# $Id: chats.tcl,v 1.153 2006/05/28 14:54:24 aleksey Exp $
 
 package require textutil
 
@@ -36,8 +36,51 @@ namespace eval chat {
 	-type radio -group Chat \
 	-values [list normal [::msgcat::mc "Normal"] \
 		      chat [string trim [::msgcat::mc "Chat "]]]
-    set url_regexp \
-	{^([^\w\d]*)((http://|https://|www\.|ftp://|ftp\.).*[\w\d/])([^\w\d]*)$}
+
+    custom::defvar open_chat_list {} [::msgcat::mc "List of users for chat."] \
+	    -group Hidden
+
+    set url_regexp {^
+		([^\w\d]*)
+		(
+
+		 (?:
+			(?: ftp|https?)://[-\w]+(\.\w[-\w]*)+
+		   |
+			(?: [a-z0-9][-a-z0-9]* \. )+
+			(?: com
+			  | edu
+			  | biz
+			  | gov
+			  | in(?:t|fo)
+			  | mil
+			  | net
+			  | org
+			  | name
+			  | aero
+			  | arpa
+			  | coop
+			  | museum
+			  | pro
+			  | travel
+			  | [a-z][a-z]
+			)
+		)
+		(?: : \d+ )?
+
+		(?:
+			/
+			[^.,?!:;"'<>()\[\]{}\s\x7F-\xFF]*
+			(?:
+				[.,?!:;]+ [^.,?!:;"'<>()\[\]{}\s\x7F-\xFF]+
+
+			)*
+
+		)?
+	     )
+	     ([^\w\d]*)$
+    }
+
 }
 
 set chat_width 50
@@ -45,6 +88,75 @@ set chat_height 18
 
 plugins::load [file join plugins chat]
 
+proc chat::open_chat_dialog {} {
+    variable open_chat_jid
+    variable open_chat_list
+    variable open_chat_connid
+
+    if {[lempty [jlib::connections]]} return
+
+    set gw .open_chat
+    catch { destroy $gw }
+
+    set connid [lindex [jlib::connections] 0]
+    set open_chat_connid [jlib::connection_jid $connid]
+
+    Dialog $gw -title [::msgcat::mc "Open chat"] -separator 1 -anchor e \
+	    -default 0 -cancel 1
+
+    set gf [$gw getframe]
+    grid columnconfigure $gf 1 -weight 1
+
+    set open_chat_jid ""
+
+    label $gf.ljid -text [::msgcat::mc "JID:"]
+    ecursor_entry [ComboBox $gf.jid -textvariable [namespace current]::open_chat_jid \
+	    -values $open_chat_list -width 35].e
+    
+    grid $gf.ljid -row 0 -column 0 -sticky e
+    grid $gf.jid  -row 0 -column 1 -sticky ew
+
+    if {[llength [jlib::connections]] > 1} {
+	set connections {}
+	foreach c [jlib::connections] {
+	    lappend connections [jlib::connection_jid $c]
+	}
+	label $gf.lconnection -text [::msgcat::mc "Connection:"]
+	ComboBox $gf.connection -textvariable [namespace current]::open_chat_connid \
+				-values $connections
+
+	grid $gf.lconnection -row 1 -column 0 -sticky e
+	grid $gf.connection  -row 1 -column 1 -sticky ew
+    }
+
+    $gw add -text [::msgcat::mc "Chat"] -command "[namespace current]::open_chat $gw"
+    $gw add -text [::msgcat::mc "Cancel"] -command "destroy $gw"
+
+    $gw draw $gf.jid
+}
+
+proc chat::open_chat {gw} {
+    variable open_chat_jid
+    variable open_chat_list
+    variable open_chat_connid
+
+    destroy $gw
+
+    foreach c [jlib::connections] {
+	if {[jlib::connection_jid $c] == $open_chat_connid} {
+	    set connid $c
+	}
+    }
+    if {![info exists connid]} {
+	set connid [jlib::route $open_chat_jid]
+    }
+
+    set open_chat_list [update_combo_list $open_chat_list $open_chat_jid 20]
+    set custom::saved([namespace current]::open_chat_list) $open_chat_list
+    custom::store
+    open_to_user $connid $open_chat_jid
+}
+
 proc chat::get_nick {connid jid type} {
     variable chats
 
@@ -127,8 +239,6 @@ proc chat::process_message {connid from id type is_subject subject body err thre
     }
 
     set from [tolower_node_and_domain $from]
-    hook::run examine_message_hook $from $id $type $is_subject $subject $body $err \
-				   $thread $priority $x
 
     hook::run process_message_hook \
 	$connid $from $id $type $is_subject $subject $body $err $thread $priority $x
@@ -144,14 +254,6 @@ proc chat::process_message_fallback {connid from id type is_subject subject body
 
 
     switch -- $type {
-	normal {
-	    message::show_dialog $connid $from $type $subject $body $thread $priority $x
-	    return
-	}
-	headline {
-	    message::show_headline $from $type $subject $body $thread $priority $x
-	    return
-	}
 	chat {
 	    if {$thread != ""} {
 		set chats(thread,$chatid) $thread
@@ -166,7 +268,12 @@ proc chat::process_message_fallback {connid from id type is_subject subject body
 	    if {$is_subject} {
 		set_subject $chatid $subject
 		if {[cequal $body ""]} {
-		    set body [format [::msgcat::mc "/me has changed the subject to: %s"] $subject]
+		    set nick [resource_from_jid $from]
+		    if {[cequal $nick ""]} {
+			set body [format [::msgcat::mc "Subject is set to: %s"] $subject]
+		    } else {
+			set body [format [::msgcat::mc "/me has set the subject to: %s"] $subject]
+		    }
 		}
 	    }
 	}
@@ -267,7 +374,7 @@ proc chat::disconnect_groupchats {connid} {
 
 hook::add disconnected_hook [namespace current]::chat::disconnect_groupchats 99
 
-proc chat::open_window {chatid type} {
+proc chat::open_window {chatid type args} {
     global font font_bold font_italic font_bold_italic
     global chat_width chat_height
     variable opened
@@ -281,11 +388,20 @@ proc chat::open_window {chatid type} {
     set jid [get_jid $chatid]
     set jid [tolower_node_and_domain $jid]
     set chatid [chatid $connid $jid]
-    
+
     set cw [winid $chatid]
 
+    set cleanroster 1
+    foreach {key val} $args {
+	switch -- $key {
+	    -cleanroster { set cleanroster $val }
+	}
+    }
+
     if {[winfo exists $cw]} {
-	if {$type == "groupchat" && $chats(status,$chatid) == "disconnected"} {
+	if {$type == "groupchat" && \
+		$chats(status,$chatid) == "disconnected" && \
+		$cleanroster} {
 	    set grouproster(users,$chatid) {}
 	}
 
@@ -342,21 +458,6 @@ proc chat::open_window {chatid type} {
 	    pack $cw.status.desc -side left
 	}
     }
-    if {![cequal [info commands ::ssj::encrypt:toggleP] ""]} {
-        if {[cequal [set sjid [roster::find_jid $connid $jid]] ""]} {
-            set sjid $jid
-        }
-        Button $cw.status.encrypted -relief flat \
-            -image [message::encrypted:icon $connid $sjid] \
-            -helptype balloon \
-            -helptext [::msgcat::mc "Toggle encryption"] \
-            -command [list ssj::encrypt:toggleP $connid $sjid]
-        ssj::encrypted:trace "
-            $cw.status.encrypted configure \
-		-image \[message::encrypted:icon $connid $sjid\]
-        " $connid $sjid
-        pack $cw.status.encrypted -side left
-    }
     if {[cequal $type chat]} {
 	menubutton $cw.status.mb -text $jid -font $font \
 	    -menu $cw.status.mb.menu
@@ -369,13 +470,26 @@ proc chat::open_window {chatid type} {
 	create_conference_menu $cw.status.mb.menu $chatid
 	pack $cw.status.mb -side left
 
-	entry $cw.status.subject -font $font
+	entry $cw.status.subject -font $font \
+	    -xscrollcommand [list [namespace current]::set_subject_tooltip $chatid]
 	pack $cw.status.subject -side left -fill x -expand yes
 
 	bind $cw.status.subject <Return> \
 	    [list chat::change_subject [double% $chatid]]
+
 	bind $cw.status.subject <Escape> \
 	    [list chat::restore_subject [double% $chatid]]
+
+	bind $cw.status.subject <Any-Enter> \
+	    +[list eval balloon::set_text \
+		   \[[namespace current]::set_subject_balloon [list [double% $chatid]]\] \
+		   -width \[winfo width %W\]]
+
+	bind $cw.status.subject <Any-Motion> \
+	    [list eval balloon::on_mouse_move \
+		  \[[namespace current]::set_subject_balloon [list [double% $chatid]]\] %X %Y]
+
+	bind $cw.status.subject <Any-Leave> {+ balloon::destroy}
     }
     foreach tag [bind Menubutton] {
         if {[string first 1 $tag] >= 0} {
@@ -414,10 +528,12 @@ proc chat::open_window {chatid type} {
 	}
 
 	ifacetk::roster::create $uw.users -width $rosterwidth \
-	    -popup ifacetk::roster::groupchat_popup_menu
+	    -popup ifacetk::roster::groupchat_popup_menu \
+	    -singleclick [list [namespace current]::user_singleclick $chatid] \
+	    -doubleclick ::ifacetk::roster::jid_doubleclick
 	pack $uw.users -fill both -side right -expand yes
 	[winfo parent $uw] configure -width $rosterwidth
-	
+
 	set grouproster(users,$chatid) {}
 
 	set pack_in $cf
@@ -485,6 +601,26 @@ proc chat::open_window {chatid type} {
     hook::run open_chat_post_hook $chatid $type
 }
 
+###############################################################################
+
+proc chat::user_singleclick {chatid cjid} {
+    lassign $cjid connid jid
+    set nick [get_nick $connid $jid groupchat]
+    hook::run groupchat_roster_user_singleclick_hook $chatid $nick
+}
+
+###############################################################################
+
+proc chat::bind_window_click {chatid type} {
+    set cw [chat::chat_win $chatid]
+    bind $cw <ButtonPress-1><ButtonRelease-1> \
+	[list hook::run chat_window_click_hook [double% $chatid] %W %x %y]
+}
+
+hook::add open_chat_post_hook [namespace current]::chat::bind_window_click
+
+###############################################################################
+
 proc chat::close_window {chatid} {
     variable opened
     variable chats
@@ -505,14 +641,8 @@ proc chat::close_window {chatid} {
     debugmsg chat $chats(opened)
 
     if {[is_groupchat $chatid]} {
-	set command [list jlib::send_presence \
-			      -to $jid/[get_our_groupchat_nick $chatid] \
-			      -type unavailable \
-			      -connection $connid]
-	if {![cequal $chats(exit_status,$chatid) ""]} {
-	    lappend command -stat $chats(exit_status,$chatid)
-	}
-	eval $command
+	muc::leave_group $connid $jid [get_our_groupchat_nick $chatid] \
+			 $chats(exit_status,$chatid)
 	set chats(status,$chatid) disconnected
 	client:presence $connid $jid unavailable "" {}
 	foreach jid [get_jids_of_user $connid $jid] {
@@ -557,81 +687,40 @@ proc chat::is_groupchat {chatid} {
     }
 }
 
-proc chat::create_user_menu {path chatid} {
-    set m [menu $path -tearoff 0]
+proc chat::create_user_menu {m chatid} {
+    menu $m -tearoff 0
 
     set connid [get_connid $chatid]
     set jid [get_jid $chatid]
-    $m add command -label [::msgcat::mc "Invite to conference..."] \
-	-command [list chat::invite_dialog $jid 0 -connection $connid]
-    $m add command -label [::msgcat::mc "Add user..."] \
-	-command [list message::send_subscribe_dialog \
-		      [node_and_server_from_jid $jid] -connection $connid]
-    hook::run chat_create_user_menu_hook $path $chatid
+
+    hook::run chat_create_user_menu_hook $m $connid $jid
+
+    return $m
+}
+
+proc chat::create_conference_menu {m chatid} {
+    menu $m -tearoff 0
+
+    set connid [get_connid $chatid]
+    set jid [get_jid $chatid]
+
+    hook::run chat_create_conference_menu_hook $m $connid $jid
+
+    return $m
+}
+
+proc chat::add_separator {m connid jid} {
     $m add separator
-    $m add command -label [::msgcat::mc "Send users..."] \
-	-command [list ifacetk::roster::send_users_dialog $jid -connection $connid]
-    ft::create_menu $m $jid -connection $connid
-    $m add separator
-    $m add command -label [::msgcat::mc "Show info"] \
-	-command [list userinfo::open $jid -connection $connid]
-    $m add command -label [::msgcat::mc "Show history"] \
-	-command [list logger::show_log $jid]
-    return $path
 }
 
-proc chat::create_conference_menu {path chatid} {
-    set m [menu $path -tearoff 0]
-    set connid [get_connid $chatid]
-    set jid [get_jid $chatid]
-    $m add command -label [::msgcat::mc "Invite users..."] \
-	-command [list chat::invite_dialog2 $jid 0 -connection $connid]
-
-    if {[info exists ::muc::muc_password($chatid)]} {
-	set password $::muc::muc_password($chatid)
-    } else {
-	set password ""
-    }
-    $m add command -label [::msgcat::mc "Add conference..."] \
-	-command [list plugins::conferences::add_conference_dialog \
-		       -group [node_from_jid $jid] \
-		       -server [server_from_jid $jid] \
-		       -password $password \
-		       -connection $connid]
-    $m add command -label [::msgcat::mc "Show history"] \
-	-command [list logger::show_log $jid]
-    add_conference_custom_presence_menu $m $chatid
-    hook::run chat_create_conference_menu_hook $path $chatid
-    return $path
-}
-
-proc chat::add_conference_custom_presence_menu {m chatid} {
-    set mm [menu $m.custom_presence -tearoff 0]
-
-    set jid [get_jid $chatid]
-    set connid [get_connid $chatid]
-
-    $mm add command -label [::msgcat::mc "Available"] \
-	-command [list send_custom_presence $jid available \
-			   -type group -connection $connid]
-    $mm add command -label [::msgcat::mc "Free to chat"] \
-	-command [list send_custom_presence $jid chat \
-			   -type group -connection $connid]
-    $mm add command -label [::msgcat::mc "Away"] \
-	-command [list send_custom_presence $jid away \
-			   -type group -connection $connid]
-    $mm add command -label [::msgcat::mc "Extended away"] \
-	-command [list send_custom_presence $jid xa \
-			   -type group -connection $connid]
-    $mm add command -label [::msgcat::mc "Do not disturb"] \
-	-command [list send_custom_presence $jid dnd \
-			   -type group -connection $connid]
-    #$mm add command -label [::msgcat::mc "Unavailable"] \
-    #    -command [list send_custom_presence $jid unavailable \
-    #   		   -type group -connection $connid]
-
-    $m add cascad -label [::msgcat::mc "Send custom presence"] -menu $mm
-}
+hook::add chat_create_user_menu_hook chat::add_separator 40
+hook::add chat_create_user_menu_hook chat::add_separator 42
+hook::add chat_create_user_menu_hook chat::add_separator 50
+hook::add chat_create_user_menu_hook chat::add_separator 70
+hook::add chat_create_user_menu_hook chat::add_separator 85
+hook::add chat_create_conference_menu_hook chat::add_separator 40
+hook::add chat_create_conference_menu_hook chat::add_separator 42
+hook::add chat_create_conference_menu_hook chat::add_separator 50
 
 proc chat::add_chat_win_popup_menu {m chatwin X Y x y} {
     set tags [$chatwin tag names "@$x,$y"]
@@ -647,15 +736,12 @@ hook::add chat_win_popup_menu_hook \
     [namespace current]::chat::add_chat_win_popup_menu 10
 
 proc chat::copy_url {chatwin tag} {
-    regsub -all %% [$chatwin tag bind $tag <1>] % res
+    regsub -all %% [$chatwin tag bind $tag <ButtonPress-1><ButtonRelease-1>] % res
     # assuming "browseurl url"
     clipboard clear -displayof $chatwin
     clipboard append -displayof $chatwin [lrange $res 1 end]
 }
 
-proc chat::copy_url2 {chatwin ign url} {
-}
-
 proc chat::send_message {cw chatid type} {
     set iw $cw.input
 
@@ -709,7 +795,7 @@ proc chat::add_message {chatid from type body x} {
 		(!$options(smart_scroll) || \
 		     ($options(smart_scroll) && $scroll) || \
 		     [chat::is_our_jid $chatid $from])} {
-	    $chatw see end
+	    after idle [list catch [list $chatw yview moveto 1]]
 	}
 
 	$chatw configure -state disabled
@@ -721,7 +807,77 @@ proc chat::highlighttext {chatw tag color cursor} {
 	$chatw tag configure $tag -foreground $color
 }
 
-proc chat::add_emoteiconed_text {chatw body defaulttag} {
+proc chat::add_emoteiconed_text {chatw body defaulttag {highlightlist {}}} {
+    if {[lempty $highlightlist]} {
+	add_emoteiconed_text1 $chatw $body $defaulttag
+    } else {
+	set ind 0
+	foreach {i1 i2} $highlightlist {
+	    add_emoteiconed_text1 $chatw [crange $body $ind [expr {$i1 - 1}]] $defaulttag
+	    $chatw insert end [crange $body $i1 [expr {$i2 - 1}]] highlight
+	    set ind $i2
+	}
+	chat::add_emoteiconed_text1 $chatw [crange $body $ind end] $defaulttag
+    }
+}
+
+proc chat::encode_url {url} {
+    set utf8_url [encoding convertto utf-8 $url]
+    set len [string length $utf8_url]
+    set encoded_url ""
+    for {set i 0} {$i < $len} {incr i} {
+	binary scan $utf8_url @${i}c sym
+	set sym [expr {$sym & 0xFF}]
+	if {$sym >= 128 || $sym <= 32} {
+	    append encoded_url [format "%%%02X" $sym]
+	} else {
+	    append encoded_url [binary format c $sym]
+	}
+    }
+    return $encoded_url
+}
+
+proc chat::add_url {tw msg url args} {
+    variable urlid
+
+    set command ""
+    foreach {key val} $args {
+	switch -- $key {
+	    -command { set command $val }
+	}
+    }
+
+    set tag url$urlid
+    set urlfg    [option get $tw urlforeground       Text]
+    set urlactfg [option get $tw urlactiveforeground Text]
+    $tw tag configure $tag -foreground $urlfg -underline 1
+    $tw tag bind $tag <ButtonPress-1><ButtonRelease-1> \
+	[list browseurl [double% [encode_url $url]]]
+    if {$command != ""} {
+	$tw tag bind $tag <ButtonPress-1><ButtonRelease-1> \
+	    +[list eval $command]
+    }
+    $tw tag bind $tag <Any-Enter> \
+	[list chat::highlighttext $tw $tag $urlactfg hand2]
+    $tw tag bind $tag <Any-Leave> \
+	[list chat::highlighttext $tw $tag $urlfg xterm]
+	    
+    $tw insert end $msg $tag
+    incr urlid
+}
+
+proc chat::on_browse_url {chatid w x y} {
+    set tags [$w tag names "@$x,$y"]
+    set idx [lsearch -glob $tags url*]
+    if {$idx >= 0} {
+	return stop
+    }
+    return
+}
+
+hook::add chat_window_click_hook chat::on_browse_url 10
+	
+proc chat::add_emoteiconed_text1 {chatw body defaulttag} {
     variable url_regexp
     variable options
 
@@ -736,20 +892,10 @@ proc chat::add_emoteiconed_text {chatw body defaulttag} {
 		continue
 	    }
 	}
-	if {[regexp -- $url_regexp $word ignore head url ignore2 tail]} {
-	    variable urlid
-	    set tag url$urlid
-	    set urlfg    [option get $chatw urlforeground       Text]
-	    set urlactfg [option get $chatw urlactiveforeground Text]
-	    $chatw tag configure $tag -foreground $urlfg -underline 1
-	    $chatw tag bind $tag <1> [list browseurl [double% $url]]
-	    $chatw tag bind $tag <Any-Enter> \
-		[list chat::highlighttext $chatw $tag $urlactfg hand2]
-	    $chatw tag bind $tag <Any-Leave> \
-		[list chat::highlighttext $chatw $tag $urlfg xterm]
-	    
-	    $chatw insert end $head $defaulttag $url $tag $tail $defaulttag
-	    incr urlid
+	if {[regexp -expanded -nocase -- $url_regexp $word ignore head url ignore2 tail]} {
+	    $chatw insert end $head $defaulttag
+	    add_url $chatw $url $url
+	    $chatw insert end $tail $defaulttag
 	} else {
 	    if {0 && !$options(emphasize)} {
 		$chatw insert end $word $defaulttag
@@ -813,6 +959,20 @@ proc chat::switch_emphasize {args} {
     }
 }
 
+proc chat::add_open_to_user_menu_item {m connid jid} {
+    $m add command -label [::msgcat::mc "Start chat"] \
+	-command [list chat::open_to_user $connid $jid]
+}
+
+hook::add roster_create_groupchat_user_menu_hook \
+    [namespace current]::chat::add_open_to_user_menu_item 10
+hook::add roster_jid_popup_menu_hook \
+    [namespace current]::chat::add_open_to_user_menu_item 10
+hook::add message_dialog_menu_hook \
+    [namespace current]::chat::add_open_to_user_menu_item 10
+hook::add search_popup_menu_hook \
+    [namespace current]::chat::add_open_to_user_menu_item 10
+
 proc chat::open_to_user {connid user args} {
     set msg ""
     foreach {opt val} $args {
@@ -855,12 +1015,28 @@ proc chat::open_to_user {connid user args} {
     }
 }
 
-proc chat::change_presence {connid jid status} {
+###############################################################################
+
+proc chat::change_presence {connid jid type x args} {
     global grouproster
     variable chats
     variable opened
     global statusdesc
 
+    switch -- $type {
+	error -
+	unavailable { set status $type }
+	available {
+	    set status available
+	    foreach {key val} $args {
+		switch -- $key {
+		    -show { set status [normalize_show $val] }
+		}
+	    }
+	}
+	default { return }
+    }
+
     set group [node_and_server_from_jid $jid]
     set chatid [chatid $connid $group]
     set nick [get_nick $connid $jid groupchat]
@@ -893,6 +1069,7 @@ proc chat::change_presence {connid jid status} {
 		hook::run chat_user_exit $chatid $nick
 		lvarpop grouproster(users,$chatid) \
 		    [lsearch -exact $grouproster(users,$chatid) $jid]
+		catch { unset grouproster(status,$chatid,$jid) }
 		debugmsg chat "GR: $grouproster(users,$chatid)"
 		chat::redraw_roster_after_idle $chatid
 	    } else {
@@ -911,20 +1088,17 @@ proc chat::change_presence {connid jid status} {
 		    hook::run chat_user_enter $chatid $nick
 		}
 		set grouproster(status,$chatid,$jid) $status
-		ifacetk::roster::changeicon $userswin $jid \
-		    $::ifacetk::roster::rostericon(user,$status)
+		ifacetk::roster::changeicon $userswin $jid roster/user/$status
 		ifacetk::roster::changeforeground $userswin $jid $status
 		chat::redraw_roster_after_idle $chatid
 	    }
 	}
-
-	#roster:changeicon $w.users [user_from_jid $from] $status
     }
 
     set cw [winid [chatid $connid $jid]]
 
     if {[winfo exists $cw.status.icon]} {
-	$cw.status.icon configure -image $::ifacetk::roster::rostericon(user,$status) \
+	$cw.status.icon configure -image roster/user/$status \
 	    -helptext [get_user_status_desc $connid $jid]
     }
 
@@ -938,10 +1112,14 @@ proc chat::change_presence {connid jid status} {
 
     if {[winfo exists $cw.status.icon]} {
 	$cw.status.icon configure \
-	    -image $::ifacetk::roster::rostericon(user,[get_user_status $connid $user])
+	    -image roster/user/[get_user_status $connid $user]
     }
 }
 
+hook::add client_presence_hook chat::change_presence 70
+
+###############################################################################
+
 proc chat::redraw_roster {group} {
     global grouproster
 
@@ -950,7 +1128,6 @@ proc chat::redraw_roster {group} {
 
     set grouproster(users,$group) [lsort $grouproster(users,$group)]
     ifacetk::roster::clear $userswin 0
-    #roster::addline $userswin group Users users
 
     set levels {}
     foreach jid $grouproster(users,$group) {
@@ -1011,7 +1188,7 @@ proc chat::redraw_roster {group} {
 	    ifacetk::roster::addline $userswin jid $nick \
 		[list $connid $jid] [crange $level 1 end] 0
 	    ifacetk::roster::changeicon $userswin \
-		[list $connid $jid] $::ifacetk::roster::rostericon(user,$status)
+		[list $connid $jid] roster/user/$status
 	    if {$::plugins::nickcolors::options(use_colored_roster_nicks)} {
 		ifacetk::roster::changeforeground $userswin \
 		    [list $connid $jid] $status \
@@ -1074,6 +1251,29 @@ proc chat::change_subject {chatid} {
 	-type groupchat -subject $subject
 }
 
+proc chat::set_subject_balloon {chatid} {
+    variable chats
+    if {[info exists chats(subject_tooltip,$chatid)]} {
+	return $chats(subject_tooltip,$chatid)
+    } else {
+	return ""
+    }
+}
+
+proc chat::set_subject_tooltip {chatid lo hi} {
+    variable chats
+
+    set sw [winid $chatid].status.subject
+
+    if {![winfo exists $sw]} return
+
+    if {($lo == 0) && ($hi == 1)} {
+	set chats(subject_tooltip,$chatid) ""
+    } else {
+	set chats(subject_tooltip,$chatid) [$sw get]
+    }
+}
+
 proc chat::is_our_jid {chatid jid} {
     return [cequal [our_jid $chatid] $jid]
 }
@@ -1100,6 +1300,34 @@ proc chat::our_jid {chatid} {
     return ""
 }
 
+###############################################################################
+
+proc chat::add_invite_menu_item {m connid jid} {
+    $m add command -label [::msgcat::mc "Invite to conference..."] \
+	   -command [list chat::invite_dialog $jid 0 -connection $connid]
+}
+
+hook::add chat_create_user_menu_hook \
+    [namespace current]::chat::add_invite_menu_item 20
+hook::add roster_create_groupchat_user_menu_hook \
+    [namespace current]::chat::add_invite_menu_item 20
+hook::add roster_jid_popup_menu_hook \
+    [namespace current]::chat::add_invite_menu_item 20
+hook::add message_dialog_menu_hook \
+    [namespace current]::chat::add_invite_menu_item 20
+hook::add search_popup_menu_hook \
+    [namespace current]::chat::add_invite_menu_item 20
+
+proc chat::add_invite2_menu_item {m connid jid} {
+    $m add command -label [::msgcat::mc "Invite users..."] \
+	   -command [list chat::invite_dialog2 $jid 0 -connection $connid]
+}
+
+hook::add chat_create_conference_menu_hook \
+    [namespace current]::chat::add_invite2_menu_item 20
+
+###############################################################################
+
 proc chat::invite_dialog {user {ignore_muc 0} args} {
     variable chats
     variable opened
@@ -1221,18 +1449,11 @@ proc chat::invitation {jid usersP ignore_muc {reason ""}} {
 		set reas $reason
 	    }
 	    if {!$ignore_muc && [muc::is_compatible $group]} {
-		muc::invitation [chatid $con $group] $to $reas
+		muc::invite_muc $con $group $to $reas
 	    } else {
-		message::send_msg $to -type normal \
-		    -subject "Invitation" \
-		    -body $reas \
-		    -xlist [list [jlib::wrapper:createtag x \
-				-vars [list xmlns jabber:x:conference \
-				jid $group]]] \
-		    -connection $con
+		muc::invite_xconference $con $group $to $reas
 	    }
 	}
     }
 }
 
-
