@@ -46,7 +46,7 @@ foreach filename $tclFileList {
 	while {[string match {*\\} $line] && \
 		    ![string match {*\\\\} $line] && \
 		    [gets $fd line1] >= 0} {
-	    set line [string replace $line end end " $line1"]
+	    set line [string replace $line end end " [string trimleft $line1]"]
 	}
         # Search: [ ::msgcat::mc "translation key"
         while {[regexp -- {\[::msgcat::mc[ \t\r\n]+\"(([^\"]|\\\")*)\"} $line whole key] || \
@@ -82,7 +82,7 @@ for {set i 0} {$i < [llength $fileList]} {incr i} {
     }
 }
 
-if {0 != [string compare "" $translationFile]} {
+if {0 != [string compare "" $translationFile] && [file exists $translationFile]} {
     # Read translation file
     set fd [open $translationFile]
     set translated [list]
@@ -127,14 +127,16 @@ if {0 != [string compare "" $translationFile]} {
         }
     }
 } else {
-    # Print result
-    foreach f [array names keyHash] {
-        if {0 < [llength $keyHash($f)]} {
-            puts "# $f"
-            foreach k $keyHash($f) {
+    if {false == $invertMatch} {
+	# Print result
+	foreach f [array names keyHash] {
+	    if {0 < [llength $keyHash($f)]} {
+		puts "# $f"
+		foreach k $keyHash($f) {
                     puts "::msgcat::mcset $lang \"$k\""
-            }
-            puts ""
-        }
+		}
+		puts ""
+	    }
+	}
     }
 }
