@@ -1,10 +1,16 @@
-# $Id: custom.tcl,v 1.23 2005/03/13 21:14:21 aleksey Exp $
+# $Id: custom.tcl,v 1.26 2006/05/11 22:07:25 aleksey Exp $
 
 option add *Customize.varforeground       blue widgetDefault
 option add *Customize.groupnameforeground blue widgetDefault
 
 namespace eval custom {
+    # Filename for saving options
     set options(customfile) [file join ~ .tkabber custom.tcl]
+
+    # -1: stored values haven't been restored yet (only config changes vars)
+    # 0: stored values are being restored now
+    # 1: stored values have been restored (changes may be stored)
+    set custom_loaded -1
 }
 
 proc custom::defgroup {id doc args} {
@@ -48,7 +54,11 @@ proc custom::defvar {vname value doc args} {
 
     if {![info exists $fullname]} {
 	set $fullname $value
+    } else {
+	set var(config,$fullname) $value
     }
+    trace variable $fullname w \
+	[list [namespace current]::on_var_change $fullname]
 
     set var(default,$fullname) $value
     set var(doc,$fullname) $doc
@@ -57,14 +67,13 @@ proc custom::defvar {vname value doc args} {
 
     foreach {attr val} $args {
 	switch -- $attr {
-	    -type {set var(type,$fullname) $val}
+	    -type {
+		set var(type,$fullname) $val
+	    }
 	    -group {
 		lappend group(members,$val) [list var $fullname]
 		#set group(members,$val) [lrmdups $group(members,$val)]
 	    }
-	    -command {
-		trace variable $fullname w $val
-	    }
 	    -values {
 		set var(values,$fullname) $val
 	    }
@@ -86,6 +95,27 @@ proc custom::defvar {vname value doc args} {
 	    }
 	}
     }
+    foreach {attr val} $args {
+	switch -- $attr {
+	    -command {
+		trace variable $fullname w $val
+	    }
+	}
+    }
+}
+
+proc custom::on_var_change {varname args} {
+    variable options
+    variable var
+    variable custom_loaded
+
+    switch -- $custom_loaded {
+	-1 {
+	    set var(config,$varname) [set $varname]
+	}
+	0 { }
+	1 { }
+    }
 }
 
 proc custom::add_radio_options {vname values} {
@@ -101,7 +131,7 @@ proc custom::add_radio_options {vname values} {
 }
 
 custom::defgroup Tkabber \
-    [::msgcat::mc "Customization of the One True Jabber Client."] \
+    [::msgcat::mc "Customization of the One True Jabber Client."]
 
 custom::defgroup Hidden "Hidden group" -group Tkabber -tag "Hidden group" \
     -type hidden
@@ -208,6 +238,7 @@ proc custom::fill_group {t gid} {
 		       -command [list [namespace current]::goto $parent]]
 	    $t window create end -window $b
 	    $t insert end " "
+	    bindscroll $b $t
 	    
 	    incr i
 	}
@@ -243,6 +274,7 @@ proc custom::fill_group {t gid} {
 			}
 		    }
 		}
+		bindscroll $b $t
 
 		$t insert end " $group(doc,$data)"
 
@@ -278,6 +310,7 @@ proc custom::fill_var {t varname idx} {
 	    set e [entry $t.entry$idx \
 		       -textvariable [namespace current]::tmp($varname)]
 	    $t window create end -window $e
+	    bindscroll $e $t
 	    $t insert end "\n"
 	}
 
@@ -289,6 +322,7 @@ proc custom::fill_var {t varname idx} {
 	    set e [entry $t.entry$idx -show * \
 		       -textvariable [namespace current]::tmp($varname)]
 	    $t window create end -window $e
+	    bindscroll $e $t
 	    $t insert end "\n"
 	}
 
@@ -300,6 +334,7 @@ proc custom::fill_var {t varname idx} {
 	    set cb [checkbutton $t.cb$idx -cursor left_ptr \
 			-variable [namespace current]::tmp($varname)]
 	    $t window create end -window $cb
+	    bindscroll $cb $t
 	    $t insert end "\n"
 	}
 
@@ -310,6 +345,7 @@ proc custom::fill_var {t varname idx} {
 	    set tmp($varname) [set $varname]
 	    set e [Spinbox $t.spin$idx -1000000000 1000000000 1 [namespace current]::tmp($varname)]
 	    $t window create end -window $e
+	    bindscroll $e $t
 	    $t insert end "\n"
 	}
 
@@ -358,9 +394,11 @@ proc custom::fill_var {t varname idx} {
 			    -text $displ -value $val \
 			    -variable [namespace current]::tmp($varname)]
 		pack $rb -anchor $anchor -side $side
+		bindscroll $rb $t
 		incr i
 	    }
 	    $t window create end -window $fr -align top
+	    bindscroll $fr $t
 	    $t insert end "\n"
 	}
 
@@ -377,7 +415,9 @@ proc custom::fill_var {t varname idx} {
 			-command [list [namespace current]::on_set_font \
 				      $fr.selectfont $varname]]
 	    pack $sf
+	    bindscroll $sf $t
 	    $t window create end -window $fr
+	    bindscroll $fr $t
 	    $t insert end "\n"
 	}
 
@@ -394,6 +434,8 @@ proc custom::fill_var {t varname idx} {
 		     -command [list [namespace current]::get_filename $varname]]
 	    $t window create end -window $e
 	    $t window create end -window $browse
+	    bindscroll $e $t
+	    bindscroll $browse $t
 	    $t insert end "\n"
 	}
 
@@ -407,10 +449,12 @@ proc custom::fill_var {t varname idx} {
 	       -menu $t.stb$idx.statemenu -relief $::tk_relief]
     create_state_menu $b.statemenu $varname
     $t window create end -window $b
+    bindscroll $b $t
     set l [label $t.stl$idx \
 	       -textvariable [namespace current]::var(state,$varname)]
     $t insert end " "
     $t window create end -window $l
+    bindscroll $l $t
     $t insert end "\n"
 
     $t insert end "$var(doc,$varname)\n"
@@ -465,6 +509,11 @@ proc custom::on_edit {varname args} {
     variable saved
 
     set is_default [cequal [set $varname] $var(default,$varname)]
+    if {[info exists var(config,$varname)]} {
+	set is_config [cequal [set $varname] $var(config,$varname)]
+    } else {
+	set is_config -1
+    }
     set is_current [cequal [set $varname] $tmp($varname)]
     if {[info exists saved($varname)]} {
 	set is_saved [cequal [set $varname] $saved($varname)]
@@ -473,15 +522,16 @@ proc custom::on_edit {varname args} {
     }
 
     if {!$is_current} {
-	set st [::msgcat::mc "you have edited the value, but you have not set the option."]
+	set st [::msgcat::mc "value is changed, but the option is not set."]
     } else {
-	switch -glob -- $is_default,$is_saved {
-	    1,1 -
-	    1,-1 {set st [::msgcat::mc "this option is unchanged from its standard setting."]}
-	    1,0 -
-	    0,0 -
-	    0,-1 {set st [::msgcat::mc "you have set this option, but not saved it for future sessions."]}
-	    0,1 {set st [::msgcat::mc "this option has been set and saved."]}
+	switch -glob -- $is_default,$is_config,$is_saved {
+	    0,0,1 -
+	    0,-1,1 {set st [::msgcat::mc "the option is set and saved."]}
+	    *,*,0 -
+	    0,0,-1 -
+	    0,-1,-1 {set st [::msgcat::mc "the option is set, but not saved."]}
+	    *,1,* {set st [::msgcat::mc "the option is taken from config file."]}
+	    1,*,* {set st [::msgcat::mc "the option is set to its default value."]}
 	}
     }
 
@@ -490,20 +540,27 @@ proc custom::on_edit {varname args} {
 
 
 proc custom::create_state_menu {m varname} {
+    variable var
+    variable saved
+
     if {[winfo exists $m]} {
 	destroy $m
     }
 
     menu $m
-    $m add command -label [::msgcat::mc "Set for Current Session"] \
+    $m add command -label [::msgcat::mc "Set for current session only"] \
 	-command [list [namespace current]::set_for_current_sess $varname]
-    $m add command -label [::msgcat::mc "Set for Future Sessions"] \
+    $m add command -label [::msgcat::mc "Set for current and future sessions"] \
 	-command [list [namespace current]::save_var $varname]
-    $m add command -label [::msgcat::mc "Reset to Current"] \
+    $m add command -label [::msgcat::mc "Reset to current value"] \
 	-command [list [namespace current]::reset_to_current $varname]
-    $m add command -label [::msgcat::mc "Reset to Saved"] \
-	-command [list [namespace current]::reset_to_saved $varname]
-    $m add command -label [::msgcat::mc "Reset to Default"] \
+    $m add command -label [::msgcat::mc "Reset to saved value"] \
+	-command [list [namespace current]::reset_to_saved $varname] \
+	-state [expr {[info exists saved($varname)] ? "normal" : "disabled"}]
+    $m add command -label [::msgcat::mc "Reset to value from config file"] \
+	-command [list [namespace current]::reset_to_config $varname] \
+	-state [expr {[info exists var(config,$varname)] ? "normal" : "disabled"}]
+    $m add command -label [::msgcat::mc "Reset to default value"] \
 	-command [list [namespace current]::reset_to_default $varname]
 
     return $m
@@ -542,6 +599,19 @@ proc custom::reset_to_saved {varname} {
     on_edit $varname
 }
 
+proc custom::reset_to_config {varname} {
+    variable var
+    variable tmp
+    variable saved
+
+    if {![info exists var(config,$varname)]} return
+
+    set tmp($varname) $var(config,$varname)
+    set $varname $var(config,$varname)
+
+    on_edit $varname
+}
+
 proc custom::reset_to_default {varname} {
     variable var
     variable tmp
@@ -575,9 +645,15 @@ proc custom::store {} {
     fconfigure $fd -encoding utf-8
 
     foreach varname [array names saved] {
-	if {![info exists var(default,$varname)] || \
-		($saved($varname) != $var(default,$varname))} {
-	    puts $fd [list [list $varname $saved($varname)]]
+	if {[info exists var(config,$varname)]} {
+	    if {$saved($varname) != $var(config,$varname)} {
+		puts $fd [list [list $varname $saved($varname)]]
+	    }
+	} else {
+	    if {![info exists var(default,$varname)] || \
+		    $saved($varname) != $var(default,$varname)} {
+		puts $fd [list [list $varname $saved($varname)]]
+	    }
 	}
     }
 
@@ -589,8 +665,14 @@ proc custom::restore {} {
     variable var
     variable saved
     variable options
+    variable custom_loaded
 
-    if {![file readable $options(customfile)]} return
+    set custom_loaded 0
+
+    if {![file readable $options(customfile)]} {
+	set custom_loaded 1
+	return
+    }
 
     set fd [open $options(customfile) r]
     fconfigure $fd -encoding utf-8
@@ -604,6 +686,8 @@ proc custom::restore {} {
 	set saved($varname) $value
 	catch {set $varname $value}
     }
+
+    set custom_loaded 1
 }
 hook::add postload_hook custom::restore 60
 
