@@ -1,4 +1,4 @@
-# $Id: datagathering.tcl,v 1.34 2005/08/04 20:33:02 aleksey Exp $
+# $Id: datagathering.tcl,v 1.38 2006/05/11 22:07:25 aleksey Exp $
 
 namespace eval data {
     set winid 0
@@ -39,11 +39,11 @@ proc data::fill_fields {g items} {
 	jlib::wrapper:splitxml $item tag vars isempty chdata children
 	set xmlns [jlib::wrapper:getattr $vars xmlns]
 	if {$xmlns == "jabber:x:data" || $xmlns == "jabber:iq:data"} {
-	    fill_fields_x $g $children
-	    return
+	    return [fill_fields_x $g $children]
 	}
     }
 
+    set focus ""
     set fields {}
     foreach item $items {
 	jlib::wrapper:splitxml $item tag vars isempty chdata children
@@ -73,23 +73,29 @@ proc data::fill_fields {g items} {
 	switch -- $tag {
 	    key {
 		entry $g.$row \
-		    -textvariable [namespace current]::data(var,$g,$tag) \
+		    -textvariable [namespace current]::data(var,$tag,$g) \
 		    -state disabled -font $font
 	    }
 	    password {
 		entry $g.$row \
-		    -textvariable [namespace current]::data(var,$g,$tag) \
+		    -textvariable [namespace current]::data(var,$tag,$g) \
 		    -show * -font $font
+		if {$focus == ""} {
+		    set focus $g.row
+		}
 	    }
 	    default {
 		entry $g.$row \
-		    -textvariable [namespace current]::data(var,$g,$tag) \
+		    -textvariable [namespace current]::data(var,$tag,$g) \
 		    -font $font
+		if {$focus == ""} {
+		    set focus $g.row
+		}
 	    }
 	}
 
 	if {$chdata != {}} {
-	    set data(var,$g,$tag) $chdata
+	    set data(var,$tag,$g) $chdata
 	}
 
 	grid $g.l$row -row $row -column 0 -sticky e
@@ -97,8 +103,14 @@ proc data::fill_fields {g items} {
 
 	incr row
     }
+    return $focus
 }
 
+proc data::cleanup {g} {
+    variable data
+
+    array unset data *,$g
+}
 
 proc data::get_tags {g} {
     variable data
@@ -111,7 +123,7 @@ proc data::get_tags {g} {
 
     foreach var $data(varlist,$g) {
 	lappend restags [jlib::wrapper:createtag $var \
-			     -chdata $data(var,$g,$var)]
+			     -chdata $data(var,$var,$g)]
     }
 
     return $restags
@@ -189,6 +201,7 @@ proc data::fill_fields_x {g items} {
     set data(x,$g) 1
     set row 0
     set data(allvarlist,$g) {}
+    set focus ""
 
     foreach item $items {
 	jlib::wrapper:splitxml $item tag vars isempty chdata children
@@ -202,7 +215,10 @@ proc data::fill_fields_x {g items} {
 		incr row
 	    }
 	    field {
-		fill_field_x $g $row $tag $vars $chdata $children
+		set widget [fill_field_x $g $row $tag $vars $chdata $children]
+		if {$focus == ""} {
+		    set focus $widget
+		}
 		incr row
 	    }
 	    title {
@@ -220,6 +236,8 @@ proc data::fill_fields_x {g items} {
 
     # FIX THIS
     set data(varlist,$g) $data(allvarlist,$g)
+
+    return $focus
 }
 
 
@@ -234,14 +252,15 @@ proc data::fill_field_x {g row tag vars chdata childrens} {
     set var   [jlib::wrapper:getattr $vars var]
     set type  [jlib::wrapper:getattr $vars type]
     set label [jlib::wrapper:getattr $vars label]
-    set data(var,$g,$var) ""
+    set data(var,$var,$g) ""
+    set widget ""
 
     foreach item $childrens {
 	jlib::wrapper:splitxml $item tag1 vars1 isempty1 chdata1 children1
 	switch $tag1 {
 	    required {set required 1}
 	    value {
-		set data(var,$g,$var) $chdata1
+		set data(var,$var,$g) $chdata1
 		lappend vals $chdata1
 	    }
 	    desc {set desc $chdata1}
@@ -265,12 +284,13 @@ proc data::fill_field_x {g row tag vars chdata childrens} {
 	text-private {
 	    add_label $g $row $label $required
 	    entry $g.entry$row \
-		-textvariable [namespace current]::data(var,$g,$var) \
+		-textvariable [namespace current]::data(var,$var,$g) \
 		-font $font
 	    if {$type == "text-private"} {
 		$g.entry$row configure -show *
 	    }
 	    grid $g.entry$row  -row $row -column 1 -sticky we
+	    set widget $g.entry$row
 	}
 	jid-multi -
 	text-multi {
@@ -280,20 +300,40 @@ proc data::fill_field_x {g row tag vars chdata childrens} {
 	    $sw setwidget $g.text$row
 	    bind $g.text$row <Control-Key-Return> { }
 	    bind $g.text$row <Return> "[bind Text <Return>]\nbreak"
-	    set data(var,$g,$var) [join $vals \n]
-	    catch { $g.text$row insert end $data(var,$g,$var) }
+	    set data(var,$var,$g) [join $vals \n]
+	    catch { $g.text$row insert end $data(var,$var,$g) }
 	    grid $sw -row $row -column 1 -sticky we
-	    set data(text,$g,$var) $g.text$row
+	    set data(text,$var,$g) $g.text$row
+	    set widget $g.text$row
 	}
 	boolean {
+	    switch -- $data(var,$var,$g) {
+		1 -
+		0 {
+		    set onvalue 1
+		    set offvalue 0
+		}
+		true -
+		false {
+		    set onvalue true
+		    set offvalue false
+		}
+		default {
+		    set onvalue 1
+		    set offvalue 0
+		    set data(var,$var,$g) 0
+		}
+	    }
 	    add_label $g $row $label $required
 	    checkbutton $g.cb$row \
-		-variable [namespace current]::data(var,$g,$var)
+		-variable [namespace current]::data(var,$var,$g) \
+		-onvalue $onvalue -offvalue $offvalue
 	    grid $g.cb$row  -row $row -column 1 -sticky w
+	    set widget $g.cb$row
 	}
 	fixed {
 	    add_label $g $row $label $required
-	    catch { message $g.m$row -text $data(var,$g,$var) -width 10c }
+	    catch { message $g.m$row -text $data(var,$var,$g) -width 10c }
 	    grid $g.m$row -row $row -column 1 \
 		-sticky w
 	    set dont_report 1
@@ -302,10 +342,10 @@ proc data::fill_field_x {g row tag vars chdata childrens} {
 	    add_label $g $row $label $required
 	    set height 0
 	    foreach {lab val} $options {
-		lappend data(combol$row,$g,$var) $lab
+		lappend data(combol$row,$var,$g) $lab
 		incr height
-		if {[string equal $data(var,$g,$var) $val]} {
-		    set data(combov$row,$g,$var) $lab
+		if {[string equal $data(var,$var,$g) $val]} {
+		    set data(combov$row,$var,$g) $lab
 		}
 	    }
 	    if {$height > 10} {
@@ -315,13 +355,14 @@ proc data::fill_field_x {g row tag vars chdata childrens} {
 			-font $font \
 			-height $height \
 			-editable no \
-			-values $data(combol$row,$g,$var) \
+			-values $data(combol$row,$var,$g) \
 			-textvariable \
-			[namespace current]::data(combov$row,$g,$var)]
+			[namespace current]::data(combov$row,$var,$g)]
 	    grid $cb -row $row -column 1 -sticky we
-	    trace variable [namespace current]::data(combov$row,$g,$var) w \
+	    trace variable [namespace current]::data(combov$row,$var,$g) w \
 		[list data::trace_combo $options \
-		     [namespace current]::data(var,$g,$var)]
+		     [namespace current]::data(var,$var,$g)]
+	    set widget $g.combo$row
 	}
 	list-multi {
 	    add_label $g $row $label $required
@@ -336,12 +377,13 @@ proc data::fill_field_x {g row tag vars chdata childrens} {
 		}
 	    }
 	    grid $sw  -row $row -column 1 -sticky we
-	    set data(multi,$g,$var) 1
+	    set data(multi,$var,$g) 1
 	    trace_listmulti $l $options \
-		data::data(var,$g,$var)
+		data::data(var,$var,$g)
 	    bind $l <<ListboxSelect>> \
 		[list data::trace_listmulti %W $options \
-		     [namespace current]::data(var,$g,$var)]
+		     [namespace current]::data(var,$var,$g)]
+	    set widget $sw
 	}
 	hidden {}
 
@@ -353,6 +395,7 @@ proc data::fill_field_x {g row tag vars chdata childrens} {
     if {![info exists dont_report]} {
 	lappend data(allvarlist,$g) $var
     }
+    return $widget
 }
 
 proc data::trace_combo {assoc dst name1 name2 op} {
@@ -378,22 +421,22 @@ proc data::get_tags_x {g} {
     set restags {}
 
     foreach var $data(varlist,$g) {
-	if {[info exists data(multi,$g,$var)]} {
+	if {[info exists data(multi,$var,$g)]} {
 	    set vartags {}
-	    foreach val $data(var,$g,$var) {
+	    foreach val $data(var,$var,$g) {
 	        lappend vartags [jlib::wrapper:createtag value \
 	        		     -chdata $val]
 	    }
-	} elseif {[info exists data(text,$g,$var)]} {
-	    set data(var,$g,$var) [$data(text,$g,$var) get 1.0 "end -1c"]
+	} elseif {[info exists data(text,$var,$g)]} {
+	    set data(var,$var,$g) [$data(text,$var,$g) get 1.0 "end -1c"]
 	    set vartags {}
-	    foreach val [split $data(var,$g,$var) \n] {
+	    foreach val [split $data(var,$var,$g) \n] {
 	        lappend vartags [jlib::wrapper:createtag value \
 	        		     -chdata $val]
 	    }
 	} else {
 	    set vartags [list [jlib::wrapper:createtag value \
-	        		   -chdata $data(var,$g,$var)]]
+	        		   -chdata $data(var,$var,$g)]]
 	}
 	lappend restags [jlib::wrapper:createtag field \
 			     -vars [list var $var] \
@@ -447,15 +490,16 @@ proc data::draw_window {items send_cmd {cancel_cmd destroy}} {
 	#-command [list eval $cancel_cmd [list $w]]
     bind $w <Return> "ButtonBox::invoke $bbox default"
     bind $w <Escape> "ButtonBox::invoke $bbox 1"
+    bind $f <Destroy> [list [namespace current]::cleanup $f]
 
     bindscroll $f $sf
 
     pack [Separator $w.sep] -side bottom -fill x  -pady 1m
 
-    set hf [frame $w.hf]
+    set hf [frame $w.error]
     pack $hf -side top
     set vf [frame $w.vf]
-    pack $vf -side left
+    pack $vf -side left -pady 2m
     pack $sw -side top -expand yes -fill both -padx 2m -pady 2m
 
     update idletasks
@@ -466,7 +510,7 @@ proc data::draw_window {items send_cmd {cancel_cmd destroy}} {
     if {$h > $sh - 200} {
 	set h [expr {$sh - 200}]
     }
-    $vf configure -height [expr {$h + [winfo pixels $f 4m]}]
+    $vf configure -height $h
     wm deiconify $w
 
     return $w
@@ -474,8 +518,17 @@ proc data::draw_window {items send_cmd {cancel_cmd destroy}} {
 
 ###############################################################################
 
-
 proc data::request_data {xmlns jid node args} {
+
+    foreach {key val} $args {
+	switch -- $key {
+	    -connection { set connid $val }
+	}
+    }
+    if {![info exists connid]} {
+	return -code error "data::request_data error: -connection required"
+    }
+
     set vars [list xmlns $xmlns]
     if {$node != ""} {
 	lappend vars node $node
@@ -484,15 +537,24 @@ proc data::request_data {xmlns jid node args} {
     jlib::send_iq get \
 	[jlib::wrapper:createtag query \
 	     -vars $vars] \
-	-to $jid -command [list [namespace current]::receive_data $xmlns $jid $node]
+	-to $jid \
+	-connection $connid \
+	-command [list [namespace current]::receive_data $connid $xmlns $jid $node]
 }
 
-proc data::receive_data {xmlns jid node res child} {
-    if {![cequal $res OK]} {
-	MessageDlg .data_err -aspect 50000 -icon error \
-	    -message [format \
-			  [::msgcat::mc "Error requesting data: %s"] \
-			  [error_to_string $child]] \
+proc data::receive_data {connid xmlns jid node res child} {
+    if {[cequal $res DISCONNECT]} {
+	return
+    }
+
+    if {[cequal $res ERR]} {
+	set ew .data_err
+	if {[winfo exists $ew]} {
+	    destroy $ew
+	}
+	MessageDlg $ew -aspect 50000 -icon error \
+	    -message [::msgcat::mc "Error requesting data: %s" \
+				   [error_to_string $child]] \
 	    -type user -buttons ok -default 0 -cancel 0
 	return
     }
@@ -507,10 +569,10 @@ proc data::receive_data {xmlns jid node res child} {
     }
 
     data::draw_window $children \
-	[list [namespace current]::send_data $xmlns $jid $node]
+	[list [namespace current]::send_data $connid $xmlns $jid $node]
 }
 
-proc data::send_data {xmlns jid node w restags} {
+proc data::send_data {connid xmlns jid node w restags} {
     switch -- $xmlns {
 	jabber:iq:data {
 	    set child [lindex $restags 0]
@@ -529,24 +591,33 @@ proc data::send_data {xmlns jid node w restags} {
     if {$node != ""} {
         lappend vars node $node
     }
-    
+
+    destroy $w.error.msg
+    $w.bbox itemconfigure 0 -state disabled
+
     jlib::send_iq set [jlib::wrapper:createtag query \
 			   -vars $vars \
 			   -subtags $children] \
 	-to $jid \
-	-command [list [namespace current]::test_error_res $xmlns $jid $node]
-
-    destroy $w
+	-connection $connid \
+	-command [list [namespace current]::test_error_res $w]
 }
 
-proc data::test_error_res {xmlns jid node res child} {
-    if {[cequal $res OK]} return
+proc data::test_error_res {w res child} {
+    if {[cequal $res OK]} {
+	destroy $w
+	return
+    }
 
-    MessageDlg .data_err -aspect 50000 -icon error \
-	-message [format \
-		      [::msgcat::mc "Error submitting data: %s"] \
-		      [error_to_string $child]] \
-	-type user -buttons ok -default 0 -cancel 0
+    $sw.bbox itemconfigure 0 -state normal
+
+    set m [message $w.error.msg \
+		   -aspect 50000 \
+		   -text [error_to_string $child] \
+		   -font $font \
+		   -pady 2m]
+    $m configure -foreground [option get $m errorForeground Message]
+    pack $m
 }
 
 disco::browser::register_feature_handler jabber:iq:data \
