@@ -1,4 +1,4 @@
-# $Id: disco.tcl,v 1.37 2005/08/06 22:13:32 aleksey Exp $
+# $Id: disco.tcl,v 1.42 2006/03/17 04:29:42 aleksey Exp $
 
 set ::NS(disco_items) "http://jabber.org/protocol/disco#items"
 set ::NS(disco_info)  "http://jabber.org/protocol/disco#info"
@@ -10,6 +10,7 @@ option add *JDisco.optioncolor   DarkViolet     widgetDefault
 
 namespace eval disco {
     variable supported_nodes
+    variable supported_features {}
 }
 
 proc disco::request_items {jid node args} {
@@ -201,7 +202,7 @@ proc disco::parse_info {jid node handler res child} {
 
     debugmsg disco \
 	"INFO: IDENTITIES [list $identities] FEATURES [list $features]\
-EXTRAS [list $extras]"
+	 EXTRAS [list $extras]"
 
     if {$handler != ""} {
 	eval $handler [list OK $identities $features $extras]
@@ -232,6 +233,8 @@ proc disco::get_jid_identities {jid node} {
 proc disco::info_query_get_handler {connid from child} {
     variable supported_nodes
     variable node_handlers
+    variable supported_features
+    variable feature_handlers
 
     jlib::wrapper:splitxml $child tag vars isempty chdata children
     set node [jlib::wrapper:getattr $vars node]
@@ -261,6 +264,10 @@ proc disco::info_query_get_handler {connid from child} {
 	    lappend restags [jlib::wrapper:createtag feature \
 				 -vars [list var $ns]]
 	}
+	foreach ns $supported_features {
+	    lappend restags [jlib::wrapper:createtag feature \
+				 -vars [list var $ns]]
+	}
     
 	set res [jlib::wrapper:createtag query \
 		     -vars [list xmlns $::NS(disco_info)] \
@@ -272,13 +279,7 @@ proc disco::info_query_get_handler {connid from child} {
 iq::register_handler get query $::NS(disco_info) \
     [namespace current]::disco::info_query_get_handler
 
-proc disco::register_node {node handler {name ""}} {
-    variable supported_nodes
-    variable node_handlers
-
-    set supported_nodes($node) $name
-    set node_handlers($node) $handler
-}
+###############################################################################
 
 proc disco::items_query_get_handler {connid from child} {
     variable supported_nodes
@@ -333,22 +334,63 @@ proc disco::items_query_get_handler {connid from child} {
 iq::register_handler get query $::NS(disco_items) \
     [namespace current]::disco::items_query_get_handler
 
+###############################################################################
+
+proc disco::register_feature {feature {handler ""}} {
+    variable supported_features
+    variable feature_handlers
+
+    lappend supported_features $feature
+    set feature_handlers($feature) $handler
+}
+
+###############################################################################
+
+proc disco::register_node {node handler {name ""}} {
+    variable supported_nodes
+    variable node_handlers
+
+    set supported_nodes($node) $name
+    set node_handlers($node) $handler
+}
+
+###############################################################################
+
+proc disco::publish_items {jid node action items args} {
+
+    set command ""
+    foreach {key val} $args {
+	switch -- {
+	    -connection { set connid $val }
+	    -command { set command $val }
+	}
+    }
+    if {![info exists connid]} {
+	return "disco::publish_items: option -connection required"
+    }
+
+    jlib::send_iq set \
+	[jlib::wrapper:createtag query \
+	     -vars [list xmlns $::NS(disco#publish) node $node]
+	     -subtags $items] \
+	-to $jid \
+	-connection $connid \
+	-command [list [namespace current]::publish_items_result $command]
+
+}
+
+proc disco::publish_items_result {command res child} {
+    if {$command != ""} {
+	eval $command [list $res $child]
+    }
+}
+
 ###############################################################################
 # Disco Browser
 
 namespace eval disco::browser {
     set winid 0
 
-    set icon(group)   [Bitmap::get [pixmap browser conference.gif]]
-    set icon(client)  [Bitmap::get [pixmap browser user.gif]]
-    set icon(jud)     [Bitmap::get [pixmap browser jud.gif]]
-    set icon(aim)     [Bitmap::get [pixmap browser aim.gif]]
-    set icon(icq)     [Bitmap::get [pixmap browser icq.gif]]
-    set icon(msn)     [Bitmap::get [pixmap browser msn.gif]]
-    set icon(gg)      [Bitmap::get [pixmap browser gg.gif]]
-    set icon(weather) [Bitmap::get [pixmap browser weather.gif]]
-    set icon(yahoo)   [Bitmap::get [pixmap browser yahoo.gif]]
-    set icon(message) [Bitmap::get [pixmap browser message.gif]]
     image create photo ""
 
     custom::defvar disco_list {} [::msgcat::mc "List of discovered JIDs."] \
@@ -449,6 +491,9 @@ proc disco::browser::open_win {jid args} {
     bind $tw.c <Return> \
 	"[namespace current]::textaction $w \[$tw selection get\]"
 
+    bind $tw.c <Delete> \
+	"[namespace current]::clear $w \[$tw selection get\]"
+
     lappend browser(opened) $w
     set browser(opened) [lrmdups $browser(opened)]
     set browser(required,$w) {}
@@ -544,10 +589,10 @@ proc disco::browser::draw_info {w jid node res identities features extras} {
     foreach identity $identities {
 	set tnode [jid_to_tag "identity $identity $jid $node"]
 	lappend identitynodes $tnode
-	set data [list identity $jid $node]
 	set name     [jlib::wrapper:getattr $identity name]
 	set category [jlib::wrapper:getattr $identity category]
 	set type     [jlib::wrapper:getattr $identity type]
+	set data [list identity $jid $node $category $type]
 	set desc "$name ($category/$type)"
 	set icon [item_icon $category $type]
 	
@@ -743,6 +788,7 @@ proc disco::browser::reorder_node {tw node {order {}}} {
 
     set identities {}
     set features {}
+    set extras {}
     set items {}
     foreach sn $subnodes {
 	lassign [$tw itemcget $sn -data] kind
@@ -752,12 +798,13 @@ proc disco::browser::reorder_node {tw node {order {}}} {
 	    error_info -
 	    identity {lappend identities $sn}
 	    feature  {lappend features   $sn}
+	    extra    {lappend extras     $sn}
 	}
     }
     if {$order == {}} {
-	$tw reorder $node [concat $identities $features $items]
+	$tw reorder $node [concat $identities $extras $features $items]
     } else {
-	$tw reorder $node [concat $identities $features $order]
+	$tw reorder $node [concat $identities $extras $features $order]
     }
 }
 
@@ -787,48 +834,47 @@ proc disco::browser::item_desc {jid node name} {
 }
 
 proc disco::browser::item_icon {category type} {
-    variable icon
     switch -- $category {
 	gateway {
 	    switch -- $type {
-		aim {return $icon(aim)}
-		icq {return $icon(icq)}
-		msn {return $icon(msn)}
-		yahoo {return $icon(yahoo)}
-		x-gadugadu {return $icon(gg)}
-		default {return ""}
+		aim        { return browser/aim }
+		icq        { return browser/icq }
+		msn        { return browser/msn }
+		yahoo      { return browser/yahoo }
+		x-gadugadu { return browser/gg }
+		default    { return ""}
 	    }
 	}
 	service {
 	    switch -- $type {
-		jud {return $icon(jud)}
-		aim {return $icon(aim)}
-		icq {return $icon(icq)}
-		msn {return $icon(msn)}
-		x-gadugadu {return $icon(gg)}
-		x-weather {return $icon(weather)}
-		yahoo {return $icon(yahoo)}
-		default {return ""}
+		jud        { return browser/jud }
+		aim        { return browser/aim }
+		icq        { return browser/icq }
+		msn        { return browser/msn }
+		x-gadugadu { return browser/gg }
+		x-weather  { return browser/weather }
+		yahoo      { return browser/yahoo }
+		default    { return "" }
 	    }
 	}
 	application {
 	    switch -- $type {
-		x-weather {return $icon(weather)}
-		default {return ""}
+		x-weather { return browser/weather }
+		default   { return "" }
 	    }
 	}
 	conference {
-	    return $icon(group)
+	    return browser/conference
 	}
 	user -
 	client {
-	    return $icon(client)
+	    return browser/user
 	}
 	directory {
-	    return $icon(jud)
+	    return browser/jud
 	}
 	headline {
-	    return $icon(message)
+	    return browser/message
 	}
 	default {
 	    return ""
@@ -885,7 +931,6 @@ proc disco::browser::textpopup {bw tnode} {
 
     switch -- $type {
 	identity {
-	    return
 	}
 
 	feature {
@@ -900,8 +945,8 @@ proc disco::browser::textpopup {bw tnode} {
 	    set category [jlib::wrapper:getattr [lindex $identities 0] category]
 	    switch -- $category {
 		client {
-		    message::subject_menu $m \
-			   [lindex $data 1] message -connection $browser(connid,$bw)
+		    message::subject_menu $m $browser(connid,$bw) \
+			   [lindex $data 1] message
 		    tk_popup $m [winfo pointerx .] [winfo pointery .]
 		    return
 		}
@@ -934,9 +979,48 @@ proc disco::browser::textpopup {bw tnode} {
 	}
     }
 
+    $m add separator
+    $m add command -label [::msgcat::mc "Clear"] \
+	-command [list [namespace current]::clear $bw $tnode]
+    $m add command -label [::msgcat::mc "Clear All"] \
+	-command [list [namespace current]::clearall $bw]
+
     tk_popup $m [winfo pointerx .] [winfo pointery .]
 }
 
+proc disco::browser::clearall {bw} {
+    variable browser
+    set tw $browser(tree,$bw)
+     
+    set subnodes [$tw nodes root]
+    foreach sn $subnodes {
+	$tw delete $sn
+    }
+}
+
+proc disco::browser::clear {bw tnode} {
+    variable browser
+    set tw $browser(tree,$bw)
+
+    set tparentnode [$tw parent $tnode]
+    
+    set data [$tw itemcget $tnode -data]
+    set data2 [lassign $data type]
+
+    if {$tparentnode != "root"} {
+	if {$type == "item"} {
+	    set subnodes [$tw nodes $tnode]
+	} else {
+	    set subnodes [$tw nodes $tparentnode]
+	}
+	foreach sn $subnodes {
+	    $tw delete $sn
+	}
+    } else {
+	$tw delete $tnode
+    }
+}
+
 proc disco::browser::browser_action {action bw tnode} {
     variable browser
 
@@ -1016,7 +1100,12 @@ proc disco::browser::textballoon {bw action X Y node} {
     variable browser
 
     set tw $browser(tree,$bw)
-    set data [$tw itemcget $node -data]
+
+    if {[catch { set data [$tw itemcget $node -data] }]} {
+	balloon::destroy
+	return
+    }
+
     set data2 [lassign $data type]
     if {$type == "jid"} {
 	    lassign $data2 jid category type name version
@@ -1048,6 +1137,17 @@ proc disco::browser::goto {bw jid node} {
     go $bw
 }
 
+proc disco::browser::get_category_type {t tnode} {
+    foreach child [$t nodes $tnode] {
+	set data [$t itemcget $child -data]
+        set data2 [lassign $data type jid node]
+	if {$type == "identity"} {
+	    return $data2
+	}
+    }
+    return {}
+}
+
 proc disco::browser::draginitcmd {t tnode top} {
     set data [$t itemcget $tnode -data]
     set data2 [lassign $data type jid node]
@@ -1056,8 +1156,21 @@ proc disco::browser::draginitcmd {t tnode top} {
 	if {[set img [$t itemcget $tnode -image]] != ""} {
 	    pack [label $top.l -image $img -padx 0 -pady 0]
 	}
-	
-	return [list JID {copy} [list $jid "" "" "" ""]]
+
+	lassign [get_category_type $t $tnode] category type
+
+	if {![info exists category]} {
+	    # Using parent tag to get conference cathegory
+	    # ??? Which else cathegory could be got from parent?
+	    lassign [get_category_type $t [$t parent $tnode]] category type
+
+	    if {![info exists category] || ($category != "conference")} {
+		set conference ""
+		set type ""
+	    }
+	}
+
+	return [list JID {copy} [list $jid $category $type "" ""]]
     } else {
 	return {}
     }
