@@ -439,7 +439,7 @@ if {(![file exists [set file .jsendrc.tcl]]) \
 source $file
 
 namespace eval zerobot {
-    proc examine_message {from id type is_subject subject body err thread priority x} {
+    proc examine_message {connid from id type is_subject subject body err thread priority x} {
         global userstatus
 
         switch -- $userstatus {
@@ -459,5 +459,5 @@ namespace eval zerobot {
         }
     }
 
-    hook::add examine_message_hook   [namespace current]::examine_message   10
+    hook::add process_message_hook   [namespace current]::examine_message   10
 }
