@@ -1,4 +1,4 @@
-# $Id: filetransfer.tcl,v 1.28 2005/03/03 01:55:50 aleksey Exp $
+# $Id: filetransfer.tcl,v 1.29 2006/01/20 03:08:00 aleksey Exp $
 
 namespace eval ft {
     
@@ -30,19 +30,29 @@ namespace eval ft {
 }
 
 
-proc ft::create_menu {m jid args} {
+proc ft::create_menu {m connid jid} {
     variable options
     
     if {$options(cascaded_menu)} {
 	set mm [menu $m.filetransfer -tearoff 0]
-	eval { hook::run create_filetransfer_menu_hook $mm 1 $jid } $args
+	hook::run create_filetransfer_menu_hook $mm 1 $jid -connection $connid
 	if {![hook::is_flag create_filetransfer_menu_hook transport]} {
 	    $m add cascad -label [::msgcat::mc "Send file"] -menu $mm
 	}
     } else {
-	eval { hook::run create_filetransfer_menu_hook $m 0 $jid } $args
+	hook::run create_filetransfer_menu_hook $m 0 $jid -connection $connid
     }
     
 }
 
+hook::add chat_create_user_menu_hook \
+    [namespace current]::ft::create_menu 46
+hook::add roster_create_groupchat_user_menu_hook \
+    [namespace current]::ft::create_menu 46
+hook::add roster_jid_popup_menu_hook \
+    [namespace current]::ft::create_menu 46
+hook::add message_dialog_menu_hook \
+    [namespace current]::ft::create_menu 46
+hook::add search_popup_menu_hook \
+    [namespace current]::ft::create_menu 46
 
