@@ -1,4 +1,4 @@
-# $Id: gpgme.tcl,v 1.29 2005/07/31 21:09:36 aleksey Exp $
+# $Id: gpgme.tcl,v 1.33 2006/03/28 19:12:17 aleksey Exp $
 
 if {[catch { package require gpgme }]} {
     debugmsg ssj "unable to load the GPGME package, so no crypto!"
@@ -8,7 +8,7 @@ if {[catch { package require gpgme }]} {
     set have_gpgme 1
 }
 
-namespace eval ssj {
+namespace eval ::ssj {
     variable options
 
     custom::defgroup GPGME [::msgcat::mc "GPGME options (signing and encryption)."] \
@@ -35,7 +35,7 @@ namespace eval ssj {
 package require base64
 
 
-namespace eval ssj {
+namespace eval ::ssj {
     variable ctx
     variable e4me
     variable j2k
@@ -55,21 +55,23 @@ namespace eval ssj {
     array set passphrase {}
 
     array set s2e \
-          [list none       "No information available"                         \
-                bad        "Invalid signature"                                \
-                nokey      "Signature not processed due to missing key"       \
-                nosig      "Malformed signature block"                        \
-                error      "Error in signature processing"                    \
-                diff       "Multiple signatures having different authenticity"\
-                expired    "The signature is good but has expired"            \
-                expiredkey "The signature is good but the key has expired"]
+          [list none       [::msgcat::mc "No information available"] \
+                bad        [::msgcat::mc "Invalid signature"] \
+                nokey      [::msgcat::mc "Signature not processed due to missing key"] \
+                nosig      [::msgcat::mc "Malformed signature block"] \
+                error      [::msgcat::mc "Error in signature processing"] \
+                diff       [::msgcat::mc "Multiple signatures having different authenticity"] \
+                expired    [::msgcat::mc "The signature is good but has expired"] \
+                expiredkey [::msgcat::mc "The signature is good but the key has expired"]]
 
     catch { unset warnings }
     array set warnings {}
+
+    variable signedid 0
 }
 
 
-proc ssj::once_only {connid {armorP 0}} {
+proc ::ssj::once_only {connid {armorP 0}} {
     global env
     variable options
     variable ctx
@@ -91,7 +93,7 @@ proc ssj::once_only {connid {armorP 0}} {
     if {![info exists env(GPG_AGENT_INFO)]} {
         $ctx($connid) -operation set                 \
 		      -property  passphrase-callback \
-		      -value     [list [namespace current]::passphrase $connid]
+		      -value     [list ::ssj::passphrase $connid]
     }
 
     set pattern [jlib::connection_bare_jid $connid]
@@ -170,13 +172,13 @@ proc ssj::once_only {connid {armorP 0}} {
     }
 
     CbDialog $dw [format [::msgcat::mc "Select Key for Signing %s Traffic"] $pattern] \
-        [list [::msgcat::mc "Select"] "[namespace current]::once_only_aux $dw $connid" \
+        [list [::msgcat::mc "Select"] "::ssj::once_only_aux $dw $connid" \
 	      [::msgcat::mc "Cancel"] "destroy $dw"] \
-	[namespace current]::selectkey$connid $titles $balloons \
+	::ssj::selectkey$connid $titles $balloons \
 	-modal local
 }
 
-proc ssj::once_only_aux {dw connid} {
+proc ::ssj::once_only_aux {dw connid} {
     variable selectkey$connid
 
     set keys {}
@@ -194,7 +196,7 @@ proc ssj::once_only_aux {dw connid} {
 }
 
 
-proc ssj::passphrase {connid data} {
+proc ::ssj::passphrase {connid data} {
     variable passphrase
     variable options
 
@@ -260,7 +262,7 @@ proc ssj::passphrase {connid data} {
 
     label $pf.lpassword -text [::msgcat::mc "Passphrase:"]
     entry $pf.password  \
-	  -textvariable [namespace current]::passphrase($connid,$keyid) \
+	  -textvariable ::ssj::passphrase($connid,$keyid) \
           -show *
     set passphrase($connid,$keyid) ""
 
@@ -289,7 +291,7 @@ proc ssj::passphrase {connid data} {
 }
 
 
-proc ssj::armor:encode {text} {
+proc ::ssj::armor:encode {text} {
     if {[set x [string first "\n\n" $text]] >= 0} {
         set text [string range $text [expr $x+2] end]
     }
@@ -300,11 +302,11 @@ proc ssj::armor:encode {text} {
     return $text
 }
 
-proc ssj::armor:decode {text} {
+proc ::ssj::armor:decode {text} {
     return "-----BEGIN PGP MESSAGE-----\n\n$text\n-----END PGP MESSAGE-----"
 }
 
-proc ssj::signed:input {connid from signature data what} {
+proc ::ssj::signed:input {connid from signature data what} {
     variable ctx
     variable j2k
     variable s2e
@@ -383,7 +385,7 @@ proc ssj::signed:input {connid from signature data what} {
 }
 
 
-proc ssj::signed:output {connid data args} {
+proc ::ssj::signed:output {connid data args} {
     variable ctx
     variable options
     variable warnings
@@ -429,7 +431,7 @@ proc ssj::signed:output {connid data args} {
     return $result
 }
 
-proc ssj::signed:info {pinfo} {
+proc ::ssj::signed:info {pinfo} {
 
     set text ""
     array set params $pinfo
@@ -463,16 +465,16 @@ proc ssj::signed:info {pinfo} {
 			}
 		    }
 		}
-
 		created {
 		    append info "created: [clock format $v]\n"
 		}
-
+		expires {
+		    append info "expires: [clock format $v]\n"
+		}
 		fingerprint {
 		    append info [format "keyid: 0x%s\n" [string range $v end-7 end]]
 		    append info [format "%s: %s\n" $k $v]
 		}
-
 		default {
 		    if {![cequal $v ""]} {
 			append info [format "%s: %s\n" $k $v]
@@ -492,14 +494,12 @@ proc ssj::signed:info {pinfo} {
     return [string trimleft $text]
 }
 
-proc ssj::signed:Label {lb pinfo} {
-    global messageicon
-
+proc ::ssj::signed:Label {lb pinfo} {
     array set params $pinfo
     if {[info exists params(reason)]} {
-	set args [list -image $messageicon(badsigned)]
+	set args [list -image gpg/badsigned]
     } else {
-	set args [list -image $messageicon(signed)]
+	set args [list -image gpg/signed]
     }
 
     if {![cequal [set info [signed:info $pinfo]] ""]} {
@@ -510,43 +510,46 @@ proc ssj::signed:Label {lb pinfo} {
 	 -padx 0 -pady 0 -borderwidth 0 -highlightthickness 0
 
     if {[info exists params(reason)] && [cequal $params(reason) nokey]} {
-	bind $lb <3> [list [namespace current]::signed:popup $pinfo]
+	bind $lb <3> [list ::ssj::signed:popup $pinfo]
     }
     return $lb
 }
 
-proc ssj::signed:popup {pinfo} {
+###############################################################################
+
+proc ::ssj::signed:popup {pinfo} {
     set m .signed_label_popupmenu
     if {[winfo exists $m]} {
 	destroy $m
     }
     menu $m -tearoff 0
     $m add command -label [::msgcat::mc "Fetch GPG key"] \
-	-command [list [namespace current]::fetchkeys $pinfo]
+	-command [list ::ssj::fetchkeys $pinfo]
     tk_popup $m [winfo pointerx .] [winfo pointery .]
 }
 
-proc ssj::signed:user_menu {m connid jid} {
-    global presence
+proc ::ssj::signed:user_menu {m connid jid} {
+    variable signed
     global curuser
 
     if {[cequal $jid "\$curuser"]} {
 	set jid $curuser
     }
-    if {[info exists presence(signed,$connid,$jid)]} {
-	array set params $presence(signed,$connid,$jid)
+    if {[info exists signed($connid,$jid)]} {
+	array set params $signed($connid,$jid)
 	if {[info exists params(status)] && [cequal $params(status) nokey]} {
 	    $m add command -label [::msgcat::mc "Fetch GPG key"] \
-		-command [list [namespace current]::fetchkeys \
-			       $presence(signed,$connid,$jid)]
+		-command [list ::ssj::fetchkeys \
+			       $signed($connid,$jid)]
 	}
     }
 }
 
-hook::add roster_create_user_menu_edit_hook \
-    [namespace current]::ssj::signed:user_menu 60
+hook::add chat_create_user_menu_hook ::ssj::signed:user_menu 78
 
-proc ssj::fetchkeys {pinfo} {
+###############################################################################
+
+proc ::ssj::fetchkeys {pinfo} {
     variable gpg_error_id
 
     array set params $pinfo
@@ -571,7 +574,7 @@ proc ssj::fetchkeys {pinfo} {
     }
 }
 
-proc ssj::encrypted:input {connid from data} {
+proc ::ssj::encrypted:input {connid from data} {
     variable ctx
     variable warnings
     variable gpg_error_id
@@ -602,7 +605,7 @@ proc ssj::encrypted:input {connid from data} {
 }
 
 
-proc ssj::encrypted:output {connid data to} {
+proc ::ssj::encrypted:output {connid data to} {
     variable ctx
     variable e4me
     variable j2k
@@ -641,9 +644,13 @@ proc ssj::encrypted:output {connid data to} {
 
     once_only $connid 1
 
-    set code [catch { $ctx($connid) -operation   encrypt \
-				    -input       [binary format a* [encoding convertto utf-8 $data]] \
-				    -recipients $recipient } result]
+    set code \
+	[catch {
+	    $ctx($connid) \
+		-operation encrypt \
+		-input [binary format a* [encoding convertto utf-8 $data]] \
+		-recipients $recipient
+	 } result]
 
     rename $recipient {}
 
@@ -652,9 +659,19 @@ proc ssj::encrypted:output {connid data to} {
 
         set options(encrypt,$connid,$jid) 0
         incr gpg_error_id
-        if {[MessageDlg .encrypt_error$gpg_error_id -aspect 50000 -icon error -type user \
-                -buttons {ok cancel} -default 0 -cancel 1 \
-                -message [format [::msgcat::mc "Unable to encipher data for %s: %s.\n\nEncrypting traffic to this user is now disabled.\n\nSend it as PLAINTEXT?"] $to $result]]} {
+        if {[MessageDlg .encrypt_error$gpg_error_id \
+		-aspect 50000 \
+		-icon error \
+		-type user \
+                -buttons {ok cancel} \
+		-default 0 \
+		-cancel 1 \
+                -message [format \
+			      [::msgcat::mc \
+				   "Unable to encipher data for %s:\
+				    %s.\n\nEncrypting traffic to this user is\
+				    now disabled.\n\nSend it as PLAINTEXT?"] \
+			      $to $result]]} {
             error ""
         }
 
@@ -667,7 +684,7 @@ proc ssj::encrypted:output {connid data to} {
     return $result
 }
 
-proc ssj::whichkeys {connid what} {
+proc ::ssj::whichkeys {connid what} {
     variable ctx
     variable warnings
 
@@ -694,8 +711,9 @@ proc ssj::whichkeys {connid what} {
     debugmsg ssj "${what}ing with $keys"
 }
 
+#############################################################################
 
-proc ssj::prefs {connid jid} {
+proc ::ssj::prefs {connid jid} {
     variable ctx
     variable options
     variable optionsX
@@ -712,7 +730,7 @@ proc ssj::prefs {connid jid} {
 	   -separator 1 -anchor e -default 0 -cancel 1
 
     $w add -text [::msgcat::mc "OK"] \
-	   -command [list [namespace current]::prefs_ok $w $connid $jid]
+	   -command [list ::ssj::prefs_ok $w $connid $jid]
     $w add -text [::msgcat::mc "Cancel"] -command [list destroy $w]
 
     set f [$w getframe]
@@ -724,7 +742,7 @@ proc ssj::prefs {connid jid} {
     set optionsX(encrypt,$connid,$jid) $options(encrypt,$connid,$jid)
     checkbutton $f.encrypt \
         -text     [::msgcat::mc "Encrypt traffic"] \
-        -variable [namespace current]::optionsX(encrypt,$connid,$jid)
+        -variable ::ssj::optionsX(encrypt,$connid,$jid)
 
     pack $f.encrypt -side left
     pack [frame $f.f -width 9c -height 2c]
@@ -732,7 +750,7 @@ proc ssj::prefs {connid jid} {
     $w draw $f.name
 }
 
-proc ssj::prefs_ok {w connid jid} {
+proc ::ssj::prefs_ok {w connid jid} {
     variable options
     variable optionsX
 
@@ -741,26 +759,26 @@ proc ssj::prefs_ok {w connid jid} {
     destroy $w
 }
 
-proc ssj::prefs_user_menu {m connid jid} {
-    global curuser
-
-    if {[cequal $jid "\$curuser"]} {
-	set jid $curuser
-    }
+proc ::ssj::prefs_user_menu {m connid jid} {
+    set rjid [roster::find_jid $connid $jid]
     $m add command -label [::msgcat::mc "Edit security..."] \
-	-command [list [namespace current]::prefs $connid $jid]
+	-command [list ::ssj::prefs $connid $rjid]
 }
 
-hook::add roster_create_user_menu_edit_hook \
-    [namespace current]::ssj::prefs_user_menu
+hook::add chat_create_user_menu_hook ::ssj::prefs_user_menu 78
+hook::add roster_conference_popup_menu_hook ::ssj::prefs_user_menu 78
+hook::add roster_service_popup_menu_hook ::ssj::prefs_user_menu 78
+hook::add roster_jid_popup_menu_hook ::ssj::prefs_user_menu 78
 
-proc ssj::signP {} {
+#############################################################################
+
+proc ::ssj::signP {} {
     variable options
 
     return $options(sign-traffic)
 }
 
-proc ssj::encryptP {connid jid} {
+proc ::ssj::encryptP {connid jid} {
     variable ctx
     variable j2k
     variable options
@@ -825,8 +843,9 @@ proc ssj::encryptP {connid jid} {
     return $options(encrypt-tried,$connid,$jid)
 }
 
+#############################################################################
 
-proc ssj::e4meP {connid keys} {
+proc ::ssj::e4meP {connid keys} {
     variable ctx
     variable e4me
     variable signers
@@ -852,13 +871,15 @@ proc ssj::e4meP {connid keys} {
     }
 }
 
-proc ssj::sign:toggleP {} {
+#############################################################################
+
+proc ::ssj::sign:toggleP {} {
     variable options
 
     set options(sign-traffic) [expr {!$options(sign-traffic)}]
 }
 
-proc ssj::encrypt:toggleP {{connid ""} {jid ""}} {
+proc ::ssj::encrypt:toggleP {{connid ""} {jid ""}} {
     variable options
 
     if {[cequal $jid ""]} {
@@ -870,28 +891,30 @@ proc ssj::encrypt:toggleP {{connid ""} {jid ""}} {
 	if {![info exists options(encrypt,$connid,$jid)]} {
 	    set options(encrypt,$connid,$jid) [encryptP $connid $jid]
 	}
-	set options(encrypt,$connid,$jid) [expr {!$options(encrypt,$connid,$jid)}]
+	set options(encrypt,$connid,$jid) \
+	    [expr {!$options(encrypt,$connid,$jid)}]
     } else {
-	return -code error "ssj::encrypt:toggleP: connid is empty and jid is not"
+	return -code error \
+	    "::ssj::encrypt:toggleP: connid is empty and jid is not"
     }
 }
 
+#############################################################################
 
-proc ssj::signed:trace {script} {
+proc ::ssj::signed:trace {script} {
     variable options
     variable trace
 
     if {![info exists trace(sign-traffic)]} {
         set trace(sign-traffic) {}
 
-        ::trace variable [namespace current]::options(sign-traffic) w \
-                [namespace current]::trace
+        ::trace variable ::ssj::options(sign-traffic) w ::ssj::trace
     }
 
     lappend trace(sign-traffic) $script
 }
 
-proc ssj::encrypted:trace {script {connid ""} {jid ""}} {
+proc ::ssj::encrypted:trace {script {connid ""} {jid ""}} {
     variable options
     variable trace
 
@@ -901,20 +924,20 @@ proc ssj::encrypted:trace {script {connid ""} {jid ""}} {
 	if {![cequal $connid ""]} {
 	    set k encrypt,$connid,$jid
 	} else {
-	    return -code error "ssj::encrypted:trace: connid is empty and jid is not"
+	    return -code error \
+		"::ssj::encrypted:trace: connid is empty and jid is not"
 	}
     }
     if {![info exists trace($k)]} {
         set trace($k) {}
 
-        ::trace variable [namespace current]::options($k) w \
-                [namespace current]::trace
+        ::trace variable ::ssj::options($k) w ::ssj::trace
     }
 
     lappend trace($k) $script
 }
 
-proc ssj::trace {name1 name2 op} {
+proc ::ssj::trace {name1 name2 op} {
     variable trace
 
     set new {}
@@ -927,3 +950,393 @@ proc ssj::trace {name1 name2 op} {
     }
     set trace($name2) $new
 }
+
+#############################################################################
+
+proc ::ssj::clear_signatures {connid} {
+    variable signed
+
+    if {$connid == {}} {
+	catch { array unset signed }
+    } else {
+	catch { array unset signed $connid,* }
+    }
+}
+
+hook::add disconnected_hook ::ssj::clear_signatures
+
+#############################################################################
+
+proc ::ssj::check_signature {connid from type x args} {
+    variable signed
+
+    switch -- $type {
+	unavailable -
+	available {
+	    catch { unset signed($connid,$from) }
+
+	    set signature ""
+	    foreach xs $x {
+		jlib::wrapper:splitxml $xs tag vars isempty cdata children
+		if {[jlib::wrapper:getattr $vars xmlns] == $::NS(signed)} {
+		    set signature $cdata
+		    break
+		}
+	    }
+
+	    # in case the sender didn't check the exit code from gpg...
+	    if {[cequal $signature ""]} return
+
+	    set status ""
+	    foreach {key val} $args {
+		switch -- $key {
+		    -status { set status $val }
+		}
+	    }
+
+	    set signed($connid,$from) \
+		[signed:input $connid $from $signature $status \
+		     [::msgcat::mc "Presence information"]]
+	}
+    }
+}
+
+hook::add client_presence_hook ::ssj::check_signature
+
+#############################################################################
+
+proc ::ssj::make_signature {acc connid status} {
+    if {![cequal $status ""] && \
+	    ![catch { signed:output $connid $status } cdata] && \
+	    ![cequal $cdata ""]} {
+	lappend acc [jlib::wrapper:createtag x \
+			 -vars [list xmlns $::NS(signed)] \
+			 -chdata $cdata]
+    }
+    return $acc
+}
+
+hook::add presence_xlist_hook ::ssj::make_signature
+
+#############################################################################
+
+proc ::ssj::userinfo {tab connid jid editable} {
+    variable signed
+
+    if {$editable} return
+
+    set bare_jid [node_and_server_from_jid $jid]
+    set chatid [chat::chatid $connid $bare_jid]
+    if {[chat::is_groupchat $chatid]} {
+	if {[info exists signed($connid,$jid)]} {
+	    set jids [list $connid,$jid]
+	} else {
+	    set jids [list]
+	}
+    } else {
+	set jids [array names signed $connid,$bare_jid/*]
+    }
+    if {[llength $jids] > 0} {
+	set presenceinfo [$tab insert end presenceinfo \
+			      -text [::msgcat::mc "Presence"]]
+	set i 0
+	foreach j $jids {
+	    regexp {[^,]*,(.*)} $j -> fjid
+	    set x [userinfo::pack_frame $presenceinfo.presence_$i $fjid]
+	    catch { array unset params }
+	    array set params $signed($j)
+
+	    set kv {}
+	    set addrs ""
+	    set s ""
+	    foreach signature $params(signatures) {
+		foreach {k v} $signature {
+		    switch -- $k {
+			key {
+			    foreach {k v} $v {
+				if {![cequal $k subkeys]} continue
+
+				foreach subkey $v {
+				    catch { unset sparams }
+				    array set sparams $subkey
+				    if {[info exists sparams(email)]} {
+					append addrs $s $sparams(email)
+					set s ", "
+				    }
+				}
+			    }
+			    continue
+			}
+			status { continue }
+			created -
+			expires { set v [clock format $v] }
+			fingerprint {
+			    lappend kv keyid \
+				[format "0x%s" [string range $v end-7 end]]
+			}
+			default {
+			    if {[cequal $v ""]} { continue }
+			}	
+		    }
+
+		    lappend kv $k $v
+		}
+	    }
+
+
+	    userinfo::pack_entry $jid $x $i presence_$i [::msgcat::mc "Reason:"]
+	    if {![info exists params(reason)]} {
+		set params(reason) [::msgcat::mc "Presence is signed"]
+		if {![cequal $addrs ""]} {
+		    append params(reason) [::msgcat::mc " by "] $addrs
+		}
+	    }
+	    set userinfo::userinfo(presence_$i,$jid) $params(reason)
+	    incr i
+
+	    foreach {k v} $kv {
+		userinfo::pack_entry $jid $x $i presence_$i \
+		    [::msgcat::mc [string totitle ${k}:]]
+		set userinfo::userinfo(presence_$i,$jid) $v
+		incr i
+	    }
+	}
+    }
+}
+
+hook::add userinfo_hook ::ssj::userinfo 90
+
+#############################################################################
+
+proc ::ssj::message_buttons {mw connid jid} {
+    set bbox1 [ButtonBox $mw.bottom.buttons1 -spacing 0]
+
+    set b [$bbox1 add \
+		  -image [signed:icon] \
+		  -helptype balloon \
+		  -helptext [::msgcat::mc "Toggle signing"] \
+		  -height 24 \
+		  -width 24 \
+		  -relief link \
+		  -bd $::tk_borderwidth \
+		  -command ::ssj::sign:toggleP]
+    signed:trace "$b configure -image \[::ssj::signed:icon\]"
+    
+    # TODO reflect changes of connid
+    set b [$bbox1 add \
+		  -image [encrypted:icon $connid $jid] \
+		  -helptype balloon \
+		  -helptext [::msgcat::mc "Toggle encryption"] \
+		  -height 24 \
+		  -width 24 \
+		  -relief link \
+		  -bd $::tk_borderwidth \
+		  -command [list ::ssj::encrypt:toggleP $connid $jid]]
+    encrypted:trace \
+	"$b configure -image \[::ssj::encrypted:icon [list $connid] [list $jid]\]" \
+	$connid $jid
+
+    pack $bbox1 -side left -fill x -padx 2m -pady 2m
+}
+
+hook::add open_message_post_hook ::ssj::message_buttons
+
+#############################################################################
+
+proc ::ssj::process_x_signed {acc f x connid from type replyP} {
+    if {!$replyP || [cequal $type error]} {
+	return $acc
+    }
+
+    lassign $acc row body
+
+    foreach xa $x {
+	jlib::wrapper:splitxml $xa tag vars isempty chdata children
+	set xmlns [jlib::wrapper:getattr $vars xmlns]
+
+	if {$xmlns != $::NS(signed)} continue
+
+	# in case the sender didn't check the exit code from gpg...
+	if {[cequal $chdata ""]} {
+	    return $acc
+	}
+
+	set lb [join [lrange [split $f .] 0 end-1] .].title.signed
+	if {[winfo exists $lb]} {
+	    destroy $lb
+	}
+
+	signed:Label $lb \
+	    [signed:input $connid $from $chdata $body \
+		 [::msgcat::mc "Message body"]]
+	grid $lb -row 1 -column 2 -sticky e
+    }
+
+    return $acc
+}
+
+hook::add message_process_x_hook ::ssj::process_x_signed 20
+
+#############################################################################
+
+proc ::ssj::process_x_encrypted {acc f x connid from type replyP} {
+    if {!$replyP || [cequal $type error]} {
+	return $acc
+    }
+
+    lassign $acc row body
+
+    foreach xa $x {
+	jlib::wrapper:splitxml $xa tag vars isempty chdata children
+	set xmlns [jlib::wrapper:getattr $vars xmlns]
+
+	if {$xmlns != $::NS(encrypted)} continue
+
+	# we already deciphered it in client:message...
+	set lb [join [lrange [split $f .] 0 end-1] .].title.encrypted
+	if {[winfo exists $lb]} {
+	    destroy $lb
+	}
+
+	Label $lb -image gpg/encrypted
+	grid $lb -row 1 -column 3 -sticky e
+    }
+
+    return $acc
+}
+
+hook::add message_process_x_hook ::ssj::process_x_encrypted 21
+
+#############################################################################
+
+proc ::ssj::signed:icon {} {
+    return [lindex [list toolbar/gpg-unsigned toolbar/gpg-signed] \
+                   [signP]]
+}
+
+proc ::ssj::encrypted:icon {{connid ""} {jid ""}} {
+    return [lindex [list toolbar/gpg-unencrypted toolbar/gpg-encrypted] \
+                   [encryptP $connid $jid]]
+}
+
+#############################################################################
+
+proc ::ssj::draw_encrypted {chatid from type body x} {
+    # we already deciphered it in client:message...
+
+    set chatw [chat::chat_win $chatid]
+
+    foreach xe $x {
+	jlib::wrapper:splitxml $xe tag vars isempty chdata children
+
+	if {![cequal [jlib::wrapper:getattr $vars xmlns] $::NS(encrypted)]} {
+	    continue
+	}
+
+	$chatw image create end -image gpg/encrypted
+	break
+    }
+}
+
+hook::add draw_message_hook ::ssj::draw_encrypted 7
+
+#############################################################################
+
+proc ::ssj::draw_signed {chatid from type body x} {
+    variable signedid
+
+    set chatw [chat::chat_win $chatid]
+
+    foreach xe $x {
+        jlib::wrapper:splitxml $xe tag vars isempty chdata children
+
+	# in case the sender didn't check the exit code from gpg...
+        if {[cequal $chdata ""] || \
+		![cequal [jlib::wrapper:getattr $vars xmlns] $::NS(signed)]} {
+            continue
+        }
+
+        incr signedid
+	set connid [chat::get_connid $chatid]
+        catch {
+            $chatw window create end \
+                  -window [signed:Label $chatw.signed$signedid \
+                              [signed:input $connid $from $chdata $body \
+                                  [::msgcat::mc "Message body"]]]
+        }
+    }
+}
+
+hook::add draw_message_hook ::ssj::draw_signed 6
+
+###############################################################################
+
+proc ::ssj::chat_window_button {chatid type} {
+    set connid [chat::get_connid $chatid]
+    set jid [chat::get_jid $chatid]
+    set cw [chat::winid $chatid]
+
+    if {[cequal [set sjid [roster::find_jid $connid $jid]] ""]} {
+        set sjid $jid
+    }
+
+    Button $cw.status.encrypted \
+	   -relief flat \
+           -image [encrypted:icon $connid $sjid] \
+           -helptype balloon \
+           -helptext [::msgcat::mc "Toggle encryption"] \
+           -command [list ::ssj::encrypt:toggleP $connid $sjid]
+
+    encrypted:trace "$cw.status.encrypted configure \
+		-image \[::ssj::encrypted:icon $connid $sjid\]" \
+        $connid $sjid
+    pack $cw.status.encrypted -side left -before $cw.status.mb
+}
+
+hook::add open_chat_post_hook ::ssj::chat_window_button
+
+###############################################################################
+
+proc ::ssj::toolbar {} {
+    set mf .mainframe
+    set toolbar [$mf gettoolbar 0]
+    set bbox $toolbar.bbox
+
+    ifacetk::add_toolbar_button $bbox \
+	[signed:icon] \
+	::ssj::sign:toggleP \
+	[::msgcat::mc "Toggle signing"]
+    signed:trace \
+	[list ifacetk::set_toolbar_icon $bbox [$bbox index end] \
+	      ::ssj::signed:icon]
+
+    ifacetk::add_toolbar_button $bbox \
+	[encrypted:icon] \
+	::ssj::encrypt:toggleP \
+	[::msgcat::mc "Toggle encryption (when possible)"]
+    encrypted:trace \
+	[list ifacetk::set_toolbar_icon $bbox [$bbox index end] \
+	      ::ssj::encrypted:icon]
+}
+
+hook::add finload_hook ::ssj::toolbar
+
+###############################################################################
+
+proc ::ssj::add_user_popup_info {infovar connid jid} {
+    variable signed
+
+    upvar 0 $infovar info
+
+    if {[info exists signed($connid,$jid)]} {
+	set signed_info [signed:info $signed($connid,$jid)]
+	append info [::msgcat::mc "\n\tPresence is signed:"]
+	regsub -all {(\n)} "\n$signed_info" "\\1\t    " extra
+	append info $extra
+    }
+}
+
+hook::add roster_user_popup_info_hook ::ssj::add_user_popup_info 99
+
+###############################################################################
+
