@@ -1,4 +1,4 @@
-# $Id: hooks.tcl,v 1.6 2004/07/10 20:05:04 aleksey Exp $
+# $Id: hooks.tcl,v 1.7 2004/09/17 20:30:19 aleksey Exp $
 
 namespace eval hook {
 }
@@ -52,3 +52,32 @@ proc hook::run {hook args} {
 	}
     }
 }
+
+proc hook::foldl {hook acc0 args} {
+    variable F
+    variable $hook
+
+    if {![info exists $hook]} {
+	return $acc0
+    }
+
+    set F(flags,$hook) {}
+
+    set acc $acc0
+    foreach func [set $hook] {
+	set func [lindex $func 0]
+	set code [catch { eval $func [list $acc] $args } state]
+        debugmsg hook "$hook: [lindex $func 0] -> $state (code $code)"
+	if {$code} {
+	    if {![winfo exists .hook_err]} {
+		MessageDlg .hook_err -aspect 50000 -icon error \
+		    -message "Hook $hook failed: $code\n$::errorInfo" \
+		    -type user -buttons ok -default 0 -cancel 0
+	    }
+	} else {
+	    set acc $state
+	}
+    }
+    return $acc
+}
+
