@@ -1,4 +1,4 @@
-# $Id: bwidget_workarounds.tcl,v 1.5 2005/03/27 20:31:12 aleksey Exp $
+# $Id: bwidget_workarounds.tcl,v 1.7 2006/04/27 20:29:35 aleksey Exp $
 
 Button .fake_button
 destroy .fakebutton
@@ -56,52 +56,9 @@ if {[info tclversion] >= 8.4} {
     }
 }
 
-proc ::TreeHandleSelection {win nodes} {
-    if {![cequal [selection own] $win] && ![cequal $nodes {}]} {
-	selection own -command [list $win selection clear] $win
-    }
-}
-
-proc ::TreeSelection {win offset len} {
-    global tcl_platform
-
-    set selnodes [$win selection get]
-    if {![cequal $selnodes {}]} {
-	set ::xselection [$win itemcget [lindex $selnodes 0] -text]
-	if {[cequal $tcl_platform(platform) unix]} {
-	    encoding convertto \
-		[crange $::xselection $offset [expr $offset + $len - 1]]
-	} else {
-	    return [crange $::xselection $offset [expr $offset + $len - 1]]
-	}
-    } else {
-	set ::xselection ""
-    }
-}
-
-proc ::TreeUTF8Selection {win offset len} {
-    return -code error
-}
-
 Tree .fake_tree
 destroy .fake_tree
 
-rename Tree::create Tree::create:old
-
-proc Tree::create {path args} {
-    if {[catch { eval [list Tree::create:old $path] $args \
-		     -selectcommand TreeHandleSelection }]} {
-	eval [list Tree::create:old $path] $args
-    } else {
-	::selection handle $path "TreeSelection $path"
-	::selection handle -type UTF8_STRING $path "TreeUTF8Selection $path"
-    }
-    Tree::bindText  $path <Control-Button-1> [list $path selection set]
-    Tree::bindImage $path <Control-Button-1> [list $path selection set]
-
-    return $path
-}
-
 proc Tree::_see {path idn {side "none"}} {
     set bbox [$path.c bbox $idn]
     set scrl [$path.c cget -scrollregion]
@@ -143,6 +100,17 @@ proc Tree::_see {path idn {side "none"}} {
     }
 }
 
+rename Tree::create Tree::create:old
+
+proc Tree::create {path args} {
+    eval [list Tree::create:old $path] $args
+
+    Tree::bindText  $path <Control-Button-1> [list $path selection set]
+    Tree::bindImage $path <Control-Button-1> [list $path selection set]
+
+    return $path
+}
+
 proc PanedWin {path args} {
    
     if {[catch { PanedWindow .fake_pw -activator button }]} {
@@ -218,3 +186,6 @@ if {($::tcl_platform(platform) != "unix") || ($::aquaP)} {
 	return $path
     }
 }
+
+proc BWidget::bindMouseWheel {widget} {}
+
