@@ -1,4 +1,4 @@
-# $Id: idefault.tcl,v 1.4 2005/03/27 20:31:12 aleksey Exp $
+# $Id: idefault.tcl,v 1.5 2006/01/28 20:45:08 aleksey Exp $
 
 switch -exact -- $::tcl_platform(platform) {
     "unix" {
@@ -58,9 +58,6 @@ hook::add postload_hook [namespace current]::define_fonts 70
 if {![info exists usetabbar]} {
     set usetabbar 1
 }
-if {![info exists pixmaps_theme]} {
-    set pixmaps_theme default
-}
 
 if {$::tcl_platform(platform) == "unix"} {
     option add *Entry.font $::font widgetDefault
