@@ -1,4 +1,4 @@
-# $Id: iface.tcl,v 1.36 2005/08/06 22:13:32 aleksey Exp $
+# $Id: iface.tcl,v 1.50 2006/06/12 20:38:16 aleksey Exp $
 
 namespace eval ifacetk {
     variable options
@@ -63,26 +63,14 @@ namespace eval ifacetk {
 		     nothing  [::msgcat::mc "Do nothing"] \
 		     minimize [::msgcat::mc "Minimize"] \
 		     iconize  [::msgcat::mc "Iconize"] \
+		     systray  [::msgcat::mc "Minimize to systray (if systray\
+					     icon is enabled, otherwise do nothing)"] \
 		    ]
 
     custom::defvar mainwindowstate normal \
 	[::msgcat::mc "Stored main window state (normal or zoomed)"] \
 	-type string -group Hidden
 
-    variable toolbaricon
-
-    set toolbaricon(groupchat) [Bitmap::get [pixmap tkabber toolbar-join-conference.gif]]
-    set toolbaricon(adduser)   [Bitmap::get [pixmap tkabber toolbar-add-user.gif]]
-    set toolbaricon(disco)     [Bitmap::get [pixmap tkabber toolbar-disco.gif]]
-    set toolbaricon(online)    [Bitmap::get [pixmap tkabber toolbar-show-online.gif]]
-    set toolbaricon(offline)   [Bitmap::get [pixmap tkabber toolbar-show-offline.gif]]
-
-    # TODO
-    set ::searchicon(case)       [Bitmap::get [pixmap tkabber search_case.gif]]
-    set ::searchicon(exact)      [Bitmap::get [pixmap tkabber search_exact.gif]]
-    set ::searchicon(back)       [Bitmap::get [pixmap tkabber search_bk.gif]]
-    set ::searchicon(forward)    [Bitmap::get [pixmap tkabber search_fw.gif]]
-
     proc isource {file} {
 	set dir [file dirname [info script]]
 	source [file join $dir $file]
@@ -90,6 +78,7 @@ namespace eval ifacetk {
 
     isource ilogin.tcl
     isource iroster.tcl
+    isource systray.tcl
     
     variable focused ""
     variable after_focused_id ""
@@ -129,14 +118,19 @@ hook::add map_window_hook [namespace current]::ifacetk::save_mainwindowstate
 
 option add *errorForeground red widgetDefault
 
-wm protocol . WM_DELETE_WINDOW {
-    hook::run protocol_wm_delete_window_hook
-}
+wm protocol . WM_DELETE_WINDOW \
+   [list [namespace current]::ifacetk::wm_delete_window]
 
-proc ifacetk::closebuttonproc {} { 
+proc ifacetk::wm_delete_window {} {
     variable options
 
-    switch -- $options(closebuttonaction) {
+    hook::run protocol_wm_delete_window_hook $options(closebuttonaction)
+}
+
+proc ifacetk::closebuttonproc {action} { 
+    variable options
+
+    switch -- $action {
         nothing {}
         minimize {
 	    wm iconify .
@@ -243,8 +237,6 @@ proc ifacetk::load_descmenu {} {
 				 -command {set userstatus xa}] \
 			    [list command [::msgcat::mc "Do not disturb"] {} {} {} \
 				 -command {set userstatus dnd}] \
-			    [list command [::msgcat::mc "Invisible"] {} {} {} \
-				 -command {set userstatus invisible}] \
 			    {separator} \
 			    [list command [::msgcat::mc "Change priority..."] {} {} {} \
 				 -command {change_priority_dialog}]]] \
@@ -262,19 +254,15 @@ proc ifacetk::load_descmenu {} {
 			    set connid [jlib::route ""]
 			    userinfo::open [jlib::connection_bare_jid $connid] \
 				-editable 1 -connection $connid}] \
-		  [list cascad [::msgcat::mc "Privacy rules"] {} {} $ifacetk::options(show_tearoffs) \
+		  [list cascad [::msgcat::mc "Privacy rules"] {} privacy $ifacetk::options(show_tearoffs) \
 		       [list \
-			    [list command [::msgcat::mc "Edit invisible list..."] {} {} {} \
+			    [list command [string trim [::msgcat::mc "Edit invisible list "]] {} {} {} \
 				 -command {privacy::edit_special_list invisible}] \
-			    [list command [::msgcat::mc "Edit ignore list..."] {} {} {} \
+			    [list command [string trim [::msgcat::mc "Edit ignore list "]] {} {} {} \
 				 -command {privacy::edit_special_list ignore}] \
-			    [list command [::msgcat::mc "Manually edit rules..."] {} {} {} \
-				 -command {privacy::request_lists}] \
-			    [list checkbutton [::msgcat::mc "Activate lists at startup"] {} {} {} \
-				 -variable privacy::options(activate_at_startup)] \
 			    {separator} \
-			    [list command [::msgcat::mc "Message filters..."] {} {} {} \
-				 -command {filters::open}]]] \
+			    [list command [::msgcat::mc "Manually edit rules"] {} {} {} \
+				 -command {privacy::request_lists}]]] \
 		  {separator} \
 		  [list cascad [::msgcat::mc "View"] {} {} $ifacetk::options(show_tearoffs) \
 		   [list \
@@ -330,15 +318,15 @@ proc ifacetk::load_descmenu {} {
 		  [list command [::msgcat::mc "Quit"] {} {} {} \
 		       -command {quit}] \
 		 ] \
-	     "&[::msgcat::mc Services]" {} services $ifacetk::options(show_tearoffs) \
+	     [::msgcat::mc "&Services"] {} services $ifacetk::options(show_tearoffs) \
 	     [list \
-	      [list command [::msgcat::mc "Add user to roster..."] {} {} {} \
+		  [list command [::msgcat::mc "Add user to roster..."] {} {} {} \
 		       -command {message::send_subscribe_dialog ""}] \
-	      [list command [::msgcat::mc "Add conference to roster..."] {} {} {} \
-		       -command {plugins::conferences::add_conference_dialog}] \
 		  {separator} \
 		  [list command [::msgcat::mc "Send message..."] {} {} {} \
 		       -command {message::send_dialog}] \
+		  [list command [::msgcat::mc "Open chat..."] {} {} {} \
+		       -command {chat::open_chat_dialog}] \
 		  [list command [::msgcat::mc "Join group..."] {} {} {}\
 		       -command {join_group_dialog}] \
 		  [list command [::msgcat::mc "Show user info..."] {} {} {} \
@@ -357,21 +345,17 @@ proc ifacetk::load_descmenu {} {
 		  [list cascad [::msgcat::mc "Admin tools"] {} admin $ifacetk::options(show_tearoffs) \
 		       [list \
 			    [list command [::msgcat::mc "Send broadcast message..."] {} {} {} \
-				 -command {eval {message::send_dialog \
-						     -to "$loginconf(server)/announce/online"}}] \
+				 -command {ifacetk::send_announce_message announce/online}] \
 			    [list command [::msgcat::mc "Send message of the day..."] {} {} {} \
-				 -command {eval {message::send_dialog \
-						     -to "$loginconf(server)/announce/motd"}}] \
+				 -command {ifacetk::send_announce_message announce/motd}] \
 			    [list command [::msgcat::mc "Update message of the day..."] {} {} {} \
-				 -command {eval {message::send_dialog \
-						     -to "$loginconf(server)/announce/motd/update"}}] \
+				 -command {ifacetk::send_announce_message announce/motd/update}] \
 			    [list command [::msgcat::mc "Delete message of the day"] {} {} {} \
-				 -command {eval {message::send_msg \
-						     "$loginconf(server)/announce/motd/delete" -type normal}}] \
+				 -command {ifacetk::send_announce_message announce/motd/delete}] \
 		        {separator} \
                ]] \
 		 ] \
-	     "&[::msgcat::mc Help]" {} {} $ifacetk::options(show_tearoffs) \
+	     [::msgcat::mc "&Help"] {} {} $ifacetk::options(show_tearoffs) \
 	     [list \
 		  [list command [::msgcat::mc "Quick help"] {} {} {} \
 		       -command ifacetk::quick_help_window] \
@@ -402,6 +386,20 @@ proc ifacetk::load_descmenu {} {
 
 ###############################################################################
 
+proc ifacetk::send_announce_message {resource} {
+    if {[lempty [jlib::connections]]} return
+
+    set server [jlib::connection_server [jlib::route ""]]
+
+    if {$resource == "announce/motd/delete"} {
+	message::send_msg "$server/$resource" -type normal
+    } else {
+	message::send_dialog -to "$server/$resource"
+    }
+}
+
+###############################################################################
+
 proc ifacetk::quick_help_window {} {
     help_window [::msgcat::mc "Quick Help"] \
 "\n[::msgcat::mc {Main window:}]
@@ -431,7 +429,7 @@ proc ifacetk::about_window {} {
     help_window [::msgcat::mc "About"] "
 Tkabber $version
 
-Copyright \u00a9 2002-2005 [::msgcat::mc {Alexey Shchepin}]
+Copyright \u00a9 2002-2006 [::msgcat::mc {Alexey Shchepin}]
 \n[::msgcat::mc Authors:]
     [::msgcat::mc {Alexey Shchepin}]
     [::msgcat::mc {Marshall T. Rose}]
@@ -442,8 +440,7 @@ http://tkabber.jabberstudio.org/"
 }
 
 proc ifacetk::help_window {title message} {
-    set mainlogo [Bitmap::get [pixmap tkabber tkabber-logo.gif]]
-    Dialog .m -anchor e -separator yes -title $title -image $mainlogo \
+    Dialog .m -anchor e -separator yes -title $title -image tkabber/logo \
 	    -side bottom -modal local -default 0 -cancel 0
     .m add -text [::msgcat::mc "OK"]
     set frame [.m getframe]
@@ -574,12 +571,10 @@ proc ifacetk::add_toolbar_button {path icon command helptext} {
 }
 
 proc ifacetk::online_icon {args} {
-    variable toolbaricon
-
     if {$roster::show_only_online} {
-	return $toolbaricon(online)
+	return toolbar/show-online
     } else {
-	return $toolbaricon(offline)
+	return toolbar/show-offline
     }
 }
 
@@ -601,7 +596,6 @@ proc ifacetk::create_main_window {} {
     variable rw
     variable nw
     variable ssl_ind
-    variable toolbaricon
     variable main_window_title
     variable options
 
@@ -629,11 +623,11 @@ proc ifacetk::create_main_window {} {
 
     set bbox [ButtonBox [$mf addtoolbar].bbox -spacing 0 -padx 1 -pady 1]
 
-    add_toolbar_button $bbox $toolbaricon(adduser) {message::send_subscribe_dialog ""} \
+    add_toolbar_button $bbox toolbar/add-user {message::send_subscribe_dialog ""} \
 	[::msgcat::mc "Add new user..."]
-    add_toolbar_button $bbox $toolbaricon(disco) {disco::browser::open_win ""} \
+    add_toolbar_button $bbox toolbar/disco {disco::browser::open_win ""} \
 	[::msgcat::mc "Service Discovery"]
-    add_toolbar_button $bbox $toolbaricon(groupchat) join_group_dialog \
+    add_toolbar_button $bbox toolbar/join-conference join_group_dialog \
 	[::msgcat::mc "Join group..."]
 
     add_toolbar_button $bbox [ifacetk::online_icon] \
@@ -643,21 +637,6 @@ proc ifacetk::create_main_window {} {
 	[list [namespace current]::set_toolbar_icon $bbox [$bbox index end] \
 	      [namespace current]::online_icon]
 
-    if {![cequal [info commands ::ssj::sign:toggleP] ""]} {
-	add_toolbar_button $bbox [message::signed:icon] ssj::sign:toggleP \
-	[::msgcat::mc "Toggle signing"]
-	ssj::signed:trace \
-	    [list [namespace current]::set_toolbar_icon $bbox [$bbox index end] \
-		  message::signed:icon]
-    }
-    if {![cequal [info commands ::ssj::encrypt:toggleP] ""]} {
-	add_toolbar_button $bbox [message::encrypted:icon] ssj::encrypt:toggleP \
-	[::msgcat::mc "Toggle encryption (when possible)"]
-	ssj::encrypted:trace \
-	    [list [namespace current]::set_toolbar_icon $bbox [$bbox index end] \
-		  message::encrypted:icon]
-    }
-
     pack $bbox -side left -anchor w
     switch_toolbar
 
@@ -676,8 +655,9 @@ proc ifacetk::create_main_window {} {
 
     frame .presence
     menubutton .presence.button -menu .presence.button.menu -relief $::tk_relief \
-	-textvariable userstatusdesc -direction above -width $ww \
-	-state disabled 
+	-textvariable userstatusdesc -direction above -width $ww
+
+    disallow_presence_change ""
 
     bind [entry .presence.status -textvariable textstatus] \
 	<Return> {set userstatus $userstatus}
@@ -695,9 +675,14 @@ proc ifacetk::create_main_window {} {
     }
     menu $m -tearoff $ifacetk::options(show_tearoffs)
     foreach {status str} $user_status_list {
-	if {![cequal $status unavailable]} {
-	    $m add command -label $str \
-	    -command [list set userstatus $status]
+	switch -- $status {
+	    invisible -
+	    unavailable {}
+	    default {
+		$m add command \
+		       -label $str \
+		       -command [list set userstatus $status]
+	    }
 	}
     }
 
@@ -717,7 +702,9 @@ proc ifacetk::create_main_window {} {
 
 	roster::create .roster -width $rosterwidth -height 300 \
 	    -popup [namespace current]::roster::popup_menu \
-	    -grouppopup [namespace current]::roster::group_popup_menu
+	    -grouppopup [namespace current]::roster::group_popup_menu \
+	    -singleclick [namespace current]::roster::user_singleclick \
+	    -doubleclick [namespace current]::roster::jid_doubleclick
 	pack .roster -expand yes -fill both -side left -in $rw
 
 	if {$options(tabs_at_bottom)} {
@@ -762,6 +749,7 @@ proc ifacetk::create_main_window {} {
 		    ifacetk::tab_move .nb 0
 		}
 	    }
+	$m add separator
 	$m add command -label [::msgcat::mc "Close other tabs"] \
 	    -command {
 		foreach tab [.nb pages] {
@@ -780,7 +768,9 @@ proc ifacetk::create_main_window {} {
 		}
 	    }
 
-	.nb bindtabs <3> {ifacetk::tab_menu %X %Y}
+	.nb bindtabs <3> [list [namespace current]::tab_menu %X %Y]
+	.nb bindtabs <4> [list [namespace current]::tab_move .nb -1]
+	.nb bindtabs <5> [list [namespace current]::tab_move .nb 1]
 
 	DragSite::register .nb.c -draginitcmd [namespace current]::draginitcmd
 	DropSite::register .nb.c -dropovercmd [namespace current]::dropovercmd \
@@ -793,7 +783,9 @@ proc ifacetk::create_main_window {} {
     } else {
 	roster::create .roster -width $rosterwidth -height 300 \
 	    -popup [namespace current]::roster::popup_menu \
-	    -grouppopup [namespace current]::roster::group_popup_menu
+	    -grouppopup [namespace current]::roster::group_popup_menu \
+	    -singleclick [namespace current]::roster::user_singleclick \
+	    -doubleclick [namespace current]::roster::jid_doubleclick
 	pack .roster -expand yes -fill both -side left -in [$mf getframe]
 	set geometry [option get . geometry [winfo class .]]
 	if {$geometry == ""} {
@@ -878,11 +870,18 @@ hook::add finload_hook [namespace current]::ifacetk::create_main_window 1
 
 proc ifacetk::allow_presence_change {connid} {
     .presence.button configure -state normal
+
+    set m [.mainframe getmenu tkabber]
+    $m entryconfigure [$m index [::msgcat::mc "Presence"]] -state normal
+
 }
 
 proc ifacetk::disallow_presence_change {connid} {
     if {[llength [jlib::connections]] == 0} {
 	.presence.button configure -state disabled
+
+	set m [.mainframe getmenu tkabber]
+	$m entryconfigure [$m index [::msgcat::mc "Presence"]] -state disabled
     }
 }
 
@@ -1151,7 +1150,7 @@ proc ifacetk::loose_focus {path} {
     balloon::destroy
 }
 
-proc ifacetk::tab_move {nb shift} {
+proc ifacetk::tab_move {nb shift args} {
     set len [llength [$nb pages]]
 
     if {$len > 0} {
@@ -1320,9 +1319,14 @@ proc ifacetk::switch_presencebar {args} {
 	    catch {
 		pack .presence -side bottom -anchor w \
 		    -in [.mainframe getframe] -fill x -before $before
+		.presence.status configure -takefocus 1
 	    }
 	} else {
 	    pack forget .presence
+	    .presence.status configure -takefocus 0
+	    if {[focus] == ".presence.status"} {
+	    focus [Widget::focusPrev .presence.status]
+	    }
 	}
     }
 }
