@@ -1,13 +1,14 @@
-# $Id: ilogin.tcl,v 1.12 2005/06/22 02:56:27 aleksey Exp $
+# $Id: ilogin.tcl,v 1.16 2006/03/28 19:12:17 aleksey Exp $
 
 proc update_login_entries {l {i 0}} {
     global ltmp
 
     if {$i} {
         array set ltmp [array get ::loginconf$i]
+	array set loginconf [array get ltmp]
     }
     foreach ent {username server port password resource priority \
-            altserver sslport httpproxy httpproxyport httplogin httppassword \
+            altserver httpproxy httpproxyport httplogin httppassword \
             sslcertfile pollurl} {
         if {[winfo exists $l.$ent]} {
             catch { $l.$ent icursor end }
@@ -15,12 +16,11 @@ proc update_login_entries {l {i 0}} {
     }
     foreach {check enable disable} { \
             usehttppoll {lpollurl pollurl usepollkeys} \
-			{dontusessl legacyssl starttls sslport lsslport \
+			{dontusessl usecompress legacyssl encrypted \
 			 sslcertfile lsslcertfile bsslcertfile} \
-            usealtserver {altserver laltserver} {} \
+            usealtserver {altserver laltserver port lport} {} \
             useproxy {httpproxy httpproxyport httplogin httppassword \
             lhttpproxy lhttpproxyport lhttplogin lhttppassword} {} \
-            usesasl {allowsaslplain} {} \
 	    } {
         if {![info exists ltmp($check)] || ![winfo exists $l.$check]} {
             continue
@@ -51,18 +51,9 @@ proc update_login_entries {l {i 0}} {
 	}
     }
     catch {
-	if {$ltmp(usessl) == 1 && \
-		![cequal [$l.starttls cget -state] disabled]} {
-	    $l.sslport configure -state normal
-	    $l.lsslport configure -state normal
-	} else {
-	    $l.sslport configure -state disabled
-	    $l.lsslport configure -state disabled
-	    if {[cequal [focus] $l.sslport]} {
-		focus [Widget::focusPrev $l.sslport]
-	    }
-	}
-	if {$ltmp(usessl) != 0} {
+	if {![cequal [$l.dontusessl cget -state] disabled] && \
+		($ltmp(stream_options) == "ssl" || \
+		 $ltmp(stream_options) == "encrypted")} {
 	    $l.sslcertfile configure -state normal
 	    $l.lsslcertfile configure -state normal
 	    $l.bsslcertfile configure -state normal
@@ -81,7 +72,7 @@ proc update_login_entries {l {i 0}} {
 proc login_dialog {} {
     global loginconf
     global ltmp
-    global use_tls have_sasl have_http_poll have_proxy
+    global use_tls have_compress have_sasl have_http_poll have_proxy
 
     if {[winfo exists .login]} {
 	focus -force .login
@@ -136,8 +127,6 @@ proc login_dialog {} {
     entry $l.server -textvariable ltmp(server)
     label $l.lpassword -text [::msgcat::mc "Password:"]
     entry $l.password -show * -textvariable ltmp(password)
-    checkbutton $l.usedigest -text [::msgcat::mc "Use hashed password"] \
-	-variable ltmp(usedigest)
     label $l.lresource -text [::msgcat::mc "Resource:"]
     entry $l.resource -textvariable ltmp(resource)
     label $l.lpriority -text [::msgcat::mc "Priority:"]
@@ -149,7 +138,6 @@ proc login_dialog {} {
     grid $l.server    -row 0 -column 3 -sticky ew -in $account_page
     grid $l.lpassword -row 1 -column 0 -sticky e -in $account_page
     grid $l.password  -row 1 -column 1 -sticky ew -in $account_page
-    grid $l.usedigest -row 1 -column 3 -sticky w -in $account_page
     grid $l.lresource -row 2 -column 0 -sticky e -in $account_page
     grid $l.resource  -row 2 -column 1 -sticky ew -in $account_page
     grid $l.lpriority -row 2 -column 2 -sticky e -in $account_page
@@ -161,62 +149,120 @@ proc login_dialog {} {
 
     set connection_page [$nb insert end connection_page -text [::msgcat::mc "Connection"]]
 
-    label $l.lport -text [::msgcat::mc "Server Port:"]
-    Spinbox $l.port 0 65535 1 ltmp(port)
-    checkbutton $l.usealtserver -text [::msgcat::mc "Connect via alternate server"] \
+    checkbutton $l.usealtserver -text [::msgcat::mc "Explicitly specify host and port to connect"] \
 	-variable ltmp(usealtserver) \
 	-command [list [namespace current]::update_login_entries $l]
-    label $l.laltserver -text [::msgcat::mc "Server:"]
+    label $l.laltserver -text [::msgcat::mc "Host:"]
     entry $l.altserver -textvariable ltmp(altserver)
+    label $l.lport -text [::msgcat::mc "Port:"]
+    Spinbox $l.port 0 65535 1 ltmp(altport)
 
-    grid $l.lport     -row 0 -column 0 -sticky e -in $connection_page
-    grid $l.port      -row 0 -column 1 -sticky w -in $connection_page
-    grid $l.usealtserver -row 1 -column 0 -sticky w -columnspan 2 -in $connection_page
-    grid $l.laltserver -row 1 -column 2 -sticky e -in $connection_page
-    grid $l.altserver -row 1 -column 3 -sticky ew -in $connection_page
+    grid $l.usealtserver -row 0 -column 0 -sticky w -columnspan 4 -in $connection_page
+    grid $l.laltserver -row 1 -column 0 -sticky e -in $connection_page
+    grid $l.altserver -row 1 -column 1 -sticky ew -in $connection_page
+    grid $l.lport     -row 1 -column 2 -sticky e -in $connection_page
+    grid $l.port      -row 1 -column 3 -sticky we -in $connection_page
 
     checkbutton $l.replace -text [::msgcat::mc "Replace opened connections"] \
 	-variable ltmp(replace_opened)
     grid $l.replace   -row 3 -column 0 -sticky w -columnspan 3 -in $connection_page
 
-    grid columnconfigure $connection_page 1 -weight 1
-    grid columnconfigure $connection_page 3 -weight 2
+    grid columnconfigure $connection_page 1 -weight 6
+    grid columnconfigure $connection_page 2 -weight 1
 
-    if {$use_tls} {
-	set ssl_page [$nb insert end ssl_page -text [::msgcat::mc "SSL"]]
 
-	radiobutton $l.dontusessl -text [::msgcat::mc "Don't use SSL"] \
-	    -variable ltmp(usessl) -value 0 \
+    set auth_page [$nb insert end auth_page -text [::msgcat::mc "Authentication"]]
+
+    checkbutton $l.allowauthplain \
+	-text [::msgcat::mc "Allow plaintext authentication mechanisms"] \
+	-variable ltmp(allowauthplain) \
+	-command [list [namespace current]::update_login_entries $l]
+
+    grid $l.allowauthplain -row 0 -column 0 -sticky w -in $auth_page
+
+    if {$have_sasl} {
+	checkbutton $l.usesasl \
+	    -text [::msgcat::mc "Use SASL authentication"] \
+	    -variable ltmp(usesasl) \
 	    -command [list [namespace current]::update_login_entries $l]
-	radiobutton $l.legacyssl -text [::msgcat::mc "Use legacy SSL"] \
-	    -variable ltmp(usessl) -value 1 \
-	    -command [list [namespace current]::update_login_entries $l]
-	radiobutton $l.starttls -text [::msgcat::mc "Use STARTTLS"] \
-	    -variable ltmp(usessl) -value 2 \
-	    -command [list [namespace current]::update_login_entries $l]
-	label $l.lsslport -text [::msgcat::mc "SSL Port:"]
-	Spinbox $l.sslport 0 65535 1 ltmp(sslport)
-	label $l.lsslcertfile -text [::msgcat::mc "SSL Certificate:"]
-	entry $l.sslcertfile -textvariable ltmp(sslcertfile)
-	button $l.bsslcertfile -text [::msgcat::mc "Browse..."] \
-	    -command [list eval set ltmp(sslcertfile) {[tk_getOpenFile]}]
 
-
-	grid $l.dontusessl   -row 0 -column 0 -sticky w -in $ssl_page
-	grid $l.legacyssl    -row 0 -column 1 -sticky w -in $ssl_page
-	grid $l.starttls     -row 0 -column 2 -sticky w -in $ssl_page
-	grid $l.lsslport     -row 1 -column 0 -sticky e -in $ssl_page
-	grid $l.sslport      -row 1 -column 1 -sticky w -in $ssl_page
-	grid $l.lsslcertfile -row 2 -column 0 -sticky e -in $ssl_page
-	grid $l.sslcertfile  -row 2 -column 1 -sticky ew -columnspan 2 -in $ssl_page
-	grid $l.bsslcertfile -row 2 -column 3 -sticky w -in $ssl_page
-
-	grid columnconfigure $ssl_page 0 -weight 1
-	grid columnconfigure $ssl_page 1 -weight 6
-	grid columnconfigure $ssl_page 2 -weight 6
-	grid columnconfigure $ssl_page 3 -weight 1
+	grid $l.usesasl -row 1 -column 0 -sticky w -in $auth_page
     }
 
+    grid columnconfigure $auth_page 0 -weight 1
+
+    if {$use_tls || $have_compress} {
+	if {$use_tls && $have_compress} {
+		set page_label [::msgcat::mc "SSL & Compression"]
+	} elseif {$have_compress} {
+	    set page_label [::msgcat::mc "Compression"]
+	} else {
+	    set page_label [::msgcat::mc "SSL"]
+	}
+
+	set ssl_page [$nb insert end ssl_page -text $page_label]
+
+	radiobutton $l.dontusessl -text [::msgcat::mc "Plaintext"] \
+	    -variable ltmp(stream_options) -value plaintext \
+	    -command [list [namespace current]::update_login_entries $l]
+	if {$have_compress} {
+	    radiobutton $l.usecompress -text [::msgcat::mc "Compression"] \
+		-variable ltmp(stream_options) -value compressed \
+		-command [list [namespace current]::update_login_entries $l]
+	}
+	if {$use_tls} {
+	    radiobutton $l.encrypted -text [::msgcat::mc "Encryption (STARTTLS)"] \
+		-variable ltmp(stream_options) -value encrypted \
+		-command [list [namespace current]::update_login_entries $l]
+	    radiobutton $l.legacyssl -text [::msgcat::mc "Encryption (legacy SSL)"] \
+		-variable ltmp(stream_options) -value ssl \
+		-command [list [namespace current]::update_login_entries $l]
+	    label $l.lsslcertfile -text [::msgcat::mc "SSL Certificate:"]
+	    entry $l.sslcertfile -textvariable ltmp(sslcertfile)
+	    button $l.bsslcertfile -text [::msgcat::mc "Browse..."] \
+		-command [list eval set ltmp(sslcertfile) {[tk_getOpenFile]}]
+	}
+
+	set column 0
+	grid $l.dontusessl -row 0 -column $column -sticky w -in $ssl_page
+	
+	if {$have_compress} {
+	    grid $l.usecompress -row 0 -column [incr column] -sticky w -in $ssl_page
+	}
+	if {$use_tls} {
+	    grid $l.encrypted     -row 0 -column [incr column] -sticky w -in $ssl_page
+	    grid $l.legacyssl    -row 0 -column [incr column] -sticky w -in $ssl_page
+
+	    grid $l.lsslcertfile -row 1 -column 0 -sticky e -in $ssl_page
+	    grid $l.sslcertfile  -row 1 -column 1 -sticky ew -columnspan 2 -in $ssl_page
+	    grid $l.bsslcertfile -row 1 -column 3 -sticky w -in $ssl_page
+	}
+
+	grid columnconfigure $ssl_page 1 -weight 1
+	grid columnconfigure $ssl_page 2 -weight 1
+    }
+
+    if {$have_http_poll} {
+	set httppoll_page [$nb insert end httpoll_page -text [::msgcat::mc "HTTP Poll"]]
+
+	checkbutton $l.usehttppoll -text [::msgcat::mc "Connect via HTTP polling"] \
+	    -variable ltmp(usehttppoll) \
+	    -command [list [namespace current]::update_login_entries $l]
+	label $l.lpollurl -text [::msgcat::mc "URL to poll:"]
+	entry $l.pollurl -textvariable ltmp(pollurl)
+	checkbutton $l.usepollkeys -text [::msgcat::mc "Use client security keys"] \
+	    -state disabled \
+	    -variable ltmp(usepollkeys) \
+	    -command [list [namespace current]::update_login_entries $l]
+    
+	grid $l.usehttppoll -row 0 -column 0 -sticky w -columnspan 3 -in $httppoll_page
+	grid $l.lpollurl -row 1 -column 0 -sticky e -in $httppoll_page
+	grid $l.pollurl -row 1 -column 1 -sticky ew -in $httppoll_page
+	grid $l.usepollkeys -row 2 -column 0 -sticky w -columnspan 3 -in $httppoll_page
+
+	grid columnconfigure $httppoll_page 1 -weight 1
+    }
+    
     if {$have_proxy} {
 	set proxy_page [$nb insert end proxy_page -text [::msgcat::mc "Proxy"]]
 
@@ -244,60 +290,12 @@ proc login_dialog {} {
 	grid $l.httplogin     -row 2 -column 1 -sticky ew -in $proxy_page
 	grid $l.lhttppassword -row 2 -column 2 -sticky e -in $proxy_page
 	grid $l.httppassword  -row 2 -column 3 -sticky ew -in $proxy_page
+
+	grid columnconfigure $proxy_page 1 -weight 3
+	grid columnconfigure $proxy_page 2 -weight 1
+	grid columnconfigure $proxy_page 3 -weight 3
     }
 
-    if {$have_http_poll} {
-	set httppoll_page [$nb insert end httpoll_page -text [::msgcat::mc "HTTP Poll"]]
-
-	checkbutton $l.usehttppoll -text [::msgcat::mc "Connect via HTTP polling"] \
-	    -variable ltmp(usehttppoll) \
-	    -command [list [namespace current]::update_login_entries $l]
-	label $l.lpollurl -text [::msgcat::mc "URL to poll:"]
-	entry $l.pollurl -textvariable ltmp(pollurl)
-	checkbutton $l.usepollkeys -text [::msgcat::mc "Use client security keys"] \
-	    -state disabled \
-	    -variable ltmp(usepollkeys) \
-	    -command [list [namespace current]::update_login_entries $l]
-    
-	grid $l.usehttppoll -row 0 -column 0 -sticky w -columnspan 3 -in $httppoll_page
-	grid $l.lpollurl -row 1 -column 0 -sticky e -in $httppoll_page
-	grid $l.pollurl -row 1 -column 1 -sticky ew -in $httppoll_page
-	grid $l.usepollkeys -row 2 -column 0 -sticky w -columnspan 3 -in $httppoll_page
-
-	grid columnconfigure $httppoll_page 1 -weight 1
-    }
-    
-    if {$have_sasl} {
-	set sasl_page [$nb insert end sasl_page -text [::msgcat::mc "SASL"]]
-
-	checkbutton $l.usesasl \
-	    -text [::msgcat::mc "Use SASL authentication"] \
-	    -variable ltmp(usesasl) \
-	    -command [list [namespace current]::update_login_entries $l]
-	checkbutton $l.allowsaslplain \
-	    -text [::msgcat::mc "Allow plaintext SASL mechanisms"] \
-	    -variable ltmp(allowsaslplain) \
-	    -command [list [namespace current]::update_login_entries $l]
-	#label $l.lsaslport -text [::msgcat::mc "SASL Port:"]
-	#Spinbox $l.saslport 0 65535 1 ltmp(saslport)
-	#label $l.lsaslcertfile -text [::msgcat::mc "SASL Certificate:"] -state disabled
-	#entry $l.saslcertfile -textvariable ltmp(saslcertfile) -state disabled
-	#button $l.bsaslcertfile -text [::msgcat::mc "Browse..."] -state disabled \
-	#    -command [list eval set ltmp(saslcertfile) {[tk_getOpenFile]}]
-
-
-	grid $l.usesasl        -row 0 -column 0 -sticky w -columnspan 3 -in $sasl_page
-	grid $l.allowsaslplain -row 1 -column 0 -sticky w -columnspan 3 -in $sasl_page
-	#grid $l.lsaslport     -row 1 -column 0 -sticky e -in $sasl_page
-	#grid $l.saslport      -row 1 -column 1 -sticky w -in $sasl_page
-	#grid $l.lsaslcertfile -row 2 -column 0 -sticky e -in $sasl_page
-	#grid $l.saslcertfile  -row 2 -column 1 -sticky ew -in $sasl_page
-	#grid $l.bsaslcertfile -row 2 -column 2 -sticky w -in $sasl_page
-
-	grid columnconfigure $sasl_page 0 -weight 1
-	grid columnconfigure $sasl_page 1 -weight 6
-	grid columnconfigure $sasl_page 2 -weight 1
-    }
 
     $nb compute_size
     $nb raise account_page
