@@ -1,13 +1,8 @@
-# $Id: itemedit.tcl,v 1.13 2005/08/14 16:25:27 aleksey Exp $
+# $Id: itemedit.tcl,v 1.15 2006/04/09 19:45:37 aleksey Exp $
 
 namespace eval itemedit {}
 
 proc itemedit::show_dialog {connid jid} {
-    variable gra_name
-    variable gra_nick
-    variable gra_password
-    variable gra_autojoin
-
     set allowed_name [jid_to_tag $jid]
     set w .gredit_${connid}_$allowed_name
 
@@ -21,70 +16,7 @@ proc itemedit::show_dialog {connid jid} {
 
     set f [$w getframe]
 
-    if {[roster::itemconfig $connid $jid -subsc] == "bookmark"} {
-	set tf [TitleFrame $f.prop -text [format [::msgcat::mc "Edit properties for %s"] $jid]]
-	pack $tf -side top -expand yes -fill both
-	set g [$tf getframe]
-
-	set gra_name $::plugins::conferences::bookmarks($connid,name,$jid)
-	set gra_autojoin $::plugins::conferences::bookmarks($connid,autojoin,$jid)
-	if {[info exists ::plugins::conferences::bookmarks($connid,nick,$jid)]} {
-	    set gra_nick $::plugins::conferences::bookmarks($connid,nick,$jid)
-	} else {
-	    set gra_nick ""
-	}
-	if {[info exists ::plugins::conferences::bookmarks($connid,password,$jid)]} {
-	    set gra_password $::plugins::conferences::bookmarks($connid,password,$jid)
-	} else {
-	    set gra_password ""
-	}
-
-	label $g.lname -text [string trim [::msgcat::mc "Name: "]]
-	entry $g.name -textvariable [namespace current]::gra_name
-	label $g.lnick -text [::msgcat::mc "Nick:"]
-	entry $g.nick -textvariable [namespace current]::gra_nick
-	label $g.lpassword -text [::msgcat::mc "Password:"]
-	entry $g.password -show * -textvariable [namespace current]::gra_password
-	checkbutton $g.autojoin -text [::msgcat::mc "Automatically join conference upon connect"] \
-	    -variable [namespace current]::gra_autojoin
-	grid columnconfigure $g 0 -weight 0
-	grid columnconfigure $g 1 -weight 1
-
-	grid $g.lname  -row 0 -column 0 -sticky e
-	grid $g.name   -row 0 -column 1 -sticky ew
-	grid $g.lnick -row 1 -column 0 -sticky e
-	grid $g.nick  -row 1 -column 1 -sticky ew
-	grid $g.lpassword -row 2 -column 0 -sticky e
-	grid $g.password  -row 2 -column 1 -sticky ew
-	grid $g.autojoin -row 3 -column 0 -sticky w -columnspan 2
-    } else {
-	set tf [TitleFrame $f.name -text [format [::msgcat::mc "Edit nickname for %s"] $jid]]
-	pack $tf -side top -expand yes -fill both
-	set g [$tf getframe]
-
-	label $g.lname -text [::msgcat::mc "Nickname:"]
-	set gn [entry $g.name]
-
-	set name [roster::itemconfig $connid $jid -name]
-	if {$name == ""} {
-	    if {[info exists userinfo::userinfo(nickname,$jid)] && \
-		    ![cequal $userinfo::userinfo(nickname,$jid) ""]} {
-		set name $userinfo::userinfo(nickname,$jid)
-	    } else {
-		set name [node_from_jid $jid]
-		jlib::send_iq get \
-		    [jlib::wrapper:createtag vCard \
-			 -vars [list xmlns vcard-temp]] \
-		    -to [node_and_server_from_jid [get_jid_of_user $connid $jid]] \
-		    -connection $connid \
-		    -command [list [namespace current]::fetch_nickname $gn $name $jid]
-	    }
-	}
-	$g.name insert 0 $name
-
-	pack $g.lname -side left
-	pack $g.name -side left -expand yes -fill x
-    }
+    hook::run roster_itemedit_setup_hook $f $connid $jid
 
     set g [[TitleFrame $f.gr -text [format [::msgcat::mc "Edit groups for %s"] $jid]] getframe]
     pack $f.gr -side top -expand yes -fill both
@@ -107,7 +39,6 @@ proc itemedit::show_dialog {connid jid} {
     pack $gasw -side top -expand yes -fill both
 
 
-
     TitleFrame $g.current -text [::msgcat::mc "Current groups"]
     pack $g.current -side right -expand yes -fill both
     set gc [$g.current getframe]
@@ -126,13 +57,8 @@ proc itemedit::show_dialog {connid jid} {
     pack $g.buttons -side left
 
 
-    if {[roster::itemconfig $connid $jid -subsc] == "bookmark"} {
-	$w add -text [::msgcat::mc "OK"] \
-	    -command [list itemedit::commit_bookmark_changes $gcl $connid $jid]
-    } else {
-	$w add -text [::msgcat::mc "OK"] \
-	    -command [list itemedit::commit_changes $gn $gcl $connid $jid]
-    }
+    $w add -text [::msgcat::mc "OK"] \
+	-command [list [namespace current]::commit_changes $gcl $connid $jid]
     $w add -text [::msgcat::mc "Cancel"] -command [list destroy $w]
 
 
@@ -153,18 +79,55 @@ proc itemedit::show_dialog {connid jid} {
     $w draw
 }
 
-proc itemedit::prefs_user_menu {m connid jid} {
-    global curuser
+proc itemedit::edit_item_setup_fallback {f connid jid} {
+    variable gra_name
 
-    if {[cequal $jid "\$curuser"]} {
-	set jid $curuser
+    set tf [TitleFrame $f.name -text [format [::msgcat::mc "Edit nickname for %s"] $jid]]
+    pack $tf -side top -expand yes -fill both
+    set g [$tf getframe]
+
+    label $g.lname -text [::msgcat::mc "Nickname:"]
+    set gn [entry $g.name -textvariable [namespace current]::gra_name]
+
+    set name [roster::itemconfig $connid $jid -name]
+    if {$name == ""} {
+        if {[info exists userinfo::userinfo(nickname,$jid)] && \
+		![cequal $userinfo::userinfo(nickname,$jid) ""]} {
+	   set name $userinfo::userinfo(nickname,$jid)
+        } else {
+	    set name [node_from_jid $jid]
+	    jlib::send_iq get \
+		[jlib::wrapper:createtag vCard \
+		     -vars [list xmlns vcard-temp]] \
+		-to [node_and_server_from_jid [get_jid_of_user $connid $jid]] \
+		-connection $connid \
+		-command [list [namespace current]::fetch_nickname $gn $name $jid]
+	}
     }
-    $m add command -label [::msgcat::mc "Edit item..."] \
-	-command [list [namespace current]::show_dialog $connid $jid]
+    $g.name delete 0 end
+    $g.name insert 0 $name
+
+    pack $g.lname -side left
+    pack $g.name -side left -expand yes -fill x
 }
 
-hook::add roster_create_user_menu_edit_hook \
-    [namespace current]::itemedit::prefs_user_menu 30
+hook::add roster_itemedit_setup_hook \
+    [namespace current]::itemedit::edit_item_setup_fallback 100
+
+proc itemedit::prefs_user_menu {m connid jid} {
+    set rjid [roster::find_jid $connid $jid]
+    $m add command -label [::msgcat::mc "Edit item..."] \
+	-command [list [namespace current]::show_dialog $connid $rjid]
+}
+
+hook::add chat_create_user_menu_hook \
+    [namespace current]::itemedit::prefs_user_menu 74
+hook::add roster_conference_popup_menu_hook \
+    [namespace current]::itemedit::prefs_user_menu 74
+hook::add roster_service_popup_menu_hook \
+    [namespace current]::itemedit::prefs_user_menu 74
+hook::add roster_jid_popup_menu_hook \
+    [namespace current]::itemedit::prefs_user_menu 74
 
 proc itemedit::add_group {grlist group} {
     set group [string trim $group]
@@ -196,14 +159,24 @@ proc itemedit::remove_current_group {grlist} {
 }
 
 
-proc itemedit::commit_changes {name_entry grlist connid jid} {
-    roster::itemconfig $connid $jid \
-	-group [$grlist get 0 end] \
-	-name [$name_entry get]
-    roster::send_item $connid $jid
+proc itemedit::commit_changes {grlist connid jid} {
+    hook::run roster_itemedit_commit_hook $connid $jid [$grlist get 0 end]
+
     destroy [winfo toplevel $grlist]
 }
 
+proc itemedit::commit_changes_fallback {connid jid groups} {
+    variable gra_name
+
+    roster::itemconfig $connid $jid \
+	-name $gra_name \
+	-group $groups
+    roster::send_item $connid $jid
+}
+
+hook::add roster_itemedit_commit_hook \
+    [namespace current]::itemedit::commit_changes_fallback 100
+
 proc itemedit::fetch_nickname {name_entry name jid res child} {
     if {![winfo exists $name_entry] || ![cequal $res OK]} {
 	return
@@ -224,16 +197,3 @@ proc itemedit::fetch_nickname {name_entry name jid res child} {
 
 }
 
-proc itemedit::commit_bookmark_changes {grlist connid jid} {
-    variable gra_name
-    variable gra_nick
-    variable gra_password
-    variable gra_autojoin
-
-    plugins::conferences::update_bookmark $connid $jid \
-	-name $gra_name -nick $gra_nick -password $gra_password \
-	-autojoin $gra_autojoin -groups [$grlist get 0 end]
-
-    destroy [winfo toplevel $grlist]
-}
-
