@@ -1,4 +1,4 @@
-# $Id: transports.tcl,v 1.6 2005/07/14 22:32:36 aleksey Exp $
+# $Id: transports.tcl,v 1.10 2006/01/24 22:52:37 aleksey Exp $
 
 namespace eval transport {
     variable capabilities [list tcp http_poll]
@@ -260,13 +260,14 @@ proc transport::tcp::outmsg {connid msg} {
 
     if {![info exists lib($connid,socket)]} {
 	::LOG "error ([namespace current]::outmsg)\
-Cannot write to socket: socket for connection $connid doesn't exist"
+	       Cannot write to socket: socket for\
+	       connection $connid doesn't exist"
 	return -2
     }
 
     if {[catch { puts -nonewline $lib($connid,socket) $msg }]} {
 	::LOG "error ([namespace current]::outmsg)\
-Cannot write to socket: $lib($connid,socket)"
+	       Cannot write to socket: $lib($connid,socket)"
 	return -2
     }
 }
@@ -302,6 +303,129 @@ proc transport::tcp::inmsg {connid sock} {
     jlib::inmsg $connid $msg [eof $sock]
 }
 
+######################################################################
+# TODO Cleanup
+proc transport::tcp::to_compress {connid method} {
+    variable lib
+
+    set [namespace parent]::${method}::lib($connid,socket) \
+	$lib($connid,socket)
+    eval [list [namespace parent]::${method}::import $connid]
+    set ::jlib::lib($connid,transport) $method
+
+    array unset lib $connid,*
+}
+
+proc transport::tcp::to_tls {connid args} {
+    variable lib
+
+    set [namespace parent]::tls::lib($connid,socket) $lib($connid,socket)
+    eval [list [namespace parent]::tls::tls_import $connid] $args
+    set ::jlib::lib($connid,transport) tls
+
+    array unset lib $connid,*
+}
+
+
+######################################################################
+#
+# Zlib Compressed Socket Support
+#
+######################################################################
+
+if {![catch { package require zlib 1.0 }]} {
+    lappend transport::capabilities compress
+}
+
+namespace eval transport::zlib {}
+
+proc transport::zlib::connect {connid server port args} {
+    variable lib
+
+    set proxytype none
+    foreach {opt val} $args {
+	switch -- $opt {
+	    -proxytype { set proxytype $val }
+	}
+    }
+
+    set sock [eval [list [namespace parent]::proxy::${proxytype}::connect \
+			 $connid $server $port] $args]
+
+    set lib($connid,socket) $sock
+    import $connid
+
+    return $sock
+}
+
+proc transport::zlib::outmsg {connid msg} {
+    variable lib
+
+    if {![info exists lib($connid,socket)]} {
+	::LOG "error ([namespace current]::outmsg)\
+	       Cannot write to socket: socket for connection\
+	       $connid doesn't exist"
+	return -2
+    }
+
+    if {[catch { puts -nonewline $lib($connid,socket) $msg }]} {
+	::LOG "error ([namespace current]::outmsg)\
+	       Cannot write to socket: $lib($connid,socket)"
+	return -2
+    }
+    flush $lib($connid,socket)
+    fconfigure $lib($connid,socket) -flush output
+}
+
+proc transport::zlib::disconnect {connid} {
+    variable lib
+
+    catch {
+	if {[set [namespace parent]::disconnect] == "quick"} {
+	    flush $lib($connid,socket)
+	    fconfigure $lib($connid,socket) -finish output
+	} else {
+	    fconfigure $lib($connid,socket) -blocking 1
+	    flush $lib($connid,socket)
+	    fconfigure $lib($connid,socket) -finish output
+	    vwait [namespace current]::lib($connid,socket)
+	}
+    }
+}
+
+proc transport::zlib::close {connid} {
+    variable lib
+
+    catch { fileevent $lib($connid,socket) readable {} }
+    catch { ::close $lib($connid,socket) }
+
+    array unset lib $connid,*
+}
+
+######################################################################
+proc transport::zlib::inmsg {connid sock} {
+    set msg ""
+    catch {
+	fconfigure $sock -flush input
+	set msg [read $sock]
+    }
+
+    jlib::inmsg $connid $msg [eof $sock]
+}
+
+######################################################################
+proc transport::zlib::import {connid args} {
+    variable lib
+
+    set sock $lib($connid,socket)
+    fconfigure $sock -blocking 0 -buffering none \
+	       -translation auto -encoding utf-8
+    zlib stream $lib($connid,socket) RDWR -output compress \
+	 -input decompress
+
+    fileevent $sock readable \
+	      [list [namespace current]::inmsg $connid $sock]
+}
 
 ######################################################################
 #
@@ -323,8 +447,16 @@ proc transport::tls::connect {connid server port args} {
     foreach {opt val} $args {
 	switch -- $opt {
 	    -proxytype { set proxytype $val }
+	    -cacertstore {
+		if {$val != ""} {
+		    if {[file isdirectory $val]} {
+			lappend tlsargs -cadir $val
+		    } else {
+			lappend tlsargs -cafile $val
+		    }
+		}
+	    }
 	    -certfile  -
-	    -cafile    -
 	    -keyfile   {
 		if {$val != ""} {
 		    lappend tlsargs $opt $val
@@ -349,13 +481,14 @@ proc transport::tls::outmsg {connid msg} {
 
     if {![info exists lib($connid,socket)]} {
 	::LOG "error ([namespace current]::outmsg)\
-Cannot write to socket: socket for connection $connid doesn't exist"
+	       Cannot write to socket: socket for connection\
+	       $connid doesn't exist"
 	return -2
     }
 
     if {[catch { puts -nonewline $lib($connid,socket) $msg }]} {
 	::LOG "error ([namespace current]::outmsg)\
-Cannot write to socket: $lib($connid,socket)"
+	       Cannot write to socket: $lib($connid,socket)"
 	return -2
     }
 }
@@ -427,6 +560,19 @@ proc transport::tls::tls_import {connid args} {
 	      [list [namespace current]::inmsg $connid $sock]
 }
 
+######################################################################
+# TODO Cleanup
+proc transport::tls::to_compress {connid method} {
+    variable lib
+
+    set [namespace parent]::${method}::lib($connid,socket) \
+	$lib($connid,socket)
+    eval [list [namespace parent]::${method}::import $connid]
+    set ::jlib::lib($connid,transport) $method
+
+    array unset lib $connid,*
+}
+
 ######################################################################
 #
 # HTTP Polling
