@@ -1,4 +1,4 @@
-# $Id: login.tcl,v 1.77 2005/08/19 20:13:19 aleksey Exp $
+# $Id: login.tcl,v 1.85 2006/05/11 22:07:25 aleksey Exp $
 
 
 if {[lcontain [jlib::capabilities transport] tls]} {
@@ -7,6 +7,12 @@ if {[lcontain [jlib::capabilities transport] tls]} {
     set use_tls 0
 }
 
+if {[lcontain [jlib::capabilities transport] compress]} {
+    set have_compress 1
+} else {
+    set have_compress 0
+}
+
 if {[lcontain [jlib::capabilities auth] sasl]} {
     set have_sasl 1
 } else {
@@ -40,43 +46,56 @@ custom::defgroup Login \
 custom::defvar loginconf(user) "" \
     [::msgcat::mc "User name."] \
     -group Login -type string
-custom::defvar loginconf(password) "" \
-    [::msgcat::mc "Password."] \
-    -group Login -type password
-custom::defvar loginconf(usedigest) 1 \
-    [::msgcat::mc "Use hashed password transmission."] \
-    -group Login -type boolean
-custom::defvar loginconf(resource) "tkabber" \
-    [::msgcat::mc "Resource."] \
-    -group Login -type string
 custom::defvar loginconf(server) "localhost" \
     [::msgcat::mc "Server name."] \
     -group Login -type string
-custom::defvar loginconf(port) "5222" \
-    [::msgcat::mc "Server port."] \
-    -group Login -type integer
+custom::defvar loginconf(password) "" \
+    [::msgcat::mc "Password."] \
+    -group Login -type password
+custom::defvar loginconf(resource) "tkabber" \
+    [::msgcat::mc "Resource."] \
+    -group Login -type string
 custom::defvar loginconf(priority) "8" \
     [::msgcat::mc "Priority."] \
     -group Login -type integer
+
 custom::defvar loginconf(connect_forever) 0 \
     [::msgcat::mc "Retry to connect forever."] \
     -group Login -type boolean
 
+custom::defvar loginconf(allowauthplain) 0 \
+    [::msgcat::mc "Allow plaintext authentication mechanisms (when password\
+		   is transmitted unencrypted)."] \
+	-group Login -type boolean
+
+if {$have_sasl} {
+    custom::defvar loginconf(usesasl) 1 \
+	[::msgcat::mc "Use SASL authentication."] \
+	-group Login -type boolean
+}
+
+set values [list plaintext [::msgcat::mc "Plaintext"]]
+if {$have_compress} {
+    lappend values compressed [::msgcat::mc "Compression"]
+}
 if {$use_tls} {
-    custom::defvar loginconf(usessl) 0 \
-	[::msgcat::mc "SSL options when connecting to server."] \
+    lappend values encrypted [::msgcat::mc "Encryption (STARTTLS)"] \
+		   ssl       [::msgcat::mc "Encryption (legacy SSL)"]
+}
+
+if {$use_tls || $have_compress} {
+    custom::defvar loginconf(stream_options) plaintext \
+	[::msgcat::mc "XMPP stream options when connecting to server."] \
 	-group Login -type radio -layout horizontal \
-	-values [list 0 [::msgcat::mc "Don't use SSL"] \
-		      1 [::msgcat::mc "Use legacy SSL"] \
-		      2 [::msgcat::mc "Use STARTTLS"]]
-    custom::defvar loginconf(sslport) "5223" \
-	[::msgcat::mc "SSL port."] \
-	-group Login -type integer
+	-values $values
+}
+
+if {$use_tls} {
     custom::defvar loginconf(sslcertfile) "" \
 	[::msgcat::mc "SSL certificate file (optional)."] \
 	-group Login -type file
-    custom::defvar loginconf(sslcafile) "" \
-	[::msgcat::mc "SSL CA file (optional)."] \
+    custom::defvar loginconf(sslcacertstore) "" \
+	[::msgcat::mc "SSL certification authority file or directory (optional)."] \
 	-group Login -type file
     custom::defvar loginconf(sslkeyfile) "" \
 	[::msgcat::mc "SSL private key file (optional)."] \
@@ -106,11 +125,15 @@ $::tcl_platform(os) $::tcl_platform(osVersion))" \
 }
 
 custom::defvar loginconf(usealtserver) 0 \
-    [::msgcat::mc "Use explicitly-specified server address."] \
+    [::msgcat::mc "Use explicitly-specified server address and port."] \
     -group Login -type boolean
 custom::defvar loginconf(altserver) "" \
     [::msgcat::mc "Server name or IP-address."] \
     -group Login -type string
+custom::defvar loginconf(altport) "5222" \
+    [::msgcat::mc "Server port."] \
+    -group Login -type integer
+
 custom::defvar loginconf(replace_opened) 1 \
     [::msgcat::mc "Replace opened connections."] \
     -group Login -type boolean
@@ -119,7 +142,7 @@ if {$have_http_poll} {
     custom::defvar loginconf(usehttppoll) 0 \
 	[::msgcat::mc "Use HTTP poll connection method."] \
 	-group Login -type boolean
-    custom::defvar loginconf(pollurl) "http://connect.jabber.cz/" \
+    custom::defvar loginconf(pollurl) "" \
 	[::msgcat::mc "URL to connect to."] \
 	-group Login -type string
     custom::defvar loginconf(usepollkeys) 1 \
@@ -141,15 +164,6 @@ to zero, Tkabber will wait forever)."] \
 	-group Login -type integer
 }
 
-if {$have_sasl} {
-    custom::defvar loginconf(usesasl) 1 \
-	[::msgcat::mc "Use SASL authentication."] \
-	-group Login -type boolean
-    custom::defvar loginconf(allowsaslplain) 0 \
-	[::msgcat::mc "Allow plaintext SASL authentication mechanisms."] \
-	-group Login -type boolean
-}
-
 custom::defvar reasonlist {} [::msgcat::mc "List of logout reasons."] \
 	-group Hidden
 
@@ -265,7 +279,7 @@ proc client:tls_callback {connid args} {
 }
 
 proc login_connect {logindata} {
-    global use_tls have_sasl have_http_poll have_proxy
+    global use_tls have_compress have_sasl have_http_poll have_proxy
     global tls_warning_info
 
     array set lc $logindata
@@ -280,11 +294,12 @@ proc login_connect {logindata} {
 
     set tls_warning_info($connid) ""
 
-    set args {}
+    set args [list -password $lc(password) \
+		   -allowauthplain $lc(allowauthplain)]
     if {$have_sasl} {
-	lappend args -usesasl $lc(usesasl) \
-		     -allowsaslplain $lc(allowsaslplain)
+	lappend args -usesasl $lc(usesasl)
     }
+
     if {$have_proxy && $lc(useproxy)} {
 	lappend args -proxytype http
 	lappend args -proxyhost $lc(httpproxy)
@@ -295,33 +310,52 @@ proc login_connect {logindata} {
     }
 
     if {$have_http_poll && $lc(usehttppoll)} {
+	if {$lc(pollurl) != ""} {
+	    set url $lc(pollurl)
+	} else {
+	    set url [jlibdns::get_http_poll_url $lc(server)]
+	}
 	return [eval [list jlib::connect $connid \
 				-transport http_poll \
 				-polltimeout $lc(polltimeout) \
 				-pollint $lc(pollmin) \
 				-pollmin $lc(pollmin) \
 				-pollmax $lc(pollmax) \
-				-pollurl $lc(pollurl) \
+				-pollurl $url \
 				-pollusekeys $lc(usepollkeys) \
 				-pollnumkeys $lc(numberofpollkeys)] $args]
     } else {
+	if {$have_compress && $lc(stream_options) == "compressed"} {
+	    lappend args -usecompression 1
+	}
+   
 	if {$lc(usealtserver)} {
-	    set host $lc(altserver)
+	    set hosts {}
 	} else {
-	    set host $lc(server)
+	    set hosts [jlibdns::get_addr_port $lc(server)]
+	    if {[lempty $hosts]} {
+		set hosts [list [list $lc(server) 5222]]
+	    }
 	}
-
 	set transport tcp
-	set port $lc(port)
 	set sslopts {}
 	if {$use_tls} {
-	    switch -- $lc(usessl) {
-		1 {
+	    switch -- $lc(stream_options) {
+		ssl {
 		    set usestarttls 0
 		    set transport tls
-		    set port $lc(sslport)
+		    # Do some heuristic.
+		    # Traditionally legacy SSL port is 5223,
+		    # so let's add 1 to all ports from SRV reply
+		    set hosts1 {}
+		    foreach hp $hosts {
+			lappend hosts1 \
+				[list [lindex $hp 0] \
+				      [expr {[lindex $hp 1] + 1}]]
+		    }
+		    set hosts $hosts1
 		}
-		2 {
+		encrypted {
 		    set usestarttls 1
 		}
 		default {
@@ -330,81 +364,39 @@ proc login_connect {logindata} {
 	    }
 	    set sslopts [list -usestarttls $usestarttls \
 	                      -certfile $lc(sslcertfile) \
-			      -cafile $lc(sslcafile) \
+			      -cacertstore $lc(sslcacertstore) \
 			      -keyfile $lc(sslkeyfile)]
 	}
 
+	if {$lc(usealtserver)} {
+	    set hosts [list [list $lc(altserver) $lc(altport)]]
+	}
+
 	return [eval [list jlib::connect $connid \
 				-transport $transport \
-				-host $host \
-				-port $port] \
+				-hosts $hosts] \
 				$sslopts $args]
     }
 }
 
+########################################################################
+
 proc login_login {logindata connid} {
-    global loginconf_hist_$connid
+    global loginconf_hist
     global gr_nick gr_server gra_server
 
     array set lc $logindata
-    array set loginconf_hist_$connid $logindata
+    set loginconf_hist($connid) $logindata
 
     set gr_nick $lc(user)
     set gr_server conference.$lc(server)
     set gra_server conference.$lc(server)
 
-    jlib::wait_for_stream $connid
-
-    if {$lc(usedigest)} {
-	set autht digest
-    } else {
-	set autht plain
-    }
-
-    lassign [jlib::get_authtypes $lc(user) $connid] res data
-
-    switch -- $res {
-	ERR {
-	    clear_status
-	    recv_auth_result $connid $logindata ERR $data
-	    return
-	}
-	OK {
-	    set authtype ""
-	    foreach at [list $autht digest plain] {
-		if {[lcontain $data $at]} {
-		    set authtype $at
-		    break
-		}
-	    }
-
-	    if {[cequal $authtype ""]} {
-		MessageDlg .auth_err -aspect 50000 -icon error \
-		    -message [::msgcat::mc "Can't authenticate: Remote server\
-doesn't support\nplain or digest authentication method"] \
-		    -type user -buttons ok -default 0 -cancel 0
-		return
-	    }
-	    if {[cequal $authtype plain] && [cequal $autht digest]} {
-		set res [MessageDlg .auth_err -aspect 50000 -icon warning \
-		    -message [::msgcat::mc "Warning: Remote server doesn't\
-support\nhashed password authentication.\n\nProceed with PLAINTEXT authentication?"] \
-		    -type user -buttons {yes no} -default 0 -cancel 1]
-		if {$res} {
-		    return
-		}
-	    }
-	}
-	SASL {
-	    set authtype ""
-	}
-    }
-
-    jlib::send_auth $connid \
-	$lc(user) $lc(password) $lc(resource) \
-	[list recv_auth_result $connid $logindata] $authtype
+    jlib::login $connid [list recv_auth_result $connid $logindata]
 }
 
+########################################################################
+
 set reconnect_retries 0
 
 proc logout {{connid {}}} {
@@ -438,7 +430,7 @@ proc client:disconnect {connid} {
 # TODO
 proc client:reconnect {connid} {
     global reconnect_retries
-    global loginconf_hist_$connid
+    global loginconf_hist
 
     debugmsg login "RECONNECT $connid"
     roster::clean_connection $connid
@@ -449,11 +441,11 @@ proc client:reconnect {connid} {
 
     disconnected $connid
     if {[incr reconnect_retries] <= 3} {
-        after 1000 [list login [array get loginconf_hist_$connid]]
+        after 1000 [list login $loginconf_hist($connid)]
     }
 }
 
-proc connected {logindata connid} {
+proc connected {connid logindata} {
     hook::run connected_hook $connid
 }
 
@@ -470,11 +462,14 @@ proc disconnected {connid} {
 }
 
 proc recv_auth_result {connid logindata res args} {
-    upvar #0 loginconf_hist_$connid lc
+    array set lc $logindata
 
     if {$res == "OK"} {
-	connected $logindata $connid
+	connected $connid $logindata
     } else {
+	if {[winfo exists .auth_err]} {
+	    destroy .auth_err
+	}
 	lassign [error_type_condition [lindex $args 0]] type cond
 	if {($type == "sasl") || ($type == "auth" && $cond == "not-authorized")} {
 	    set res [MessageDlg .auth_err -aspect 50000 -icon error \
@@ -490,7 +485,7 @@ proc recv_auth_result {connid logindata res args} {
 					     -chdata $lc(user)] \
 					[jlib::wrapper:createtag password \
 					     -chdata $lc(password)]]] \
-		    -command [list recv_register_result [array get lc]]
+		    -command [list recv_register_result $connid $logindata]
 		return
 	    }
 	} else {
@@ -504,11 +499,14 @@ proc recv_auth_result {connid logindata res args} {
     }
 }
 
-proc recv_register_result {logindata res args} {
+proc recv_register_result {connid logindata res args} {
     if {$res == "OK"} {
 	jlib::disconnect
 	login $logindata
     } else {
+	if {[winfo exists .auth_err]} {
+	    destroy .auth_err
+	}
 	MessageDlg .auth_err -aspect 50000 -icon error \
 	    -message [format [::msgcat::mc "Registration failed: %s"] \
 			  [error_to_string [lindex $args 0]]] \
