@@ -1,4 +1,4 @@
-# $Id: plugins.tcl,v 1.8 2005/03/03 01:55:50 aleksey Exp $
+# $Id: plugins.tcl,v 1.10 2006/02/25 19:57:36 aleksey Exp $
 
 namespace eval plugins {}
 
@@ -22,10 +22,14 @@ proc plugins::load {dir args} {
 }
 
 proc plugins::load_dir {plugins_dir} {
-    foreach dir [lsort [glob -nocomplain [file join $plugins_dir *]]] {
+    foreach dir [lsort [glob -nocomplain -type d [file join $plugins_dir *]]] {
 	set file [file join $dir [file tail $dir].tcl]
-	debugmsg plugins "Loading plugin from $file"
-	source $file
+	if {[file exists $file]} {
+	    debugmsg plugins "Loading plugin from $file"
+	    source $file
+	} else {
+	    debugmsg plugins "Can't load plugin from $file"
+	}
     }
 }
 
