@@ -1,4 +1,4 @@
-# $Id: completion.tcl,v 1.8 2005/03/12 22:03:04 aleksey Exp $
+# $Id: completion.tcl,v 1.9 2006/01/17 00:47:14 aleksey Exp $
 
 namespace eval completion {
     set options(prefix) ""
@@ -240,6 +240,14 @@ proc completion::delete_suffix {chatid} {
     set completion(state,$chatid) normal
 }
 
+proc completion::on_keypress {chatid key} {
+    variable completion
+
+    if {$key != ""} {
+	set completion(state,$chatid) normal
+    }
+}
+
 proc completion::setup_bindings {chatid type} {
     variable history
 
@@ -254,11 +262,10 @@ proc completion::setup_bindings {chatid type} {
     bind $cc <Key-Tab> \
 	[list [namespace current]::complete [double% $chatid]]
     bind $cc <Key-Tab> +break
-    bind $cc <Key-Return> [list [namespace current]::delete_suffix [double% $chatid]]
+    bind $cc <Key-Return> \
+	[list [namespace current]::delete_suffix [double% $chatid]]
     bind $cc <KeyPress> \
-	[list set \
-	     [double% [namespace current]::completion(state,$chatid)] \
-	     normal]
+	[list [namespace current]::on_keypress [double% $chatid] %A]
 }
 
 hook::add open_chat_post_hook [namespace current]::completion::setup_bindings
