@@ -1,6 +1,16 @@
-# $Id: events.tcl,v 1.7 2005/03/12 22:03:04 aleksey Exp $
+# $Id: events.tcl,v 1.8 2005/10/11 23:28:29 aleksey Exp $
+#
+# Message Events (JEP-0022) support.
+#
 
-namespace eval events {}
+namespace eval events {
+    custom::defgroup Events \
+	[::msgcat::mc "Chat message events plugin options."] \
+	-group Chat
+    custom::defvar options(enable) 1 \
+	[::msgcat::mc "Enable sending chat message events."] \
+	-type boolean -group Events
+}
 
 proc events::process_x {check_request chatid from type body xs} {
     variable chats
@@ -10,12 +20,11 @@ proc events::process_x {check_request chatid from type body xs} {
 	foreach x $xs {
 	    jlib::wrapper:splitxml $x tag vars isempty chdata children
 	    set xmlns [jlib::wrapper:getattr $vars xmlns]
-	    switch -- $xmlns {
-		jabber:x:event {
+	    switch -- $xmlns \
+		$::NS(event) {
 		    return [process_x_event \
 				$check_request $chatid $from $type $body $x]
 		}
-	    }
 	}
     }
 }
@@ -74,6 +83,8 @@ proc events::process_x_event {check_request chatid from type body x} {
 	set events(displayed,$chatid) $displayed
 	set events(composing,$chatid) $composing
 
+	if {!$options(enable)} return
+
 	lappend eventtags [jlib::wrapper:createtag id \
 			       -chdata $::chat::chats(id,$chatid)]
 	if {$delivered} {
@@ -93,7 +104,7 @@ proc events::process_x_event {check_request chatid from type body x} {
 	}
 	if {[llength $eventtags] > 1} {
 	    lappend xlist [jlib::wrapper:createtag x \
-			       -vars [list xmlns jabber:x:event] \
+			       -vars [list xmlns $::NS(event)] \
 			       -subtags $eventtags]
 
 	    jlib::send_msg $from -xlist $xlist -connection $connid
@@ -146,6 +157,7 @@ proc events::clear_status {chatid} {
 }
 
 proc events::send_event_on_raise {cw chatid} {
+    variable options
     variable events
 
     if {$::chat::chats(type,$chatid) != "chat"} return
@@ -157,12 +169,14 @@ proc events::send_event_on_raise {cw chatid} {
 
     set events(displayed,$chatid) 0
 
+    if {!$options(enable)} return
+
     lappend eventtags [jlib::wrapper:createtag id \
 			   -chdata $::chat::chats(id,$chatid)]
     lappend eventtags [jlib::wrapper:createtag displayed]
 
     lappend xlist [jlib::wrapper:createtag x \
-		       -vars [list xmlns jabber:x:event] \
+		       -vars [list xmlns $::NS(event)] \
 		       -subtags $eventtags]
 
     jlib::send_msg $jid -xlist $xlist -connection $connid
@@ -171,6 +185,7 @@ proc events::send_event_on_raise {cw chatid} {
 hook::add raise_chat_tab_hook [namespace current]::events::send_event_on_raise
 
 proc events::event_composing {chatid sym} {
+    variable options
     variable events
 
     if {$::chat::chats(type,$chatid) != "chat"} return
@@ -183,13 +198,14 @@ proc events::event_composing {chatid sym} {
 
     set events(composing,$chatid) 0
 
+    if {!$options(enable)} return
 
     lappend eventtags [jlib::wrapper:createtag id \
 			   -chdata $::chat::chats(id,$chatid)]
     lappend eventtags [jlib::wrapper:createtag composing]
 
     lappend xlist [jlib::wrapper:createtag x \
-		       -vars [list xmlns jabber:x:event] \
+		       -vars [list xmlns $::NS(event)] \
 		       -subtags $eventtags]
 
     jlib::send_msg $jid -xlist $xlist -connection $connid
@@ -203,8 +219,11 @@ proc events::setup_ui {chatid type} {
 
     set cw [chat::winid $chatid]
 
-    label $cw.status.event -font $font
-    pack $cw.status.event -side left
+    set l $cw.status.event
+    if {![winfo exists $l]} {
+	label $l -font $font
+	pack $l -side left
+    }
 
     set iw [chat::input_win $chatid]
     bind $iw <Key> +[list [namespace current]::event_composing \
@@ -221,3 +240,23 @@ proc events::clear_status_on_send {chatid user body type} {
 hook::add chat_send_message_hook \
     [namespace current]::events::clear_status_on_send
 
+proc events::make_xlist {xlist chatid user body type} {
+    variable options
+
+    if {!$options(enable) || $type != "chat"} {
+	return $xlist
+    }
+
+    lappend events [jlib::wrapper:createtag offline]
+    lappend events [jlib::wrapper:createtag delivered]
+    lappend events [jlib::wrapper:createtag displayed]
+    lappend events [jlib::wrapper:createtag composing]
+    lappend xlist [jlib::wrapper:createtag x \
+		       -vars [list xmlns $::NS(event)] \
+		       -subtags $events]
+    return $xlist
+}
+
+hook::add chat_send_message_xlist_hook \
+    [namespace current]::events::make_xlist
+
