@@ -1,4 +1,4 @@
-# $Id: highlight.tcl,v 1.3 2004/09/05 20:16:10 aleksey Exp $
+# $Id: highlight.tcl,v 1.4 2005/09/15 20:19:25 aleksey Exp $
 
 namespace eval highlight {
     custom::defgroup Highlight [::msgcat::mc "Groupchat message highlighting plugin options."] \
@@ -44,6 +44,7 @@ proc highlight::check_message {nick body} {
     set ind_end 0
     set stop_ind [string length $body]
     set matches {}
+    set highlight 0
     set found 1
     while {$found && $ind_end < $stop_ind} {
 	set found 0
@@ -66,17 +67,18 @@ proc highlight::check_message {nick body} {
 	    }
 	}
 	if {$found} {
+	    set highlight 1
 	    set i 0
 	    while {[lindex $wordborders $i] <= $ind_start} {
 		incr i
 	    }
 	    set word [lindex $words $i]
 	    
-	    if {![regexp -- $::chat::url_regexp $word]} {
+	    if {![regexp -expanded -nocase -- $::chat::url_regexp $word]} {
 		lappend matches $ind_start $ind_end
 	    }
 	}
     }
-    return [list [expr {[llength $matches] > 0}] $matches]
+    return [list $highlight $matches]
 }
 
