@@ -1,4 +1,4 @@
-# $Id: irc_commands.tcl,v 1.9 2005/03/12 22:03:04 aleksey Exp $
+# $Id: irc_commands.tcl,v 1.14 2006/03/23 21:05:01 aleksey Exp $
 
 namespace eval irc {}
 
@@ -11,32 +11,31 @@ proc irc::handle_irc_commands {chatid user body type} {
     } elseif {[cequal [crange $body 0 8] "/subject "]} {
 	set command subject
 	set subject [string trim [crange $body 9 end]]
-    } elseif {[cequal $body "/topic"] || [cequal $body "/subject"]} {
+    } elseif {[cequal $body "/topic"] || \
+	      [cequal $body "/subject"]} {
 	set command show_subject
     } elseif {[cequal [crange $body 0 5] "/nick "]} {
 	set command nick
 	set nick [string trim [crange $body 6 end]]
     } elseif {[cequal [crange $body 0 7] "/invite "]} {
 	set command invite
-	set arg [string trim [crange $body 8 end]]
+	set arg [crange $body 8 end]
+	lassign [parse_body $arg "\n"] to reason
     } elseif {[cequal [crange $body 0 5] "/join "]} {
 	set command join
-	set room [string trim [crange $body 6 end]]
+	set arg [crange $body 6 end]
+	lassign [parse_body $arg " "] room password
     } elseif {[cequal $body "/join"]} {
 	set command join
 	set room ""
+	set password ""
     } elseif {[cequal [crange $body 0 4] "/msg "]} {
 	set command msg
-	set arg [string trim [crange $body 5 end]]
-	set ind [string first " " $arg]
-	if {$ind < 0} {
-	    return stop
-	} else {
-	    set to [crange $arg 0 [expr {$ind - 1}]]
-	    set text [string trim [crange $arg $ind end]]
-	}
-    } elseif {[cequal $body "/nick"] || [cequal $body "/msg"] || \
-		  [cequal $body "/invite"]} {
+	set arg [crange $body 5 end]
+	lassign [parse_body $arg "\n"] nick body
+    } elseif {[cequal $body "/nick"] || \
+	      [cequal $body "/msg"] || \
+	      [cequal $body "/invite"]} {
 	return stop
     } elseif {[cequal [crange $body 0 5] "/part "]} {
 	set command leave
@@ -44,11 +43,10 @@ proc irc::handle_irc_commands {chatid user body type} {
     } elseif {[cequal [crange $body 0 6] "/leave "]} {
 	set command leave
 	set status [string trim [crange $body 7 end]]
-    } elseif {[cequal $body "/part"] || [cequal $body "/leave"]} {
+    } elseif {[cequal $body "/part"] || \
+	      [cequal $body "/leave"]} {
 	set command leave
 	set status ""
-    } elseif {[cequal $body "/clear"]} {
-	set command clear
     } else {
 	return
     }
@@ -59,8 +57,18 @@ proc irc::handle_irc_commands {chatid user body type} {
     switch -- $command {
 	nick {
 	    if {![cequal $type groupchat]} return
-	    muc::change_nick $chatid $nick
-	    debugmsg plugins "NICK: $nick"
+	    if {$chat::chats(status,$chatid) == "disconnected"} {
+		if {[info exists muc::muc_password($chatid)]} {
+		    muc::join_group $connid $jid $nick \
+				    $muc::muc_password($chatid)
+		} else {
+		    muc::join_group $connid $jid $nick
+		}
+		debugmsg plugins "NICK -> JOIN: $nick"
+	    } else {
+		muc::change_nick $chatid $nick
+		debugmsg plugins "NICK: $nick"
+	    }
 	}
 	subject {
 	    if {![cequal $type groupchat]} return
@@ -72,8 +80,13 @@ proc irc::handle_irc_commands {chatid user body type} {
 	    chat::add_message $chatid $jid info \
 		"[::msgcat::mc Subject:] $chat::chats(subject,$chatid)" {}
 	}
-	# TODO
 	invite {
+	    if {[cequal $type groupchat]} {
+		muc::invite_muc $connid $jid $to $reason
+	    } else {
+		muc::invite_muc $connid $to $jid $reason
+	    }
+	    debugmsg plugins "INVITE: $to $reason"
 	}
 	join {
 	    if {[cequal $type groupchat]} {
@@ -83,38 +96,49 @@ proc irc::handle_irc_commands {chatid user body type} {
 		    set room $room@[server_from_jid $jid]
 		}
 	    }
-	    join_group $room \
-		-nick [get_group_nick $room [get_our_groupchat_nick $chatid]] \
-		-connection $connid
+	    muc::join_group $connid \
+			    $room \
+			    [get_our_groupchat_nick $chatid] \
+			    $password
 	    debugmsg plugins "JOIN: $room"
 	}
 	msg {
-	    if {![cequal $type groupchat]} return
-	    chat::open_to_user $connid $jid/$to -message $text
-	    debugmsg plugins "MSG: $to: $text"
+	    if {[cequal $type groupchat] && \
+		    $nick != "" && $body != ""} {
+		chat::open_to_user $connid $jid/$nick -message $body
+		debugmsg plugins "MSG: $jid/$nick: $body"
+	    } else {
+		chat::add_message $chatid $jid error \
+		    "/msg to $nick failed" {}
+	    }
 	}
 	leave {
 	    set chat::chats(exit_status,$chatid) $status
 	    after idle [list ifacetk::destroy_win [chat::winid $chatid]]
 	    debugmsg plugins "LEAVE: $jid $status"
 	}
-	clear {
-	    set chatw [chat::chat_win $chatid]
-	    $chatw configure -state normal
-	    $chatw delete 0.0 end
-	    $chatw configure -state disabled
-	}
     }
     return stop
 }
 
 hook::add chat_send_message_hook [namespace current]::irc::handle_irc_commands 50
 
+proc irc::parse_body {line separator} {
+    set ne [string first $separator $line]
+    if {$ne < 0} {
+	set nick $line
+	set body ""
+    } else {
+	set nick [string range $line 0 [expr {$ne - 1}]]
+	set body [string range $line [expr {$ne + [string length $separator]}] end]
+    }
+    return [list $nick [string trim $body]]
+}
+
 proc irc::irc_commands_comp {chatid compsvar wordstart line} {
     upvar 0 $compsvar comps
 
     if {!$wordstart} {
-	lappend comps {/clear }
 	lappend comps {/invite }
 	lappend comps {/join }
 	lappend comps {/leave }
@@ -124,6 +148,34 @@ proc irc::irc_commands_comp {chatid compsvar wordstart line} {
 	lappend comps {/subject }
 	lappend comps {/topic }
     }
+
+    if {$wordstart && [cequal [crange $line 0 7] "/invite "] && \
+	    [string first "\n" $line] < 0} {
+	set prefix $plugins::completion::options(prefix)
+	set suffix $plugins::completion::options(suffix)
+        set jidcomps {}
+	set connid [chat::get_connid $chatid]
+	if {[chat::is_groupchat $chatid]} {
+	    foreach jid [roster::get_jids $connid] {
+		if {[roster::itemconfig $connid $jid -isuser]} {
+		    lappend jidcomps $prefix$jid$suffix
+		}
+	    }
+	} else {
+	    foreach chatid1 [chat::opened] {
+		if {[chat::get_connid $chatid1] == $connid} {
+		    if {[chat::is_groupchat $chatid1]} {
+			set jid [chat::get_jid $chatid1]
+			lappend jidcomps $prefix$jid$suffix
+		    }
+		}
+	    }
+	}
+        set jidcomps [lsort -dictionary -unique $jidcomps]
+        set comps $jidcomps
+        debugmsg plugins "COMPLETION from roster: $comps"
+	return stop
+    }
 }
 
 hook::add generate_completions_hook [namespace current]::irc::irc_commands_comp
