@@ -1,4 +1,4 @@
-# $Id: logger.tcl,v 1.24 2005/08/07 19:12:54 aleksey Exp $
+# $Id: logger.tcl,v 1.29 2006/05/11 22:30:13 aleksey Exp $
 
 option add *highlightSearchBackground        PaleGreen1    widgetDefaul
 
@@ -33,12 +33,37 @@ namespace eval ::logger {
     }
 }
 
+proc ::logger::add_menu_item {state cathegory m connid jid} {
+    switch -- $cathegory {
+	roster {
+	    set jid [roster::find_jid $connid $jid]
+	}
+	chat {
+	    set nas [node_and_server_from_jid $jid]
+	    if {![chat::is_groupchat [chat::chatid $connid $nas]]} {
+		set jid $nas
+	    }
+	}
+    }
+    $m add command -label [::msgcat::mc "Show history"] \
+	   -state $state \
+           -command [list logger::show_log $jid -connection $connid]
+}
+
+hook::add chat_create_user_menu_hook [list ::logger::add_menu_item normal chat] 65
+hook::add chat_create_conference_menu_hook [list ::logger::add_menu_item normal group] 65
+hook::add roster_create_groupchat_user_menu_hook [list ::logger::add_menu_item normal grouproster] 65
+hook::add roster_conference_popup_menu_hook [list ::logger::add_menu_item normal roster] 65
+hook::add roster_service_popup_menu_hook [list ::logger::add_menu_item disabled roster] 65
+hook::add roster_jid_popup_menu_hook [list ::logger::add_menu_item normal roster] 65
+hook::add message_dialog_menu_hook [list ::logger::add_menu_item disabled message] 65
+hook::add search_popup_menu_hook [list ::logger::add_menu_item disabled search] 65
 
 proc ::logger::log_file {jid} {
     variable options
 
-    regsub -all {[@/|*+?]} $jid _ filename 
-    return ${options(logdir)}/$filename
+    regsub -all {[:\\@/|*+?]} $jid _ filename 
+    return [file join ${options(logdir)} $filename]
 }
 
 proc ::logger::log_message {chatid from type body x} {
@@ -150,17 +175,24 @@ proc ::logger::do_search {txt entr back} {
     }
 }
 
-proc ::logger::show_log {jid} {
+proc ::logger::show_log {jid args} {
     global font
     global tcl_platform
     global defaultnick
-    global loginconf
-    global searchicon
 
     variable search_mode
 
+    foreach {key val} $args {
+	switch -- $key {
+	    -connection { set connid $val }
+	}
+    }
+    if {![info exists connid]} {
+	set connid [lindex [jlib::connections] 0]
+    }
+
     set nas [node_and_server_from_jid $jid]
-    if {![chat::is_groupchat $nas]} {
+    if {![chat::is_groupchat [chat::chatid $connid $nas]]} {
 	set jid $nas
     }
 
@@ -178,7 +210,7 @@ proc ::logger::show_log {jid} {
 	set hist [read $fd]
 	close $fd
 
-	set mynick [get_group_nick $jid $loginconf(user)]
+	set mynick [get_group_nick $jid ""]
 
 
 	toplevel $lw -relief $::tk_relief -borderwidth $::tk_borderwidth -class Chat
