@@ -1,7 +1,9 @@
-# $Id: me_command.tcl,v 1.7 2005/03/12 22:03:04 aleksey Exp $
+# $Id: me_command.tcl,v 1.9 2006/01/08 21:52:17 aleksey Exp $
 
 proc handle_me {chatid from type body x} {
     if {[cequal [crange $body 0 3] "/me "] || [cequal $body "/me"]} {
+	set body [crange $body 4 end]
+
 	if {[chat::is_our_jid $chatid $from]} {
 	    set tag me
 	} else {
@@ -18,11 +20,20 @@ proc handle_me {chatid from type body x} {
 
 	$chatw insert end "* $nick " $tag
 
-	chat::add_emoteiconed_text $chatw [crange $body 4 end] $tag
-	if {$type == "chat"} {
-	    tab_set_updated $cw 1 mesg_to_user
+	if {[cequal $type groupchat]} {
+	    set myjid [chat::our_jid $chatid]
+	    set mynick [chat::get_nick $connid $myjid $type]
+
+	    lassign [check_message $mynick $body] mymessage mylist
+	    chat::add_emoteiconed_text $chatw $body $tag $mylist
+	    if {$mymessage} {
+		tab_set_updated $cw 1 mesg_to_user
+	    } else {
+		tab_set_updated $cw 1 message
+	    }
 	} else {
-	    tab_set_updated $cw 1 message
+	    chat::add_emoteiconed_text $chatw $body $tag
+	    tab_set_updated $cw 1 mesg_to_user
 	}
 
         $chatw tag add NICK-$nick MSGLEFT "end - 1 char"
@@ -30,7 +41,7 @@ proc handle_me {chatid from type body x} {
 	return stop
     }
 }
-hook::add draw_message_hook [namespace current]::handle_me 50
+hook::add draw_message_hook [namespace current]::handle_me 83
 
 proc me_command_comp {chatid compsvar wordstart line} {
     upvar 0 $compsvar comps
