@@ -1,7 +1,7 @@
-# $Id: open_window.tcl,v 1.1 2004/07/03 22:35:59 aleksey Exp $
+# $Id: open_window.tcl,v 1.2 2005/09/27 23:14:08 aleksey Exp $
 
 proc chat_open_window {chatid from type body x} {
-    chat::open_window $chatid $type
+    chat::open_window $chatid $type -cleanroster 0
     set chatw [chat::chat_win $chatid]
     $chatw configure -state normal
 }
