@@ -1,11 +1,9 @@
-# $Id: popupmenu.tcl,v 1.7 2004/08/16 21:07:46 aleksey Exp $
+# $Id: popupmenu.tcl,v 1.9 2006/03/17 04:29:42 aleksey Exp $
 
 namespace eval popupmenu {
     hook::add postload_hook [namespace current]::init
     hook::add open_chat_post_hook [namespace current]::on_open_chat
     hook::add close_chat_post_hook [namespace current]::on_close_chat
-
-    set bookmark(image) [Bitmap::get [pixmap tkabber chat-bookmark.xpm]]
 }
 
 proc popupmenu::init {} {
@@ -111,7 +109,7 @@ proc popupmenu::BookmarkAdd {cw x y} {
 	set bookmark($cw,id) 0
     }
     $cw configure -state normal
-    $cw image create AddBookmark -image $bookmark(image)
+    $cw image create AddBookmark -image chat/bookmark
     set b [incr bookmark($cw,id)]
     $cw mark set bookmark$b AddBookmark
     $cw mark gravity bookmark$b left
@@ -193,7 +191,7 @@ proc popupmenu::BookmarkClear {cw} {
 }
 
 proc popupmenu::BookmarkRemove {cw mark} {
-    if {[lsearch [$cw mark names] $mark] != -1} {
+    if {[lsearch -exact [$cw mark names] $mark] >= 0} {
         debugmsg popupmenu "BookmarkRemove $mark"
         $cw configure -state normal
         $cw delete "$mark - 1 char"
