@@ -1,4 +1,4 @@
-# $Id: http.tcl,v 1.3 2004/09/05 20:16:10 aleksey Exp $
+# $Id: http.tcl,v 1.4 2006/03/17 04:29:42 aleksey Exp $
 
 namespace eval http {
     variable winid 0
@@ -168,6 +168,7 @@ proc http::send_file_offer {winid user filename desc ip args} {
 }
 
 proc http::send_file_accept {winid filename chan addr port} {
+    variable fds
     variable chans
     variable chanreadable$chan
 
@@ -203,6 +204,7 @@ proc http::send_file_accept {winid filename chan addr port} {
 
     set fd [open $filename]
     fconfigure $fd -translation binary
+    set fds($winid) $fd
 
     fileevent $chan writable \
 	[list [namespace current]::send_file_transfer_chunk $winid $fd $chan]
@@ -211,6 +213,7 @@ proc http::send_file_accept {winid filename chan addr port} {
 proc http::send_file_transfer_chunk {winid fd chan} {
     variable chunk_size
     variable chans
+    variable fds
 
     set chunk [read $fd $chunk_size]
     if {$chunk != "" && ![catch { puts -nonewline $chan $chunk }]} {
@@ -221,6 +224,7 @@ proc http::send_file_transfer_chunk {winid fd chan} {
 	fileevent $chan writable {}
 	close $fd
 	close $chan
+	unset fds($winid)
 	unset chans($winid)
 	destroy .ftsfd$winid
     }
@@ -228,11 +232,16 @@ proc http::send_file_transfer_chunk {winid fd chan} {
 
 proc http::send_file_cancel {winid sock} {
     variable chans
+    variable fds
 
     if {[info exists chans($winid)]} {
 	close $chans($winid)
 	unset chans($winid)
     }
+    if {[info exists fds($winid)]} {
+	close $fds($winid)
+	unset fds($winid)
+    }
 
     bind .ftsfd$winid <Destroy> {}
 
@@ -336,6 +345,7 @@ proc http::recv_file_start {winid from url} {
     variable saveas$winid
     variable chunk_size
     variable result
+    variable fds
 
     set filename [set saveas$winid]
 
@@ -344,6 +354,7 @@ proc http::recv_file_start {winid from url} {
 
     set fd [open $filename w]
     fconfigure $fd -translation binary
+    set fds($winid) $fd
 
     set geturl \
 	[list ::http::geturl $url -channel $fd \
@@ -380,6 +391,12 @@ proc http::recv_file_progress {pb token total current} {
 
 proc http::recv_file_finish {winid token} {
     variable result
+    variable fds
+
+    if {[info exists fds($winid)]} {
+	close $fds($winid)
+	unset fds($winid)
+    }
 
     upvar #0 $token state
     debugmsg filetransfer "transfer $state(status)"
@@ -391,6 +408,12 @@ proc http::recv_file_finish {winid token} {
 
 proc http::recv_file_cancel {winid {token ""}} {
     variable result
+    variable fds
+
+    if {[info exists fds($winid)]} {
+	close $fds($winid)
+	unset fds($winid)
+    }
 
     bind .ftrfd$winid <Destroy> {}
 
