@@ -1,4 +1,4 @@
-# $Id: conferenceinfo.tcl,v 1.13 2005/03/03 01:55:50 aleksey Exp $
+# $Id: conferenceinfo.tcl,v 1.15 2006/01/17 00:47:14 aleksey Exp $
 
 namespace eval conferenceinfo {
     custom::defgroup ConferenceInfo \
@@ -9,7 +9,8 @@ with the conference."] \
 	-tag "Conference Info"
 
     custom::defvar options(autoask) 0 \
-	[::msgcat::mc "Use this module"] -group ConferenceInfo -type boolean
+	[::msgcat::mc "Use this module"] -group ConferenceInfo -type boolean \
+	-command [namespace current]::ask
     custom::defvar options(interval) 2 \
 	[::msgcat::mc "Interval (in minutes) between requests of participants list."] \
 	-type integer -group ConferenceInfo
@@ -62,7 +63,7 @@ hook::add roster_user_popup_info_hook \
     [namespace current]::conferenceinfo::add_user_popup_info
 
 
-proc conferenceinfo::ask {} {
+proc conferenceinfo::ask {args} {
     variable options
     variable data
 
@@ -121,8 +122,8 @@ proc conferenceinfo::receive {jid mech res child} {
     set data(users_$mech,$jid) {}
 
     if {$res != "OK"} {
-	lassign [jlib::error_to_list $child] type \
-	    data(error_${mech}_code,$jid) data(error_$mech,$jid)
+	set data(error_${mech}_code,$jid) [lindex [error_type_condition $child] 1]
+	set data(error_$mech,$jid) [error_to_string $child]
 	return
     }
 
@@ -154,15 +155,14 @@ proc conferenceinfo::receive {jid mech res child} {
     set data(users_$mech,$jid) [lsort -dictionary $data(users_$mech,$jid)]
 }
 
-proc conferenceinfo::stop {} {
-    after cancel [list [namespace current]::ask]
+proc conferenceinfo::stop {args} {
+    if {[jlib::connections] == {}} {
+	after cancel [list [namespace current]::ask]
+    }
 }
 
-hook::add connected [namespace current]::conferenceinfo::ask
-hook::add disconnected [namespace current]::conferenceinfo::stop
-trace variable [namespace current]::conferenceinfo::options(autoask) w \
-    "[namespace current]::conferenceinfo::ask ; #"
-
+hook::add roster_end_hook [namespace current]::conferenceinfo::ask
+hook::add disconnected_hook [namespace current]::conferenceinfo::stop
 
 proc conferenceinfo::setup_menu {} {
     set m [.mainframe getmenu roster]
@@ -172,3 +172,4 @@ proc conferenceinfo::setup_menu {} {
 	-variable [namespace current]::options(autoask)
 }
 hook::add finload_hook [namespace current]::conferenceinfo::setup_menu
+
