@@ -1,26 +1,32 @@
-# $Id: conferences.tcl,v 1.3 2005/08/14 16:25:27 aleksey Exp $
+# $Id: conferences.tcl,v 1.9 2006/05/12 21:31:18 aleksey Exp $
 
 # JEP-0048 Bookmarks support (conference bookmarks in roster)
+# In addition to JEP-0048, Tkabber stores roster groups using
+# proprietory namespace tkabber:bookmarks:groups inside
+# jabber:iq:private storage (JEP-0049)
 #
 #   tkabber:bookmarks:groups description:
 #
 #   setting:
-#   <query xmlns='jabber:iq:private'>
-#	<storage xmlns='tkabber:bookmarks:groups'>
-#	    <conference jid='talks@conference.jabber.ru'>
-#		<group>Conferences</group>
-#		<group>jabber.ru</group>
-#	    </conference>
-#	</storage>
-#   </query>
+#   <iq type='set' id='setgroups'>
+#	<query xmlns='jabber:iq:private'>
+#	    <storage xmlns='tkabber:bookmarks:groups'>
+#		<conference jid='talks@conference.jabber.ru'>
+#		    <group>Conferences</group>
+#		    <group>jabber.ru</group>
+#		</conference>
+#	    </storage>
+#	</query>
+#   </iq>
 #
 #   getting:
-#   <query xmlns='jabber:iq:private'>
-#	<storage xmlns='tkabber:bookmarks:groups'/>
-#   </query>
+#   <iq type='get' id='getgroups'>
+#	<query xmlns='jabber:iq:private'>
+#	    <storage xmlns='tkabber:bookmarks:groups'/>
+#	</query>
+#   </iq>
 #
 
-
 namespace eval conferences {
     # variable to store roster conference bookmarks
     array set bookmarks {}
@@ -31,6 +37,11 @@ namespace eval conferences {
     set NS(tkabber:groups) "tkabber:bookmarks:groups"
 }
 
+###############################################################################
+#
+#   Free bookmarks on disconnect
+#
+
 proc conferences::free_bookmarks {connid} {
     variable bookmarks
 
@@ -39,6 +50,11 @@ proc conferences::free_bookmarks {connid} {
 
 hook::add disconnected_hook [namespace current]::conferences::free_bookmarks
 
+###############################################################################
+#
+#   Retrieve bookmarks on connect
+#
+
 proc conferences::request_bookmarks {connid} {
     variable NS
     variable bookmarks
@@ -85,7 +101,7 @@ proc conferences::process_bookmarks {connid res child} {
 
 		if {$btag != "conference"} continue
 
-		set jid [jlib::wrapper:getattr $bvars jid]
+		set jid [string tolower [jlib::wrapper:getattr $bvars jid]]
 		set bookmarks($connid,jid,$jid) $jid
 
 		set bookmarks($connid,name,$jid) [jlib::wrapper:getattr $bvars name]
@@ -116,7 +132,7 @@ proc conferences::process_bookmarks {connid res child} {
 
 		if {$btag != "conference"} continue
 
-		set jid [jlib::wrapper:getattr $bvars jid]
+		set jid [string tolower [jlib::wrapper:getattr $bvars jid]]
 
 		set groups {}
 		foreach bch $bchildren {
@@ -142,6 +158,11 @@ proc conferences::process_bookmarks {connid res child} {
     after idle [list [namespace current]::autojoin_groups $connid]
 }
 
+###############################################################################
+#
+#   Store bookmarks
+#
+
 proc conferences::store_bookmarks {connid} {
     variable NS
     variable bookmarks
@@ -200,7 +221,7 @@ proc conferences::store_bookmarks_result {connid res child} {
     if {$res == "OK"} return
 
     if {[winfo exists .store_bookmarks_error]} {
-	destroy .store_bookmarks_error
+	return
     }
     MessageDlg .store_bookmarks_error -aspect 50000 -icon error \
 	-message [format [::msgcat::mc "Storing conferences failed: %s"] \
@@ -208,6 +229,35 @@ proc conferences::store_bookmarks_result {connid res child} {
 	-type user -buttons ok -default 0 -cancel 0
 }
 
+###############################################################################
+#
+#   Menu item for conference window
+#
+
+proc conferences::add_conference_menu_item {m connid jid} {
+    set chatid [chat::chatid $connid $jid]
+
+    if {[info exists ::muc::muc_password($chatid)]} {
+	set password $::muc::muc_password($chatid)
+    } else {
+	set password ""
+    }
+
+    $m add command -label [::msgcat::mc "Add conference to roster..."] \
+	-command [list [namespace current]::add_conference_dialog \
+		       -group [node_from_jid $jid] \
+		       -server [server_from_jid $jid] \
+		       -password $password \
+		       -connection $connid]
+}
+
+hook::add chat_create_conference_menu_hook \
+    [namespace current]::conferences::add_conference_menu_item 35
+
+###############################################################################
+#
+#   Add conference to roster dialog
+#
 
 proc conferences::add_conference_dialog {args} {
     variable gra_group
@@ -216,6 +266,7 @@ proc conferences::add_conference_dialog {args} {
     variable gra_password
     variable gra_autojoin
     variable gra_connid
+    variable gra_rostergroup
 
     if {[lempty [jlib::connections]]} return
 
@@ -227,6 +278,7 @@ proc conferences::add_conference_dialog {args} {
     set gra_group ""
     set gra_password ""
     set gra_autojoin 0
+    set gra_rostergroup Conferences
     catch { unset gra_nick }
 
     foreach {key val} $args {
@@ -245,14 +297,14 @@ proc conferences::add_conference_dialog {args} {
     }
     set gra_connid [jlib::connection_jid $connid]
 
-    Dialog $gw -title [::msgcat::mc "Add conference"] -separator 1 -anchor e \
+    Dialog $gw -title [::msgcat::mc "Add Conference to Roster"] -separator 1 -anchor e \
 	    -default 0 -cancel 1 -parent . -modal none
 
     set gf [$gw getframe]
     grid columnconfigure $gf 0 -weight 0
     grid columnconfigure $gf 1 -weight 1
 
-    label $gf.lgroup -text [::msgcat::mc "Group:"]
+    label $gf.lgroup -text [::msgcat::mc "Conference:"]
     entry $gf.group -textvariable [namespace current]::gra_group
     label $gf.lserver -text [::msgcat::mc "Server:"]
     entry $gf.server -textvariable [namespace current]::gra_server
@@ -262,6 +314,9 @@ proc conferences::add_conference_dialog {args} {
     entry $gf.password -show * -textvariable [namespace current]::gra_password
     checkbutton $gf.autojoin -text [::msgcat::mc "Automatically join conference upon connect"] \
 	-variable [namespace current]::gra_autojoin
+    label $gf.lrostergroup -text [::msgcat::mc "Roster group:"]
+    ComboBox $gf.rostergroup -textvariable [namespace current]::gra_rostergroup \
+	-values [get_groups $connid]
 
     grid $gf.lgroup  -row 0 -column 0 -sticky e
     grid $gf.group   -row 0 -column 1 -sticky ew
@@ -272,6 +327,8 @@ proc conferences::add_conference_dialog {args} {
     grid $gf.lpassword -row 3 -column 0 -sticky e
     grid $gf.password  -row 3 -column 1 -sticky ew
     grid $gf.autojoin -row 4 -column 0 -sticky w -columnspan 2
+    grid $gf.lrostergroup  -row 5 -column 0 -sticky e
+    grid $gf.rostergroup   -row 5 -column 1 -sticky ew
 
     if {[llength [jlib::connections]] > 1} {
 	foreach c [jlib::connections] {
@@ -279,10 +336,12 @@ proc conferences::add_conference_dialog {args} {
 	}
 	label $gf.lconnection -text [::msgcat::mc "Connection:"]
 	ComboBox $gf.connection -textvariable [namespace current]::gra_connid \
-				-values $connections
+				-values $connections -editable 0 \
+				-modifycmd [list [namespace current]::change_groups \
+						 $gf.rostergroup]
 
-	grid $gf.lconnection -row 5 -column 0 -sticky e
-	grid $gf.connection  -row 5 -column 1 -sticky ew
+	grid $gf.lconnection -row 6 -column 0 -sticky e
+	grid $gf.connection  -row 6 -column 1 -sticky ew
     }
     
 
@@ -292,6 +351,24 @@ proc conferences::add_conference_dialog {args} {
     $gw draw $gf.group
 }
 
+proc conferences::change_groups {combo args} {
+    variable gra_connid
+
+    foreach connid [jlib::connections] {
+        if {[jlib::connection_jid $connid] == $gra_connid} {
+	    $combo configure -values [get_groups $connid]
+	    return
+	}
+    }
+}
+
+proc conferences::get_groups {connid} {
+    return [roster::get_groups $connid \
+		-nested $::ifacetk::roster::options(nested) \
+		-delimiter $::ifacetk::roster::options(nested_delimiter) \
+		-undefined 0]
+}
+
 proc conferences::add_conference {gw} {
     variable bookmarks
     variable gra_group
@@ -300,10 +377,16 @@ proc conferences::add_conference {gw} {
     variable gra_password
     variable gra_autojoin
     variable gra_connid
+    variable gra_rostergroup
 
     destroy $gw
 
-    set jid ${gra_group}@$gra_server
+    set jid [string tolower ${gra_group}@$gra_server]
+    if {$gra_rostergroup == ""} {
+	set groups {}
+    } else {
+	set groups [list $gra_rostergroup]
+    }
 
     foreach c [jlib::connections] {
 	if {[jlib::connection_jid $c] == $gra_connid} {
@@ -317,29 +400,70 @@ proc conferences::add_conference {gw} {
     if {[info exists bookmarks($connid,jid,$jid)]} {
 	update_bookmark $connid $jid -name $gra_group -nick $gra_nick \
 			-password $gra_password -autojoin $gra_autojoin \
-			-groups [list Conferences]
-	return
+			-groups $groups
+    } else {
+	add_bookmark $connid $jid -name $gra_group -nick $gra_nick \
+		     -password $gra_password -autojoin $gra_autojoin \
+		     -groups $groups
+    }
+}
+
+###############################################################################
+#
+#   Add bookmark to roster
+#
+
+proc conferences::add_bookmark {connid jid args} {
+    variable bookmarks
+
+    if {[info exists bookmarks($connid,jid,$jid)]} return
+
+    foreach {key val} $args {
+	switch -- $key {
+	    -name { set name $val }
+	    -nick { set nick $val }
+	    -password { set password $val }
+	    -autojoin { set autojoin $val }
+	    -groups { set groups $val }
+	}
+    }
+
+    if {![info exists name]} {
+	set name [node_from_jid $jid]
+    }
+    if {![info exists nick]} {
+	set nick [get_group_nick $jid ""]
+    }
+    if {![info exists password]} {
+	set password ""
+    }
+    if {![info exists autojoin]} {
+	set autojoin 0
+    }
+    if {![info exists groups]} {
+	set groups {}
     }
 
     set bookmarks($connid,jid,$jid) $jid
-    set bookmarks($connid,name,$jid) $gra_group
-    set bookmarks($connid,nick,$jid) $gra_nick
-    set bookmarks($connid,password,$jid) $gra_password
-    set bookmarks($connid,autojoin,$jid) $gra_autojoin
-    set bookmarks($connid,groups,$jid) [list Conferences]
+    set bookmarks($connid,name,$jid) $name
+    set bookmarks($connid,nick,$jid) $nick
+    set bookmarks($connid,password,$jid) $password
+    set bookmarks($connid,autojoin,$jid) $autojoin
+    set bookmarks($connid,groups,$jid) $groups
 
     # TODO should we remove $jid from the roster if it is here?
-    client:roster_push $connid $jid $bookmarks($connid,name,$jid) \
-		       $bookmarks($connid,groups,$jid) \
-		       bookmark "" conference ""
+    client:roster_push $connid $jid $name $groups bookmark "" conference ""
     store_bookmarks $connid
 }
 
+###############################################################################
+#
+#   Update bookmark in roster
+#
+
 proc conferences::update_bookmark {connid jid args} {
     variable bookmarks
 
-    if {![info exists bookmarks($connid,jid,$jid)]} return
-
     set store 0
 
     foreach {key val} $args {
@@ -378,10 +502,38 @@ proc conferences::update_bookmark {connid jid args} {
 	store_bookmarks $connid
     }
 }
-	    
+
+###############################################################################
+#
+#   Add or update item in roster
+#
+
+proc conferences::send_bookmark {connid jid} {
+
+    if {[roster::itemconfig $connid $jid -subsc] != "bookmark"} return
+
+    set groups [roster::itemconfig $connid $jid -group]
+
+    add_bookmark $connid $jid -groups $groups
+    update_bookmark $connid $jid -groups $groups
+
+    return stop
+}
+
+hook::add roster_send_item_hook [namespace current]::conferences::send_bookmark
+
+###############################################################################
+#
+#   Remove bookmark from roster
+#
+
 proc conferences::remove_bookmark {connid jid} {
     variable bookmarks
 
+    if {[roster::itemconfig $connid $jid -subsc] != "bookmark"} return
+
+    if {![info exists bookmarks($connid,jid,$jid)]} return
+
     client:roster_push $connid $jid $bookmarks($connid,name,$jid) \
 		       $bookmarks($connid,groups,$jid) \
 		       remove "" conference ""
@@ -394,8 +546,18 @@ proc conferences::remove_bookmark {connid jid} {
     catch { unset bookmarks($connid,groups,$jid) }
 
     store_bookmarks $connid
+
+    return stop
 }
 
+hook::add roster_remove_item_hook \
+    [namespace current]::conferences::remove_bookmark
+
+###############################################################################
+#
+#   Rename group in roster bookmarks
+#
+
 proc conferences::rename_group {connid name new_name} {
     variable bookmarks
     
@@ -424,6 +586,14 @@ proc conferences::rename_group {connid name new_name} {
     }
 }
 
+hook::add roster_rename_group_hook \
+    [namespace current]::conferences::rename_group
+
+###############################################################################
+#
+#   Remove group name from roster bookmarks
+#
+
 proc conferences::remove_bookmarks_group {connid name} {
     variable bookmarks
 
@@ -462,30 +632,13 @@ proc conferences::remove_bookmarks_group {connid name} {
     }
 }
 
-proc conferences::popup_menu {connid jid} {
-    variable bookmarks
+hook::add roster_remove_users_group_hook \
+    [namespace current]::conferences::remove_bookmarks_group
 
-    if {[winfo exists [set m .confpopupmenu]]} {
-        destroy $m
-    }
-
-    set args {}
-    if {$bookmarks($connid,nick,$jid) != ""} {
-	lappend args -nick $bookmarks($connid,nick,$jid)
-    }
-    if {$bookmarks($connid,password,$jid) != ""} {
-	lappend args -password $bookmarks($connid,password,$jid)
-    }
-
-    menu $m -tearoff 0
-    $m add command -label [::msgcat::mc "Join..."] \
-	-command [list eval [list join_group_dialog \
-				  -server [server_from_jid $jid] \
-				  -group [node_from_jid $jid] \
-				  -connection $connid] \
-				  $args]
-    return $m
-}   
+###############################################################################
+#
+#   Join group (on roster item doubleclick or during autojoin)
+#
 
 proc conferences::join_group {connid jid} {
     variable bookmarks
@@ -500,6 +653,11 @@ proc conferences::join_group {connid jid} {
     eval [list ::join_group $jid -connection $connid] $args
 }
 
+###############################################################################
+#
+#   Autojoin groups
+#
+
 proc conferences::autojoin_groups {connid} {
     variable bookmarks
 
@@ -511,3 +669,147 @@ proc conferences::autojoin_groups {connid} {
     }
 }
 
+###############################################################################
+#
+#   "Join" item in roster conference popup menu
+#
+
+proc conferences::popup_menu {m connid jid} {
+    variable bookmarks
+
+    set args {}
+    if {[roster::itemconfig $connid $jid -subsc] == "bookmark"} {
+	if {$bookmarks($connid,nick,$jid) != ""} {
+	    lappend args -nick $bookmarks($connid,nick,$jid)
+	}
+	if {$bookmarks($connid,password,$jid) != ""} {
+	    lappend args -password $bookmarks($connid,password,$jid)
+	}
+    }
+
+    $m add command -label [::msgcat::mc "Join..."] \
+	-command [list eval [list join_group_dialog \
+				  -server [server_from_jid $jid] \
+				  -group [node_from_jid $jid] \
+				  -connection $connid] \
+				  $args]
+}   
+
+hook::add roster_conference_popup_menu_hook \
+    [namespace current]::conferences::popup_menu 20
+
+###############################################################################
+#
+#   Roster doubleclick
+#
+
+proc conferences::roster_doubleclick {connid jid category subtype} {
+    switch -- $category {
+	conference {
+	    if {[roster::itemconfig $connid $jid -subsc] == "bookmark"} {
+		join_group $connid $jid
+	    } else {
+		global gr_nick
+		::join_group $jid \
+		    -nick [get_group_nick $jid $gr_nick] \
+		    -connection $connid
+	    }
+	    return stop
+	}
+    }
+}
+
+hook::add roster_jid_doubleclick \
+    [namespace current]::conferences::roster_doubleclick
+
+###############################################################################
+#
+#   Main menu setup
+#
+
+proc conferences::main_menu {} {
+    set m [.mainframe getmenu services]
+    $m insert 2 command -label [::msgcat::mc "Add conference to roster..."] \
+	-command [list [namespace current]::add_conference_dialog]
+}
+
+hook::add finload_hook [namespace current]::conferences::main_menu
+
+###############################################################################
+#
+#   Edit roster item
+#
+
+proc conferences::edit_item_setup {f connid jid} {
+    variable egra_name
+    variable egra_nick
+    variable egra_password
+    variable egra_autojoin
+    variable bookmarks
+
+    if {[roster::itemconfig $connid $jid -subsc] != "bookmark"} return
+
+    set tf [TitleFrame $f.prop \
+		-text [::msgcat::mc "Edit properties for %s" $jid]]
+    pack $tf -side top -expand yes -fill both
+    set g [$tf getframe]
+
+    set egra_name $bookmarks($connid,name,$jid)
+    set egra_autojoin $bookmarks($connid,autojoin,$jid)
+    if {[info exists bookmarks($connid,nick,$jid)]} {
+        set egra_nick $bookmarks($connid,nick,$jid)
+    } else {
+        set egra_nick ""
+    }
+    if {[info exists bookmarks($connid,password,$jid)]} {
+        set egra_password $bookmarks($connid,password,$jid)
+    } else {
+        set egra_password ""
+    }
+
+    label $g.lname -text [string trim [::msgcat::mc "Name: "]]
+    entry $g.name -textvariable [namespace current]::egra_name
+    label $g.lnick -text [::msgcat::mc "Nick:"]
+    entry $g.nick -textvariable [namespace current]::egra_nick
+    label $g.lpassword -text [::msgcat::mc "Password:"]
+    entry $g.password -show * -textvariable [namespace current]::egra_password
+    checkbutton $g.autojoin \
+	-text [::msgcat::mc "Automatically join conference upon connect"] \
+        -variable [namespace current]::egra_autojoin
+    grid columnconfigure $g 0 -weight 0
+    grid columnconfigure $g 1 -weight 1
+
+    grid $g.lname  -row 0 -column 0 -sticky e
+    grid $g.name   -row 0 -column 1 -sticky ew
+    grid $g.lnick -row 1 -column 0 -sticky e
+    grid $g.nick  -row 1 -column 1 -sticky ew
+    grid $g.lpassword -row 2 -column 0 -sticky e
+    grid $g.password  -row 2 -column 1 -sticky ew
+    grid $g.autojoin -row 3 -column 0 -sticky w -columnspan 2
+
+    return stop
+}
+
+hook::add roster_itemedit_setup_hook \
+    [namespace current]::conferences::edit_item_setup
+
+proc conferences::commit_bookmark_changes {connid jid groups} {
+    variable egra_name
+    variable egra_nick
+    variable egra_password
+    variable egra_autojoin
+
+    if {[roster::itemconfig $connid $jid -subsc] != "bookmark"} return
+
+    plugins::conferences::update_bookmark $connid $jid \
+	-name $egra_name -nick $egra_nick -password $egra_password \
+	-autojoin $egra_autojoin -groups $groups
+
+    return stop
+}
+
+hook::add roster_itemedit_commit_hook \
+    [namespace current]::conferences::commit_bookmark_changes
+
+###############################################################################
+
