@@ -1,4 +1,4 @@
-# $Id: time.tcl,v 1.4 2003/12/28 21:38:50 aleksey Exp $
+# $Id: time.tcl,v 1.5 2006/06/12 20:38:16 aleksey Exp $
 
 custom::defvar options(reply_iq_time) 1 \
     [::msgcat::mc "Reply to current time (jabber:iq:time) requests."] \
@@ -33,7 +33,8 @@ proc iq_time {connid from child} {
 proc timezone {time} {
     global tcl_platform
 
-    if {[cequal $tcl_platform(platform) "windows"]} {
+    if {[cequal $tcl_platform(platform) "windows"] && \
+	    [package vcompare [info patchlevel] 8.4.12] < 0} {
 	return [string trim [encoding convertfrom utf-8 \
 		[encoding convertto [clock format $time -format "%Z%t"]]]]
     } else {
@@ -44,7 +45,8 @@ proc timezone {time} {
 proc displaytime {time} {
     global tcl_platform
 
-    if {[cequal $tcl_platform(platform) "windows"]} {
+    if {[cequal $tcl_platform(platform) "windows"] && \
+	    [package vcompare [info patchlevel] 8.4.12] < 0} {
 	return [encoding convertfrom utf-8 \
 		[encoding convertto [clock format $time]]]
     } else {
