@@ -1,4 +1,4 @@
-# $Id: ispell.tcl,v 1.10 2005/07/26 23:22:17 aleksey Exp $
+# $Id: ispell.tcl,v 1.11 2006/02/25 19:57:36 aleksey Exp $
 
 if {![info exists ::use_ispell] || !$::use_ispell} {
     return
@@ -33,11 +33,11 @@ proc ispell::start {} {
 
     if {[info exists options(dictionary)] \
 	    && ![cequal $options(dictionary)  ""]} {
-	set dict_string "-d $options(dictionary)"
+	set dict_string "-d [list $options(dictionary)]"
     } else {
 	set dict_string ""
     }
-    set pipe [open "|$options(executable) -a $dict_string" r+]
+    set pipe [open "|[list $options(executable)] -a $dict_string" r+]
     set version [gets $pipe]
     if {[cequal $version ""]} {
 	debugmsg plugins "ISPELL: Could not start ispell server. Check your dictionary name."
