@@ -1,5 +1,6 @@
-# $Id: presence.tcl,v 1.63 2005/04/25 23:56:56 aleksey Exp $
+# $Id: presence.tcl,v 1.69 2006/06/12 20:38:16 aleksey Exp $
 
+###############################################################################
 
 proc client:presence {connid from type x args} {
     global presence
@@ -12,10 +13,6 @@ proc client:presence {connid from type x args} {
     switch -- $type {
 	error -
 	unavailable {
-            if {[cequal $type unavailable]} {
-	        presence_process_x $connid $from $type $x
-	    }
-
 	    catch { unset presence(type,$connid,$from) }
 	    catch { unset presence(status,$connid,$from) }
 	    catch { unset presence(priority,$connid,$from) }
@@ -24,7 +21,6 @@ proc client:presence {connid from type x args} {
 	    catch { unset presence(show,$connid,$from) }
 	    catch { unset presence(loc,$connid,$from) }
 	    catch { unset presence(x,$connid,$from) }
-	    catch { unset presence(signed,$connid,$from) }
 	    catch { unset presence(error,$connid,$from) }
 
 	    set user [node_and_server_from_jid $from]
@@ -50,12 +46,10 @@ proc client:presence {connid from type x args} {
 		}
 	    }
 
-	    roster::on_change_jid_presence $connid $from
 	    debugmsg presence "$connid $from unavailable"
-	    chat::change_presence $connid $from $type
 	}
-	subscribe {eval [list message::show_subscribe_dialog $from $x -connection $connid] $args}
-	subscribed {eval [list roster::process_subscribed $connid $from $x] $args}
+	subscribe {}
+	subscribed {}
 	unsubscribe {}
 	unsubscribed {
 	    MessageDlg .unsubscribed -aspect 50000 -icon info \
@@ -66,6 +60,7 @@ proc client:presence {connid from type x args} {
 	}
 	probe {}
 	default {
+	    set type available
 	    set presence(type,$connid,$from)     available
 	    set presence(status,$connid,$from)   ""
 	    set presence(priority,$connid,$from) 0
@@ -86,8 +81,8 @@ proc client:presence {connid from type x args} {
 		}
 	    }
 	    
-	    set presence(show,$connid,$from) [normalize_show $presence(show,$connid,$from)]
-	    set show $presence(show,$connid,$from)
+	    set presence(show,$connid,$from) \
+		[normalize_show $presence(show,$connid,$from)]
 
 	    set user [node_and_server_from_jid $from]
 	    if {![info exists presence(user_jids,$connid,$user)] || \
@@ -97,18 +92,13 @@ proc client:presence {connid from type x args} {
 
 	    cache_preferred_jid_on_available $connid $from $user
 	    cache_user_status $connid $user
-
-	    presence_process_x $connid $from available $x
-
-	    roster::on_change_jid_presence $connid $from
-	    chat::change_presence $connid $from $show
 	}
     }
     
-    hook::run client_presence_hook $connid $from $type $x $args
+    eval {hook::run client_presence_hook $connid $from $type $x} $args
 }
 
-
+###############################################################################
 
 proc get_jids_of_user {connid user} {
     global presence
@@ -244,7 +234,6 @@ proc get_user_status_desc {connid user} {
 }
 
 array set status_priority {
-    stalker     0
     unavailable 1
     xa          2
     away        3
@@ -277,20 +266,41 @@ proc max_status {s1 s2} {
     }
 }
 
-set curpriority [set userpriority 0]
-set curuserstatus [set userstatus available]
-set curtextstatus [set textstatus ""]
+###############################################################################
+
+set curpriority   0
+set curuserstatus unavailable
+set curtextstatus ""
+
+custom::defvar userpriority 0 [::msgcat::mc "Stored user priority."] \
+    -type integer -group Hidden
+custom::defvar userstatus available [::msgcat::mc "Stored user status."] \
+    -type string -group Hidden
+custom::defvar textstatus "" [::msgcat::mc "Stored user text status."] \
+    -type string -group Hidden
+
+proc save_user_status {} {
+    global userpriority userstatus textstatus
+
+    set custom::saved(::::userpriority) $userpriority
+    set custom::saved(::::userstatus) $userstatus
+    set custom::saved(::::textstatus) $textstatus
+    custom::store
+}
+
+hook::add quit_hook save_user_status
 
 set userstatusdesc [::msgcat::mc "Not logged in"]
 
-set statusdesc(available) [::msgcat::mc "Available"]
-set statusdesc(chat) [::msgcat::mc "Free to chat"]
-set statusdesc(away) [::msgcat::mc "Away"]
-set statusdesc(xa) [::msgcat::mc "Extended away"]
-set statusdesc(dnd) [::msgcat::mc "Do not disturb"]
-set statusdesc(invisible) [::msgcat::mc "Invisible"]
+set statusdesc(available)   [::msgcat::mc "Available"]
+set statusdesc(chat)        [::msgcat::mc "Free to chat"]
+set statusdesc(away)        [::msgcat::mc "Away"]
+set statusdesc(xa)          [::msgcat::mc "Extended away"]
+set statusdesc(dnd)         [::msgcat::mc "Do not disturb"]
+set statusdesc(invisible)   [::msgcat::mc "Invisible"]
 set statusdesc(unavailable) [::msgcat::mc "Unavailable"]
 
+###############################################################################
 
 proc change_priority_dialog {} {
     global tmppriority
@@ -323,9 +333,10 @@ proc change_priority_dialog {} {
     grid columnconfigure $f 1 -weight 1
 
     $w draw
-
 }
 
+###############################################################################
+
 proc do_change_priority {w} {
     global userstatus
     global tmppriority
@@ -338,9 +349,13 @@ proc do_change_priority {w} {
     }
 }
 
+###############################################################################
+
 trace variable userstatus w change_our_presence
 trace variable logoutuserstatus w change_our_presence
 
+###############################################################################
+
 proc change_our_presence {name1 name2 op} {
     global userstatus logoutuserstatus curuserstatus
     global textstatus logouttextstatus curtextstatus
@@ -354,6 +369,7 @@ proc change_our_presence {name1 name2 op} {
 	    set newpriority $logoutpriority
 	}
 	default {
+	    if {[lempty [jlib::connections]]} return
 	    set newstatus $userstatus
 	    set newtextstatus $textstatus
 	    set newpriority $userpriority
@@ -371,63 +387,48 @@ proc change_our_presence {name1 name2 op} {
     }
 
     set userstatusdesc $statusdesc($newstatus)
-    set status $newtextstatus
-    if {![cequal [info commands ::ssj::signP] ""] && [ssj::signP] && \
-	    [cequal $status {}]} {
+    if {[cequal $newtextstatus ""]} {
 	set status $userstatusdesc
+    } else {
+	set status $newtextstatus
     }
 
-    switch -- $newstatus {
-	available {
-	    set command [list jlib::send_presence \
-			      -pri $newpriority]
-	}
-        unavailable -
-	invisible {
-	    set command [list jlib::send_presence \
-			      -type $newstatus -pri $newpriority]
-	}
-	default {
-	    set command [list jlib::send_presence \
-			      -show $newstatus -pri $newpriority]
-	}
-    }
-    if {![cequal $status ""]} {
-	lappend command -stat $status
-    }
-    debugmsg presence $command
-
     foreach connid [jlib::connections] {
-	set x [hook::foldl presence_xlist_hook {} $connid $status]
-	eval $command [list -xlist $x -connection $connid]
+	send_presence $newstatus \
+		      -stat $status \
+		      -pri $userpriority \
+		      -connection $connid
     }
-    if {$newstatus != "invisible"} {
-	foreach chatid [array names chat::opened] {
-	    set connid [chat::get_connid $chatid]
-	    set group [chat::get_jid $chatid]
-	    if {[cequal $chat::chats(type,$chatid) groupchat]} {
-		set nick [get_our_groupchat_nick $chatid]
-		eval $command [list -to $group/$nick -connection $connid]
-	    }
-	}
-    } else {
-	foreach chatid [array names chat::opened] {
-	    set connid [chat::get_connid $chatid]
-	    set group [chat::get_jid $chatid]
-	    if {[cequal $chat::chats(type,$chatid) groupchat]} {
-		set nick [get_our_groupchat_nick $chatid]
-		jlib::send_presence -to $group/$nick -xlist $x \
-		    -connection $connid
+
+    foreach chatid [array names chat::opened] {
+	set connid [chat::get_connid $chatid]
+	set group [chat::get_jid $chatid]
+	if {[cequal $chat::chats(type,$chatid) groupchat]} {
+	    set nick [get_our_groupchat_nick $chatid]
+
+	    if {$newstatus == "invisible"} {
+		set newst available
+	    } else {
+		set newst $newstatus
 	    }
+
+	    send_presence $newst \
+			  -to $group/$nick \
+			  -stat $status \
+			  -pri $userpriority \
+			  -connection $connid
 	}
     }
 
     set curuserstatus $newstatus
     set curtextstatus $newtextstatus
     set curpriority $newpriority
+
     hook::run change_our_presence_post_hook $newstatus
 }
 
+###############################################################################
+
 proc send_first_presence {connid} {
     global userstatus curuserstatus statusdesc userstatusdesc
     global textstatus curtextstatus
@@ -439,95 +440,98 @@ proc send_first_presence {connid} {
     }
 
     set userstatusdesc $statusdesc($userstatus)
-    set status $textstatus
-    if {![cequal [info commands ::ssj::signP] ""] && [ssj::signP] && \
-	    [cequal $status {}]} {
+    if {[cequal $textstatus ""]} {
 	set status $userstatusdesc
+    } else {
+	set status $textstatus
     }
 
     set curuserstatus $userstatus
     set curtextstatus $textstatus
     set curpriority [set userpriority $loginconf(priority)]
     
-    set x [hook::foldl presence_xlist_hook {} $connid $status]
-
-    switch -- $userstatus {
-	available {
-	    set command [list jlib::send_presence \
-			      -xlist $x -pri $userpriority]
-	}
-	unavailable -
-	invisible {
-	    set command [list jlib::send_presence \
-			      -type $userstatus -xlist $x -pri $userpriority]
-	}
-	default {
-	    set command [list jlib::send_presence \
-			      -show $userstatus -xlist $x -pri $userpriority]
-	}
-    }
-    if {![cequal $status ""]} {
-	lappend command -stat $status
-    }
-    debugmsg presence $command
-
-    eval $command -connection $connid
+    send_presence $userstatus \
+		  -stat $status \
+		  -pri $userpriority \
+		  -connection $connid
 
     hook::run change_our_presence_post_hook $userstatus
 }
 
 hook::add connected_hook [namespace current]::send_first_presence 10
 
+###############################################################################
+
 proc send_custom_presence {jid status args} {
-    global statusdesc
     global userpriority
+    global statusdesc
 
     set type jid
-    foreach {opt val} $args {
-	switch -- $opt {
+    set stat ""
+    foreach {key val} $args {
+	switch -- $key {
 	    -type       { set type $val }
+	    -stat       { set stat $val }
 	    -connection { set connid $val }
 	}
     }
     if {![info exists connid]} {
-	set connid [jlib::route $jid]
+	return -code error "send_custom_presence: -connection required"
     }
-
+    if {$stat == ""} {
+	set stat $statusdesc($status)
+    }
+    
     switch -- $type {
 	group   { set to $jid/[get_our_groupchat_nick [chat::chatid $connid $jid]] }
 	default { set to $jid }
     }
 
-    if {![cequal [info commands ::ssj::signP] ""] && [ssj::signP]} {
-	set stat $statusdesc($status)
-    } else {
-	set stat ""
+    eval {send_presence $status} $args {-to $to -stat $stat -pri $userpriority}
+}
+
+###############################################################################
+
+proc send_presence {status args} {
+    set xlist {}
+    set newargs {}
+    set stat ""
+    foreach {opt val} $args {
+	switch -- $opt {
+	    -to         { lappend newargs -to $val }
+	    -pri        { lappend newargs -pri $val }
+	    -command    { lappend newargs -command $val }
+	    -xlist      { set xlist $val }
+	    -stat       { set stat $val }
+	    -connection {
+		set connid $val
+		lappend newargs -connection $val
+	    }
+	}
+    }
+    if {![info exists connid]} {
+	return -code error "send_presence: -connection required"
     }
 
-    set x [hook::foldl presence_xlist_hook {} $connid $stat]
+    if {$stat != ""} {
+	lappend newargs -stat $stat
+    }
+
+    set x [hook::foldl presence_xlist_hook $xlist $connid $stat]
+    lappend newargs -xlist $x
 
     switch -- $status {
-	available {
-	    set command [list jlib::send_presence -to $to \
-			      -xlist $x -pri $userpriority]
-	}
-	unavailable {
-	    set command [list jlib::send_presence -to $to \
-			      -type $status -xlist $x -pri $userpriority]
-	}
-	default {
-	    set command [list jlib::send_presence -to $to \
-			      -show $status -xlist $x -pri $userpriority]
-	}
-    }
-    if {![cequal $stat ""]} {
-	lappend command -stat $stat
+	available   { set command [list jlib::send_presence] }
+	unavailable { set command [list jlib::send_presence -type $status] }
+	default     { set command [list jlib::send_presence -show $status] }
     }
 
-    debugmsg presence $command
-    eval $command [list -connection $connid]
+    debugmsg presence "$command $newargs"
+    eval $command $newargs
 }
 
+###############################################################################
+
 proc normalize_show {show} {
     set res $show
     
@@ -537,57 +541,12 @@ proc normalize_show {show} {
     	dnd         {}
 	xa          {}
 	unavailable {}
-	stalker     {}
 	default     {set res available}
     }
     return $res
 }
 
-
-proc presence_process_x {connid from type xs} {
-    global presence
-
-    catch { unset presence(signed,$connid,$from) }
-
-    hook::run presence_process_x_hook $connid $from $type $xs
-}
-
-proc check_signature {connid from type xs} {
-    global presence
-
-    set signature ""
-    foreach x $xs {
-	jlib::wrapper:splitxml $x tag vars isempty chdata children
-	if {[jlib::wrapper:getattr $vars xmlns] == "jabber:x:signed"} {
-	    set signature $chdata
-	    break
-	}
-    }
-
-# in case the sender didn't check the exit code from gpg...
-    if {([cequal $signature ""]) \
-	    || ([cequal [info commands ::ssj::signed:input] ""])} {
-	return
-    }
-
-    set presence(signed,$connid,$from) \
-	[ssj::signed:input $connid $from $signature $presence(status,$connid,$from) \
-		"Presence information"]
-}
-
-hook::add presence_process_x_hook [namespace current]::check_signature
-
-proc make_signature {acc connid status} {
-    if {(![cequal [info commands ::ssj::signed:output] ""]) \
-	    && (![catch { ssj::signed:output $connid $status } chdata]) \
-	    && (![cequal $chdata ""])} {
-	lappend acc [jlib::wrapper:createtag x -vars {xmlns jabber:x:signed} \
-			 -chdata $chdata]
-    }
-    return $acc
-}
-
-hook::add presence_xlist_hook [namespace current]::make_signature
+###############################################################################
 
 proc clear_presence_info {connid} {
     global presence
@@ -611,4 +570,87 @@ proc clear_presence_info {connid} {
 	array unset presence maxpriority,$connid,*
     }
 }
+
 hook::add disconnected_hook clear_presence_info
+
+###############################################################################
+
+proc custom_presence_menu {m connid jid} {
+    set mm [menu $m.custom_presence -tearoff 0]
+
+    $mm add command -label [::msgcat::mc "Available"] \
+	-command [list send_custom_presence $jid available -connection $connid]
+    $mm add command -label [::msgcat::mc "Free to chat"] \
+	-command [list send_custom_presence $jid chat -connection $connid]
+    $mm add command -label [::msgcat::mc "Away"] \
+	-command [list send_custom_presence $jid away -connection $connid]
+    $mm add command -label [::msgcat::mc "Extended away"] \
+	-command [list send_custom_presence $jid xa -connection $connid]
+    $mm add command -label [::msgcat::mc "Do not disturb"] \
+	-command [list send_custom_presence $jid dnd -connection $connid]
+    $mm add command -label [::msgcat::mc "Unavailable"] \
+	-command [list send_custom_presence $jid unavailable -connection $connid]
+
+    $m add cascad -label [::msgcat::mc "Send custom presence"] -menu $mm
+}
+
+hook::add chat_create_user_menu_hook custom_presence_menu 43
+hook::add roster_jid_popup_menu_hook custom_presence_menu 43
+
+###############################################################################
+
+proc custom_conference_presence_menu {m connid jid} {
+    set mm [menu $m.custom_presence -tearoff 0]
+
+    $mm add command -label [::msgcat::mc "Available"] \
+	-command [list send_custom_presence $jid available -connection $connid]
+    $mm add command -label [::msgcat::mc "Free to chat"] \
+	-command [list send_custom_presence $jid chat -connection $connid]
+    $mm add command -label [::msgcat::mc "Away"] \
+	-command [list send_custom_presence $jid away -connection $connid]
+    $mm add command -label [::msgcat::mc "Extended away"] \
+	-command [list send_custom_presence $jid xa -connection $connid]
+    $mm add command -label [::msgcat::mc "Do not disturb"] \
+	-command [list send_custom_presence $jid dnd -connection $connid]
+    $mm add command -label [::msgcat::mc "Unavailable"] \
+	-command [list send_custom_presence $jid unavailable -connection $connid]
+
+    $m add cascad -label [::msgcat::mc "Send custom presence"] -menu $mm
+}
+
+hook::add chat_create_conference_menu_hook custom_conference_presence_menu 43
+
+###############################################################################
+
+proc service_login {connid jid} {
+    global userstatus
+
+    switch -- $userstatus {
+	available {
+	    jlib::send_presence -to $jid -connection $connid
+	}
+	invisible {
+	    jlib::send_presence -to $jid -type $userstatus -connection $connid
+	}
+	default {
+	    jlib::send_presence -to $jid -show $userstatus -connection $connid
+	}
+    }
+}
+
+proc service_logout {connid jid} {
+    jlib::send_presence -to $jid -type unavailable -connection $connid
+}
+
+proc service_login_logout_menu_item {m connid jid} {
+    # TODO
+    $m add command -label [::msgcat::mc "Log in"] \
+	-command [list service_login $connid $jid]
+    $m add command -label [::msgcat::mc "Log out"] \
+	-command [list service_logout $connid $jid]
+}
+
+hook::add roster_service_popup_menu_hook service_login_logout_menu_item 20
+
+###############################################################################
+
