@@ -1,4 +1,7 @@
-# $Id: privacy.tcl,v 1.12 2005/06/22 02:56:27 aleksey Exp $
+# $Id: privacy.tcl,v 1.16 2006/04/27 20:29:35 aleksey Exp $
+#
+# Privacy lists support (RFC 3921, section 10)
+#
 
 namespace eval privacy {
     variable options
@@ -18,15 +21,25 @@ namespace eval privacy {
 	      invisible [::msgcat::mc "Edit invisible list"] \
 	      visible   [::msgcat::mc "Edit visible list"]]
 
-    custom::defgroup Privacy [::msgcat::mc "Blocking communication options."] -group Tkabber
+    array set menu_messages \
+	[list ignore    [::msgcat::mc "Ignore list"] \
+	      invisible [::msgcat::mc "Invisible list"] \
+	      visible   [::msgcat::mc "Visible list"]]
+
+    custom::defgroup Privacy \
+	[::msgcat::mc "Blocking communication (XMPP privacy lists) options."] \
+	-group Tkabber
 
     custom::defvar options(activate_at_startup) 1 \
-	[::msgcat::mc "Activate visible/invisible/ignore lists before sending initial presence."] \
+	[::msgcat::mc "Activate visible/invisible/ignore lists before sending\
+		       initial presence."] \
 	-type boolean -group Privacy
 }
 
-set ::NS(privacy) jabber:iq:privacy
-
+###############################################################################
+#
+# Manual rules editing block
+#
 
 proc privacy::request_lists {args} {
     foreach {opt val} $args {
@@ -608,33 +621,6 @@ proc privacy::send_new_list {connid name} {
 }
 
 
-proc privacy::send_default_or_active_list {name tag args} {
-    set cmd jlib::noop
-    foreach {opt val} $args {
-	switch -- $opt {
-	    -command { set cmd $val }
-	    -connection { set connid $val }
-	}
-    }
-    if {![info exists connid]} {
-	set connid [jlib::route ""]
-    }
-
-    if {$name != ""} {
-	set vars [list name $name]
-    } else {
-	set vars {}
-    }
-    jlib::send_iq set \
-	[jlib::wrapper:createtag query \
-	     -vars [list xmlns $::NS(privacy)] \
-	     -subtags [list [jlib::wrapper:createtag $tag \
-				 -vars $vars]]] \
-	-command $cmd \
-	-connection $connid
-}
-
-
 proc privacy::send_list {connid ln w} {
     variable data
 
@@ -705,6 +691,38 @@ proc privacy::list_items {} {
     return $items
 }
 
+###############################################################################
+
+proc privacy::send_default_or_active_list {name tag args} {
+    set cmd jlib::noop
+    foreach {opt val} $args {
+	switch -- $opt {
+	    -command { set cmd $val }
+	    -connection { set connid $val }
+	}
+    }
+    if {![info exists connid]} {
+	set connid [jlib::route ""]
+    }
+
+    if {$name != ""} {
+	set vars [list name $name]
+    } else {
+	set vars {}
+    }
+    jlib::send_iq set \
+	[jlib::wrapper:createtag query \
+	     -vars [list xmlns $::NS(privacy)] \
+	     -subtags [list [jlib::wrapper:createtag $tag \
+				 -vars $vars]]] \
+	-command $cmd \
+	-connection $connid
+}
+
+###############################################################################
+#
+# Visible, invisible, ignore list block
+#
 
 proc privacy::edit_special_list {name args} {
     foreach {opt val} $args {
@@ -754,7 +772,8 @@ proc privacy::edit_special_list_dialog {connid name res child} {
 	-default 0 -cancel 1
 
     $w add -text [::msgcat::mc "Send"] \
-	-command [list [namespace current]::send_special_list $connid $w $name]
+	-command [list [namespace current]::edit_special_list_enddialog \
+		       $connid $w $name]
     $w add -text [::msgcat::mc "Cancel"] -command [list destroy $w]
 
     set f [$w getframe]
@@ -790,42 +809,91 @@ proc privacy::edit_special_list_dialog {connid name res child} {
 }
 
 
-proc privacy::send_special_list {connid w name} {
+proc privacy::edit_special_list_enddialog {connid w name} {
+    send_special_list $connid $name [$w.fields get 0 end]
+    destroy $w
+}
+
+proc privacy::send_special_list {connid name items} {
+    variable special_list
+    variable cboxes
+
+    if {![info exists special_list($connid,$name)]} {
+	set special_list($connid,$name) {}
+    }
+
+    set newitems {}
+    foreach jid $items {
+	if {[lsearch -exact $special_list($connid,$name) $jid] < 0} {
+	    lappend newitems $jid
+	}
+    }
+
+    set olditems {}
+    foreach jid $special_list($connid,$name) {
+	if {[lsearch -exact $items $jid] < 0} {
+	    lappend olditems $jid
+	}
+    }
+
     switch -- $name {
 	ignore {
 	    set children {}
 	    set action deny
+	    foreach jid $newitems {
+		send_custom_presence $jid unavailable -connection $connid
+	    }
+	    set postitems $olditems
 	}
 	invisible {
 	    set children [list [jlib::wrapper:createtag presence-out]]
 	    set action deny
+	    foreach jid $newitems {
+		send_custom_presence $jid unavailable -connection $connid
+	    }
+	    set postitems $olditems
 	}
 	visible {
+	    # TODO: invisibility
 	    set children [list [jlib::wrapper:createtag presence-out]]
 	    set action allow
+	    set postitems $newitems
 	}
     }
 
-    set items {}
+    set items1 {}
     set i 0
-    foreach item [$w.fields get 0 end] {
-	lappend items [jlib::wrapper:createtag item \
-			   -vars [list type jid       value $item \
-				       action $action order $i] \
-			   -subtags $children]
+    foreach item $items {
+	lappend items1 [jlib::wrapper:createtag item \
+			    -vars [list type jid \
+					value $item \
+					action $action \
+					order $i] \
+			    -subtags $children]
 	incr i
     }
 
-    send_list_iq "$name-list" $items \
-	-command [list [namespace current]::update_tkabber_lists $connid $name] \
+    send_list_iq "$name-list" $items1 \
+	-command [list [namespace current]::update_tkabber_lists \
+		       $connid $name $items $postitems] \
 	-connection $connid
-
-    destroy $w
 }
 
 
-proc privacy::update_tkabber_lists {connid name res child} {
+proc privacy::update_tkabber_lists {connid name items postitems res child} {
+    global userstatus textstatus statusdesc
     variable send_messages
+    variable special_list
+    variable cboxes
+
+    # Workaround for servers without privacy list support/push
+    if {$res == "OK"} {
+	set special_list($connid,$name) $items
+    }
+    array unset cboxes $connid,$name,*
+    foreach jid $special_list($connid,$name) {
+	set cboxes($connid,$name,$jid) 1
+    }
 
     if {$res != "OK"} {
 	MessageDlg .privacy_list_err -aspect 50000 -icon error \
@@ -835,6 +903,12 @@ proc privacy::update_tkabber_lists {connid name res child} {
     }
 
     switch -- $name {
+	ignore {
+	    join_lists $connid "i-am-visible-list" {ignore-list invisible-list} \
+		{allow {}}
+	    join_lists $connid "i-am-invisible-list" {ignore-list visible-list} \
+		[list deny [list [jlib::wrapper:createtag presence-out]]]
+	}
 	invisible {
 	    join_lists $connid "i-am-visible-list" {ignore-list invisible-list} \
 		[list allow [list]]
@@ -843,17 +917,28 @@ proc privacy::update_tkabber_lists {connid name res child} {
 	    join_lists $connid "i-am-invisible-list" {ignore-list visible-list} \
 		[list deny [list [jlib::wrapper:createtag presence-out]]]
 	}
-	ignore {
-	    join_lists $connid "i-am-visible-list" {ignore-list invisible-list} \
-		{allow {}}
-	    join_lists $connid "i-am-invisible-list" {ignore-list visible-list} \
-		[list deny [list [jlib::wrapper:createtag presence-out]]]
-	}
+    }
+
+    # TODO: wait for iq reply before sending presence?
+    if {$userstatus == "invisible"} {
+	set status available
+    } else {
+	set status $userstatus
+    }
+
+    if {$textstatus == ""} {
+	set tstatus $statusdesc($status)
+    } else {
+	set tstatus $textstatus
+    }
+
+    foreach jid $postitems {
+	send_presence $status -to $jid -stat $tstatus -connection $connid
     }
 }
 
 
-proc privacy::join_lists {connid name lists fallback} {
+proc privacy::join_lists {connid name lists fallback args} {
     variable litems
 
     set items {}
@@ -881,7 +966,7 @@ proc privacy::join_lists {connid name lists fallback} {
     lappend items [jlib::wrapper:createtag item \
 		       -vars [list action [lindex $fallback 0] order $i] \
 		       -subtags [lindex $fallback 1]]
-    send_list_iq $name $items -connection $connid
+    eval { send_list_iq $name $items -connection $connid } $args
 }
 
 
@@ -975,32 +1060,329 @@ proc privacy::add_special_jid {f item} {
     $f selection set $index
 }
 
+###############################################################################
+#
+# During connect try to activate "i-am-visible-list" privacy list
+# If it's not found then create and activate it
+# If activation or creation fails then terminate connect with error message
+#
 
-proc privacy::activate_privacy_lists {connid} {
+proc privacy::activate_privacy_list {depth connid} {
     variable options
     variable answer
 
-    if {$options(activate_at_startup)} {
-	set_status [::msgcat::mc "Waiting for activating privacy list"]
-        debugmsg privacy "requested privacy list activation"
-	send_default_or_active_list "i-am-visible-list" active \
-	    -command [namespace current]::get_answer \
-	    -connection $connid
-	vwait [namespace current]::answer
-	if {$answer == "OK"} {
+    if {!$options(activate_at_startup)} return
+
+    set_status [::msgcat::mc "Waiting for activating privacy list"]
+    debugmsg privacy "requested privacy list activation"
+    send_default_or_active_list "i-am-visible-list" active \
+	-command [list [namespace current]::get_answer $connid] \
+	-connection $connid
+
+    vwait [namespace current]::answer($connid,res)
+
+    switch -- $answer($connid,res) {
+	"OK" {
 	    set_status [::msgcat::mc "Privacy list is activated"]
-	} else {
+	}
+	"ERR" {
+	    switch -- [lindex $answer($connid,error) 1] {
+		feature-not-implemented {
+		    # Privacy lists aren't implemented
+		    # Give up
+		    set_status \
+			[::msgcat::mc "Privacy lists are not implemented"]
+		}
+		service-unavailable -
+		recipient-unavailable {
+		    # Privacy lists are unavailable
+		    # Give up
+		    set_status \
+			[::msgcat::mc "Privacy lists are unavailable"]
+		}
+		item-not-found {
+		    if {$depth >= 1} {
+			# After successfully (!) created list it
+			# mustn't be possible
+			# TODO: error message
+			return
+		    }
+		    # There's no "i-am-visible-list" privacy list
+		    # Create it
+		    set_status \
+			[::msgcat::mc "Creating default privacy list"]
+
+		    join_lists $connid "i-am-visible-list" \
+			{ignore-list invisible-list} \
+			[list allow [list]] \
+			-command [list [namespace current]::get_answer \
+				       $connid]
+
+		    vwait [namespace current]::answer($connid,res)
+
+		    switch -- $answer($connid,res) {
+			"OK" {
+			    # Activate newly created list
+			    return [activate_privacy_list [expr {$depth + 1}] \
+							  $connid]
+			}
+			"ERR" {
+			    # Disconnect with error message
+			    set_status \
+				[::msgcat::mc "Privacy list is not created"]
+
+			    NonmodalMessageDlg .privacy_list_error$connid \
+				-aspect 50000 -icon error \
+				-title [::msgcat::mc "Privacy lists error"] \
+				-message [format \
+				    [::msgcat::mc \
+					"Creating default privacy list failed:\
+					 %s\n\nTry to reconnect. If problem\
+					 persists, you may want to disable privacy\
+					 list activation at start"] \
+				    $answer($connid,errormsg)]
+
+			    logout $connid
+			    clear_answer $connid
+
+			    # Break connected_hook
+			    return stop
+			}
+			default {
+			    # "DISCONNECT"
+			    set_status \
+				[::msgcat::mc "Privacy list is not created"]
+			    clear_answer $connid
+
+			    # Break connected_hook
+			    return stop
+			}
+		    }
+		}
+		default {
+		    # Something wrong
+		    # Disconnect with error message
+		    set_status \
+			[::msgcat::mc "Privacy list is not activated"]
+
+		    NonmodalMessageDlg .privacy_list_error$connid \
+			-aspect 50000 -icon error \
+			-title [::msgcat::mc "Privacy lists error"] \
+			-message [format \
+			    [::msgcat::mc \
+				"Activating privacy list failed:\
+				 %s\n\nTry to reconnect. If problem\
+				 persists, you may want to disable privacy\
+				 list activation at start"] \
+			    $answer($connid,errormsg)]
+
+		    logout $connid
+		    clear_answer $connid
+
+		    # Break connected_hook
+		    return stop
+		}
+	    }
+	}
+	default {
+	    # "DISCONNECT"
 	    set_status [::msgcat::mc "Privacy list is not activated"]
+	    clear_answer $connid
+
+	    # Break connected_hook
+	    return stop
+	}
+    }
+
+    clear_answer $connid
+}
+
+hook::add connected_hook \
+    [list [namespace current]::privacy::activate_privacy_list 0] 5
+
+proc privacy::get_answer {connid res child} {
+    variable answer
+    debugmsg privacy "got privacy list answer $connid $res $child"
+
+    if {$res == "ERR"} {
+	set answer($connid,error) [error_type_condition $child]
+	set answer($connid,errormsg) [error_to_string $child]
+    }
+    set answer($connid,res) $res
+}
+
+proc privacy::clear_answer {connid} {
+    variable answer
+
+    array unset answer $connid,*
+}
+
+###############################################################################
+
+proc privacy::create_menu {m connid jid} {
+    variable menu_messages
+    variable special_list
+    variable cboxes
+
+    set rjid [roster::find_jid $connid $jid]
+
+    if {$rjid == ""} {
+ 	set rjid [node_and_server_from_jid $jid]
+    }
+
+    set mm [menu $m.privacy_menu -tearoff 0]
+
+    foreach name {invisible ignore} {
+	if {![info exists special_list($connid,$name)]} {
+	    set special_list($connid,$name) {}
+	}
+
+	if {[lsearch -exact $special_list($connid,$name) $rjid] >= 0} {
+	    set cboxes($connid,$name,$rjid) 1
+	}
+
+	$mm add checkbutton -label $menu_messages($name) \
+	    -variable [namespace current]::cboxes($connid,$name,$rjid) \
+	    -command [list [namespace current]::add_to_special_list \
+			   $connid $name $rjid]
+    }
+
+    $m add cascad -label [::msgcat::mc "Privacy rules"] -menu $mm
+}
+
+hook::add chat_create_user_menu_hook \
+	  [namespace current]::privacy::create_menu 79
+hook::add roster_service_popup_menu_hook \
+	  [namespace current]::privacy::create_menu 79
+hook::add roster_jid_popup_menu_hook \
+	  [namespace current]::privacy::create_menu 79
+
+
+###############################################################################
+
+proc privacy::add_to_special_list {connid name jid} {
+    variable special_list
+    variable cboxes
+
+    if {![info exists special_list($connid,$name)]} {
+	set special_list($connid,$name) {}
+    }
+
+    set idx [lsearch -exact $special_list($connid,$name) $jid]
+
+    if {[info exists cboxes($connid,$name,$jid)] && \
+	    ($cboxes($connid,$name,$jid) && $idx < 0)} {
+	send_special_list $connid $name \
+			  [linsert $special_list($connid,$name) 0 $jid]
+	return
+    }
+    if {![info exists cboxes($connid,$name,$jid)] || \
+	    (!$cboxes($connid,$name,$jid) && $idx >= 0)} {
+	send_special_list $connid $name \
+			  [lreplace $special_list($connid,$name) $idx $idx]
+	return
+    }
+}
+
+###############################################################################
+
+proc privacy::process_iq {connid from child} {
+    if {$from != "" && \
+	    !([string equal -nocase $from [jlib::connection_server $connid]] || \
+	      [string equal -nocase $from [jlib::connection_bare_jid $connid]] || \
+	      [string equal -nocase $from [jlib::connection_jid $connid]])} {
+	return {error cancel not-allowed}
+    }
+
+    jlib::wrapper:splitxml $child tag vars isempty chdata children
+
+    foreach ch $children {
+	jlib::wrapper:splitxml $ch tag1 vars1 isempty1 chdata1 children1
+
+	switch -- $tag1 {
+	    list {
+		switch -- [jlib::wrapper:getattr $vars1 name] {
+		    invisible-list {
+			reload_special_list $connid invisible
+		    }
+		    visible-list {
+			reload_special_list $connid visible
+		    }
+		    ignore-list {
+			reload_special_list $connid ignore
+		    }
+		}
+	    }
+	}
+    }
+    return {result {}}
+}
+
+iq::register_handler set query $::NS(privacy) \
+		     [namespace current]::privacy::process_iq
+
+###############################################################################
+
+proc privacy::clear_list_vars {connid} {
+    variable special_list
+    variable cboxes
+
+    if {$connid == ""} {
+	catch { array unset special_list * }
+	catch { array unset cboxes * }
+    } else {
+	catch { array unset special_list $connid,* }
+	catch { array unset cboxes $connid,* }
+    }
+}
+
+hook::add disconnected_hook [namespace current]::privacy::clear_list_vars
+
+###############################################################################
+
+proc privacy::get_list_vars {connid} {
+    foreach name {invisible visible ignore} {
+	reload_special_list $connid $name
+    }
+}
+
+hook::add connected_hook [namespace current]::privacy::get_list_vars
+
+###############################################################################
+
+proc privacy::reload_special_list {connid name} {
+    jlib::send_iq get \
+	[jlib::wrapper:createtag query \
+	     -vars [list xmlns $::NS(privacy)] \
+	     -subtags [list [jlib::wrapper:createtag list \
+				 -vars [list name "$name-list"]]]] \
+	-command [list [namespace current]::store_special_list $connid $name] \
+	-connection $connid
+}
+
+proc privacy::store_special_list {connid name res child} {
+    variable special_list
+    variable cboxes
+
+    set special_list($connid,$name) {}
+    array unset cboxes $connid,$name,*
+
+    if {$res != "OK"} return
+
+    jlib::wrapper:splitxml $child tag vars isempty chdata children
+    jlib::wrapper:splitxml [lindex $children 0] tag vars isempty chdata children
+
+    if {$tag != "list"} return
+
+    foreach item $children {
+	jlib::wrapper:splitxml $item tag1 vars1 isempty1 chdata1 children1
+	if {[jlib::wrapper:getattr $vars1 type] == "jid" && \
+		[set jid [jlib::wrapper:getattr $vars1 value]] != ""} {
+	    lappend special_list($connid,$name) $jid
+	    set cboxes($connid,$name,$jid) 1
 	}
     }
 }
 
-hook::add connected_hook [namespace current]::privacy::activate_privacy_lists 5
-
-proc privacy::get_answer {res child} {
-    variable answer
-    debugmsg privacy "got privacy list answer"
-
-    set answer $res
-}
+###############################################################################
 
