@@ -1,4 +1,4 @@
-# $Id: roster.tcl,v 1.129 2005/08/14 16:25:27 aleksey Exp $
+# $Id: roster.tcl,v 1.137 2006/04/09 19:45:37 aleksey Exp $
 
 namespace eval roster {
     variable undef_group_name [::msgcat::mc "Undefined"]
@@ -26,13 +26,12 @@ proc roster::process_item {connid jid name groups subsc ask category subtype} {
 	if {![lcontain $roster(jids,$connid) $jid]} {
 	    lappend roster(jids,$connid) $jid
 	}
-	set ind [lsearch -exact $groups $undef_group_name]
-	if {$ind >= 0} {
-	    set groups [lreplace $groups $ind $ind]
-	}
-	set ind [lsearch -exact $groups $chats_group_name]
-	if {$ind >= 0} {
-	    set groups [lreplace $groups $ind $ind]
+	set groups [lrmdups $groups]
+	foreach group [list "" $undef_group_name $chats_group_name] {
+	    set ind [lsearch -exact $groups $group]
+	    if {$ind >= 0} {
+		set groups [lreplace $groups $ind $ind]
+	    }
 	}
 	set roster(group,$connid,$jid)    $groups
 	set roster(name,$connid,$jid)     $name
@@ -44,24 +43,16 @@ proc roster::process_item {connid jid name groups subsc ask category subtype} {
 	catch {unset roster(cached_category_and_subtype,$connid,$jid)}
 	lassign [get_category_and_subtype $connid $jid] ccategory csubtype
 	set roster(isuser,$connid,$jid) [cequal $ccategory user]
-
-	if {[info exists roster(subscribed_name,$connid,$jid)]} {
-	    if {$roster(name,$connid,$jid) == ""} {
-		set roster(name,$connid,$jid) $roster(subscribed_name,$connid,$jid)
-		send_item $connid $jid
-	    }
-	    unset roster(subscribed_name,$connid,$jid)
-	}
     } else {
-	lvarpop roster(jids,$connid) [lsearch $roster(jids,$connid) $jid]
+	lvarpop roster(jids,$connid) [lsearch -exact $roster(jids,$connid) $jid]
 
-	unset roster(group,$connid,$jid)
-	unset roster(name,$connid,$jid)
-	unset roster(subsc,$connid,$jid)
-	unset roster(ask,$connid,$jid)
-	unset roster(category,$connid,$jid)
-	unset roster(subtype,$connid,$jid)
-	unset roster(isuser,$connid,$jid)
+	catch {unset roster(group,$connid,$jid)}
+	catch {unset roster(name,$connid,$jid)}
+	catch {unset roster(subsc,$connid,$jid)}
+	catch {unset roster(ask,$connid,$jid)}
+	catch {unset roster(category,$connid,$jid)}
+	catch {unset roster(subtype,$connid,$jid)}
+	catch {unset roster(isuser,$connid,$jid)}
 	catch {unset roster(cached_category_and_subtype,$connid,$jid)}
     }
 }
@@ -84,6 +75,7 @@ proc client:roster_cmd {connid status} {
     debugmsg roster "ROSTER_CMD: $status"
     
     if {[cequal $status END_ROSTER]} {
+	hook::run roster_end_hook $connid
 	::redraw_roster
     }
 }
@@ -223,6 +215,8 @@ proc roster::itemconfig {connid jid args} {
 	}
 	if {[info exists roster($param,$connid,$jid)]} {
 	    return $roster($param,$connid,$jid)
+	} elseif {$param == "isuser"} {
+	    return 0
 	} else {
 	    return ""
 	}
@@ -243,10 +237,17 @@ proc roster::itemconfig {connid jid args} {
     }
 }
 
-proc roster::on_change_jid_presence {connid jid} {
+proc roster::on_change_jid_presence {connid jid type x args} {
     variable long_statusdesc
     variable roster
-    
+
+    switch -- $type {
+	error -
+	unavailable -
+	available {}
+	default { return }
+    }
+
     set rjid [find_jid $connid $jid]
     debugmsg roster "$jid $rjid"
 
@@ -265,6 +266,8 @@ proc roster::on_change_jid_presence {connid jid} {
     ::redraw_roster
 }
 
+hook::add client_presence_hook roster::on_change_jid_presence 60
+
 proc roster::find_jid {connid jid} {
     variable roster
 
@@ -398,22 +401,7 @@ proc roster::is_user {connid jid} {
     return [cequal [lindex [get_category_and_subtype $connid $jid] 0] "user"]
 }
 
-proc roster::process_subscribed {connid from x args} {
-    variable roster
-
-    set jid [tolower_node_and_domain $from]
-
-    set name ""
-    foreach {opt val} $args {
-	switch -- $opt {
-	    -name { set name $val }
-	}
-    }
-
-    if {([find_jid $connid $jid] == "") && ($name != "")} {
-	set roster(subscribed_name,$connid,$jid) $name
-    }
-}
+###############################################################################
 
 proc roster::item_to_xml {connid jid} {
     variable roster
@@ -422,7 +410,9 @@ proc roster::item_to_xml {connid jid} {
 
     set grtags {}
     foreach group $roster(group,$connid,$jid) {
-	if {![cequal $group $undef_group_name] && ![cequal $group $chats_group_name]} {
+	if {![cequal $group ""] && \
+		![cequal $group $undef_group_name] && \
+		![cequal $group $chats_group_name]} {
 	    lappend grtags [jlib::wrapper:createtag group -chdata $group]
 	}
     }
@@ -433,19 +423,17 @@ proc roster::item_to_xml {connid jid} {
 	lappend vars name $roster(name,$connid,$jid)
     }
 
-#    if {$roster(category,$connid,$jid) != ""} {
-#	lappend vars category $roster(category,$connid,$jid)
-#	if {$roster(subtype,$connid,$jid) != ""} {
-#	    lappend vars type $roster(subtype,$connid,$jid)
-#	}
-#    }
-
-    return [jlib::wrapper:createtag item \
-		-vars $vars \
-		-subtags $grtags]
+    return [jlib::wrapper:createtag item -vars $vars -subtags $grtags]
 }
 
+###############################################################################
+
 proc roster::send_item {connid jid} {
+    hook::run roster_send_item_hook $connid $jid
+}
+
+proc roster::send_item_fallback {connid jid} {
+
     jlib::send_iq set \
 	[jlib::wrapper:createtag query \
 	     -vars {xmlns jabber:iq:roster} \
@@ -453,12 +441,15 @@ proc roster::send_item {connid jid} {
 	-connection $connid
 }
 
-proc roster::remove_item {connid jid} {
+hook::add roster_send_item_hook roster::send_item_fallback 100
 
-    if {[itemconfig $connid $jid -subsc] == "bookmark"} {
-	plugins::conferences::remove_bookmark $connid $jid
-	return
-    }
+###############################################################################
+
+proc roster::remove_item {connid jid} {
+    hook::run roster_remove_item_hook $connid $jid
+}
+
+proc roster::remove_item_fallback {connid jid} {
 
     jlib::send_iq set \
 	[jlib::wrapper:createtag query \
@@ -482,6 +473,8 @@ proc roster::remove_item {connid jid} {
     }
 }
 
+hook::add roster_remove_item_hook roster::remove_item_fallback 100
+
 ###############################################################################
 
 proc roster::send_rename_group {connid name new_name} {
@@ -490,13 +483,14 @@ proc roster::send_rename_group {connid name new_name} {
 
     if {[string equal $new_name $name]} return
 
-    plugins::conferences::rename_group $connid $name $new_name
+    hook::run roster_rename_group_hook $connid $name $new_name
 
     set items {}
 
     foreach jid $roster(jids,$connid) {
-	if {[itemconfig $connid $jid -subsc] == "bookmark"} {
-	    continue
+	switch -- [itemconfig $connid $jid -subsc] {
+	    none - from - to - both { }
+	    default { continue }
 	}
 
 	if {[lcontain $roster(group,$connid,$jid) $name] || \
@@ -524,17 +518,20 @@ proc roster::send_rename_group {connid name new_name} {
     }
 }
 
+###############################################################################
+
 proc roster::send_remove_users_group {connid name} {
     variable roster
     variable undef_group_name
 
-    plugins::conferences::remove_bookmarks_group $connid $name
+    hook::run roster_remove_users_group_hook $connid $name
 
     set items {}
 
     foreach jid $roster(jids,$connid) {
-	if {[itemconfig $connid $jid -subsc] == "bookmark"} {
-	    continue
+	switch -- [itemconfig $connid $jid -subsc] {
+	    none - from - to - both { }
+	    default { continue }
 	}
 
 	set groups $roster(group,$connid,$jid)
@@ -557,11 +554,16 @@ proc roster::send_remove_users_group {connid name} {
     }
 }
 
+###############################################################################
+
 proc roster::resubscribe_group {connid name} {
     variable roster
+    variable undef_group_name
 
     foreach jid $roster(jids,$connid) {
-	if {[lcontain $roster(group,$connid,$jid) $name]} {
+	if {[lcontain $roster(group,$connid,$jid) $name] || \
+		($name == $undef_group_name && \
+		     $roster(group,$connid,$jid) == {})} {
 	    lassign [get_category_and_subtype $connid $jid] category type
 	    if {$category == "user"} {
 		jlib::send_presence \
@@ -573,6 +575,26 @@ proc roster::resubscribe_group {connid name} {
     }
 }
 
+###############################################################################
+
+proc roster::send_custom_presence_group {connid name status} {
+    variable roster
+    variable undef_group_name
+
+    foreach jid $roster(jids,$connid) {
+	if {[lcontain $roster(group,$connid,$jid) $name] || \
+		($name == $undef_group_name && \
+		     $roster(group,$connid,$jid) == {})} {
+	    lassign [get_category_and_subtype $connid $jid] category type
+	    if {$category == "user"} {
+		send_custom_presence $jid $status -connection $connid
+	    }
+	}
+    }
+}
+
+###############################################################################
+
 proc roster::add_group_by_jid_regexp {name regexp} {
     variable roster
 
@@ -602,49 +624,6 @@ proc roster::add_group_by_jid_regexp {name regexp} {
     }
 }
 
-
-
-###############################################################################
-
-proc roster::send_users {gw jid args} {
-    variable roster
-    global send_uc
-
-    foreach {opt val} $args {
-	switch -- $opt {
-	    -connection { set connid $val }
-	}
-    }
-    if {![info exists connid]} {
-	set connid [jlib::route $jid]
-    }
-
-    set sf [$gw getframe].sw.sf
-    set choices {}
-    foreach uc [array names send_uc] {
-        if {$send_uc($uc)} {
-            lappend choices $uc
-        }
-    }
-
-    destroy $gw
-
-    set subtags {}
-    set body [::msgcat::mc "Contact Information"]
-    foreach choice $choices {
-	lassign $choice con uc
-	lappend subtags [item_to_xml $con $uc]
-	set nick [roster::get_label $con $uc]
-        append body "\n$nick - xmpp:$uc"
-    }
-
-    message::send_msg $jid -type normal -body $body \
-	-xlist [list [jlib::wrapper:createtag x \
-	                  -vars [list xmlns jabber:x:roster] \
-	                  -subtags $subtags]] \
-	-connection $connid
-}
-
 ###############################################################################
 
 proc roster::export_to_file {connid} {
