@@ -1,4 +1,4 @@
-# $Id: search.tcl,v 1.41 2005/01/17 22:15:11 aleksey Exp $
+# $Id: search.tcl,v 1.43 2006/01/20 03:08:00 aleksey Exp $
 
 namespace eval search {
     set winid 0
@@ -41,8 +41,8 @@ proc search::open {jid args} {
     $sw.bbox add -text [::msgcat::mc "Cancel"] -command [list destroy $sw]
     pack $sw.bbox -padx 2m -pady 2m -anchor e -side bottom
 
-    bind $sw <Return> "ButtonBox::invoke $sw.bbox default"
-    bind $sw <Escape> "ButtonBox::invoke $sw.bbox 1"
+    bind $sw <Return> [list ButtonBox::invoke $sw.bbox default]
+    bind $sw <Escape> [list ButtonBox::invoke $sw.bbox 1]
 
     Separator::create $sw.sep -orient horizontal
     pack $sw.sep -side bottom -fill x -pady 1m
@@ -165,8 +165,8 @@ proc search::recv_items {sw jid connid res child} {
 							destroy [list $sw]"
     pack $rw.bbox -padx 2m -pady 2m -anchor e -side bottom
 
-    bind $rw <Return> "ButtonBox::invoke $sw.bbox default"
-    bind $rw <Escape> "ButtonBox::invoke $sw.bbox 1"
+    bind $rw <Return> [list ButtonBox::invoke $rw.bbox default]
+    bind $rw <Escape> [list ButtonBox::invoke $rw.bbox 1]
 
     Separator::create $rw.sep -orient horizontal
     pack $rw.sep -side bottom -fill x -pady 1m
@@ -188,9 +188,9 @@ proc search::recv_items {sw jid connid res child} {
     bind $sww.listbox <Destroy> +[list [namespace current]::delete_lastsort $sww.listbox]
     
     bind $sww.listbox <3> \
-	"[namespace current]::select_and_popup_menu $sww.listbox \
+	"[namespace current]::select_and_popup_menu [list $sww.listbox] \
 	     \[$sww.listbox nearest \[::mclistbox::convert %W -y %y\]\] \
-	     -connection $connid"
+	     $connid"
 
     bindscroll $sww $sww.listbox
 
@@ -430,46 +430,31 @@ proc search::search_again {sw jid connid {delwidget ""}} {
     }
 }
 
-proc search::select_and_popup_menu {w index args} {
+proc search::select_and_popup_menu {w index connid} {
 
     $w selection clear 0 end
     $w selection set $index
     set jid [string trim [lindex [$w get $index] 1]]
 
-    foreach {opt val} $args {
-	switch -- $opt {
-	    -connection { set connid $val }
-	}
-    }
-    if {![info exists connid]} {
-	set connid [jlib::route $jid]
-    }
-
     if {[winfo exists [set m .searchpopupmenu]]} {
 	destroy $m
     }
     menu $m -tearoff 0
-    $m add command -label [::msgcat::mc "Start chat"] \
-	-command [list chat::open_to_user $connid $jid]
-    $m add command -label [::msgcat::mc "Send message..."] \
-	-command [list message::send_dialog -to $jid -connection $connid]
-    $m add command -label [::msgcat::mc "Invite to conference..."] \
-	-command [list chat::invite_dialog $jid 0 -connection $connid]
-    $m add separator
-    $m add command -label [::msgcat::mc "Send users..."] \
-	-command [list ifacetk::roster::send_users_dialog $jid -connection $connid]
-    ft::create_menu $m $jid -connection $connid
-    $m add separator
-    $m add command -label [::msgcat::mc "Add user..."] \
-	-command [list message::send_subscribe_dialog $jid -connection $connid]
-    $m add command -label [::msgcat::mc "Show info"] \
-	-command [list userinfo::open $jid -connection $connid]
-    $m add command -label [::msgcat::mc "Show history"] \
-	-command {} -state disabled
+
+    hook::run search_popup_menu_hook $m $connid $jid
 
     tk_popup $m [winfo pointerx .] [winfo pointery .]
 }
 
+proc search::add_separator {m connid jid} {
+    $m add separator
+}
+
+hook::add search_popup_menu_hook \
+    [namespace current]::search::add_separator 40
+hook::add search_popup_menu_hook \
+    [namespace current]::search::add_separator 50
+
 hook::add postload_hook \
     [list browser::register_ns_handler jabber:iq:search \
 	[namespace current]::search::open \
