@@ -1,4 +1,4 @@
-# $Id: splash.tcl,v 1.19 2005/03/13 21:14:21 aleksey Exp $
+# $Id: splash.tcl,v 1.23 2006/05/26 22:01:38 aleksey Exp $
 
 proc splash_start {{aboutP 0}} {
     global splash_count splash_image splash_info splash_max splash_text
@@ -56,8 +56,13 @@ proc splash_start {{aboutP 0}} {
     wm overrideredirect $w 1
 
     catch {
-	if {[lsearch [wm attributes $w] -alpha] >= 0} {
-	    wm attributes $w -alpha 0.86
+	if {[lsearch -exact [wm attributes $w] -topmost] >= 0} {
+	    wm attributes $w -topmost 1
+	}
+    }
+    catch {
+	if {[lsearch -exact [wm attributes $w] -alpha] >= 0} {
+	    wm attributes $w -alpha 0.7
 	}
     }
 
@@ -74,8 +79,10 @@ proc splash_start {{aboutP 0}} {
 
     frame $w.frame
 
-    label $w.frame.image \
-	  -image      [Bitmap::get [pixmap tkabber tkabber-logo.gif]]
+    image create photo tkabber/logo \
+	  -file [fullpath pixmaps default tkabber tkabber-logo.gif]
+
+    label $w.frame.image -image tkabber/logo
 
     label $w.frame.msg   \
 	  -anchor  nw    \
@@ -83,7 +90,7 @@ proc splash_start {{aboutP 0}} {
 	  -text          \
 "Tkabber $version
 
-Copyright \u00a9 2002-2005 [::msgcat::mc {Alexey Shchepin}]
+Copyright \u00a9 2002-2006 [::msgcat::mc {Alexey Shchepin}]
 
 [::msgcat::mc Authors:]
     [::msgcat::mc {Alexey Shchepin}]
