@@ -2,7 +2,7 @@
 # the next line restarts using the correct interpreter \
 exec wish "$0" -name tkabber "$@"
 
-# $Id: tkabber.tcl,v 1.232 2005/07/31 21:09:36 aleksey Exp $
+# $Id: tkabber.tcl,v 1.236 2006/06/08 20:21:44 aleksey Exp $
 
 set interface tk
 namespace eval ifacetk {}
@@ -75,7 +75,7 @@ proc get_snapshot {changelog} {
     return $snapshot
 }
 
-set version "0.9.8"
+set version "0.9.9"
 set toolkit_version "Tcl/Tk [info patchlevel]"
 set debug_lvls {}
 
@@ -96,21 +96,6 @@ proc fullpath {args} {
     return [eval file join [list $rootdir] $args]
 }
 
-proc pixmap {args} {
-    global pixmaps_theme
-
-    if {[cequal [cindex $pixmaps_theme end] /]} {
-	set p [eval file join [list $pixmaps_theme] $args]
-    } else {
-	set p [eval fullpath pixmaps [list $pixmaps_theme] $args]
-    }
-    if {[file exists $p]} {
-	return $p
-    } else {
-	return [eval fullpath pixmaps default $args]
-    }
-}
-
 proc load_source {args} {
     set fullpath [eval fullpath $args]
 
@@ -125,6 +110,7 @@ proc quit {{status 0}} {
     exit $status
 }
 
+namespace eval ifacetk {}
 namespace eval ssj {}
 load_source hooks.tcl
 
@@ -235,6 +221,7 @@ The IMG package is available at http://www.xs4all.nl/~nijtmans/img.html"
 load_source custom.tcl
 load_source utils.tcl
 load_source plugins.tcl
+load_source pixmaps.tcl
 load_source balloon.tcl
 load_source presence.tcl
 load_source iq.tcl
@@ -269,6 +256,10 @@ load_source ifacetk iface.tcl
 
 plugins::load [file join plugins general]
 plugins::load [file join plugins $tcl_platform(platform)]
+if {[info exists env(TKABBER_SITE_PLUGINS)] && \
+	[file isdirectory $env(TKABBER_SITE_PLUGINS)]} {
+    plugins::load_dir  $env(TKABBER_SITE_PLUGINS)
+}
 plugins::load_dir [file join ~ .tkabber plugins]
 
 hook::run postload_hook
