@@ -1,34 +1,52 @@
-# $Id: userinfo.tcl,v 1.56 2005/04/25 23:56:56 aleksey Exp $
+# $Id: userinfo.tcl,v 1.64 2006/05/28 14:54:24 aleksey Exp $
 
 
 namespace eval userinfo {
-
     custom::defvar show_info_list {} [::msgcat::mc "List of users for userinfo."] \
 	    -group Hidden
 }
 
 proc userinfo::show_info_dialog {} {
-    global show_info_jid
+    variable show_info_jid
     variable show_info_list
+    variable show_info_connid
+
+    if {[lempty [jlib::connections]]} return
 
     set gw .userinfo
     catch { destroy $gw }
 
+    set connid [lindex [jlib::connections] 0]
+    set show_info_connid [jlib::connection_jid $connid]
+
     Dialog $gw -title [::msgcat::mc "Show user info"] -separator 1 -anchor e \
 	    -default 0 -cancel 1
 
     set gf [$gw getframe]
     grid columnconfigure $gf 1 -weight 1
 
-    if {[llength $show_info_list]} {set show_info_jid [lindex $show_info_list 0]}
+    set show_info_jid ""
 
     label $gf.ljid -text [::msgcat::mc "JID:"]
-    ecursor_entry [ComboBox $gf.jid -textvariable show_info_jid \
+    ecursor_entry [ComboBox $gf.jid -textvariable [namespace current]::show_info_jid \
 	    -values $show_info_list -width 35].e
     
     grid $gf.ljid -row 0 -column 0 -sticky e
     grid $gf.jid  -row 0 -column 1 -sticky ew
-		    
+
+    if {[llength [jlib::connections]] > 1} {
+	set connections {}
+	foreach c [jlib::connections] {
+	    lappend connections [jlib::connection_jid $c]
+	}
+	label $gf.lconnection -text [::msgcat::mc "Connection:"]
+	ComboBox $gf.connection -textvariable [namespace current]::show_info_connid \
+				-values $connections
+
+	grid $gf.lconnection -row 1 -column 0 -sticky e
+	grid $gf.connection  -row 1 -column 1 -sticky ew
+    }
+
     $gw add -text [::msgcat::mc "Show"] -command "[namespace current]::show_info $gw"
     $gw add -text [::msgcat::mc "Cancel"] -command "destroy $gw"
 
@@ -36,14 +54,25 @@ proc userinfo::show_info_dialog {} {
 }
 
 proc userinfo::show_info {gw} {
-    global show_info_jid
+    variable show_info_jid
     variable show_info_list
+    variable show_info_connid
 
     destroy $gw
+
+    foreach c [jlib::connections] {
+	if {[jlib::connection_jid $c] == $show_info_connid} {
+	    set connid $c
+	}
+    }
+    if {![info exists connid]} {
+	set connid [jlib::route $show_info_jid]
+    }
+
     set show_info_list [update_combo_list $show_info_list $show_info_jid 10]
     set custom::saved([namespace current]::show_info_list) $show_info_list
     custom::store
-    userinfo::open $show_info_jid
+    userinfo::open $show_info_jid -connection $connid
 }
 
 proc userinfo::w_from_jid {jid} {
@@ -100,8 +129,11 @@ proc userinfo::pack_text_entry {jid g row name text} {
     grid $g.l$name -row $row -column 0 -sticky e
     grid $g.$name  -row $row -column 1 -sticky we
     grid columnconfig $g 1 -weight 1 -minsize 0
-    trace variable userinfo($name,$jid) w \
+    trace variable [namespace current]::userinfo($name,$jid) w \
 	[list userinfo::fill_user_description $g.$name userinfo($name,$jid) 0]
+    bind $g.$name <Destroy> \
+	[list trace vdelete [namespace current]::userinfo($name,$jid) w \
+		  [list userinfo::fill_user_description $g.$name userinfo($name,$jid) 0]]
 }
 
 proc userinfo::pack_spinbox {jid g row col name low high text} {
@@ -298,13 +330,22 @@ proc userinfo::open {jid args} {
     fill_user_description $a.text userinfo(desc,$jid) $editable
     pack $sw -fill both -expand yes
     pack $a -fill both -expand yes
-    trace variable userinfo(desc,$jid) w \
+    trace variable [namespace current]::userinfo(desc,$jid) w \
 	[list userinfo::fill_user_description $a.text userinfo(desc,$jid) $editable]
+    bind $a.text <Destroy> \
+	[list trace variable [namespace current]::userinfo(desc,$jid) w \
+		  [list userinfo::fill_user_description $a.text userinfo(desc,$jid) $editable]]
     set userinfo(descfield,$jid) $a.text
 
     manage_focus $jid about $b.bday[expr {$editable ? "year" : ""}] $editable
 
-    set photo [$tab insert end photo -text [::msgcat::mc "Photo"]]
+    if {!$editable} {
+	set photo [$tab insert end photo -text [::msgcat::mc "Photo"] \
+		       -raisecmd [list after idle \
+				      [list [namespace current]::reconfigure_photo $jid]]]
+    } else {
+	set photo [$tab insert end photo -text [::msgcat::mc "Photo"]]
+    }
     
     set p [pack_frame $photo.photo [::msgcat::mc "Photo"]]
     set photo_img photo_$jid
@@ -318,8 +359,8 @@ proc userinfo::open {jid args} {
 	grid rowconfig $p 1 -weight 1
 	set sf [ScrollableFrame $p.sf]
 	$sw setwidget $sf
-	set l [label [$sf getframe].photo -image $photo_img]
-	pack $l
+	set l [label [$sf getframe].photo -image $photo_img -bd 0]
+	grid $l -row 0 -column 0
 	bindscroll $l $sf
 	bindscroll $sf $sf
     } else {
@@ -358,93 +399,9 @@ proc userinfo::open {jid args} {
 
     if {!$editable} {
 	$a.text configure -state disabled
-
-	hook::run userinfo_hook $tab $connid $jid $editable
-
-	global presence
-	set bare_jid [node_and_server_from_jid $jid]
-	set chatid [chat::chatid $connid $bare_jid]
-	if {[chat::is_groupchat $chatid]} {
-	    if {[info exists presence(signed,$connid,$jid)]} {
-		set jids [list signed,$connid,$jid]
-	    } else {
-		set jids [list]
-	    }
-	} else {
-	    set jids [array names presence signed,$connid,$bare_jid/*]
-	}
-	if {[llength $jids] > 0} {
-	    set presenceinfo [$tab insert end presenceinfo -text [::msgcat::mc "Presence"]]
-	    set i 0
-	    foreach j $jids {
-		regsub {signed,\d*,(.*)} $j {\1} fjid
-		set x [pack_frame $presenceinfo.presence_$i $fjid]
-		catch { array unset params }
-		array set params $presence($j)
-
-	        set kv {}
-		set addrs ""
-		set s ""
-		foreach signature $params(signatures) {
-		    foreach {k v} $signature {
-			switch -- $k {
-			    key {
-				foreach {k v} $v {
-				    if {![cequal $k subkeys]} {
-					continue
-				    }
-
-				    foreach subkey $v {
-					catch { unset sparams }
-					array set sparams $subkey
-					if {[info exists sparams(email)]} {
-					    append addrs $s $sparams(email)
-					    set s ", "
-					}
-				    }
-				}
-				continue
-			    }
-
-			    status {
-				continue
-			    }
-
-			    created {
-				set v [clock format $v]
-			    }
-
-			    default {
-				if {[cequal $v ""]} {
-				    continue
-				}
-			    }	
-			}
-
-			lappend kv $k $v
-		    }
-		}
-
-
-		pack_entry $jid $x $i presence_$i Summary:
-		if {![info exists params(reason)]} {
-		    set params(reason) [::msgcat::mc "Presence is signed"]
-		    if {![cequal $addrs ""]} {
-			append params(reason) [::msgcat::mc " by "] $addrs
-		    }
-		}
-		set userinfo(presence_$i,$jid) $params(reason)
-		incr i
-
-		foreach {k v} $kv {
-		    pack_entry $jid $x $i presence_$i ${k}:
-		    set userinfo(presence_$i,$jid) $v
-		    incr i
-		}
-	    }
-	}
     }
 
+    hook::run userinfo_hook $tab $connid $jid $editable
 
     jlib::send_iq get \
 	[jlib::wrapper:createtag vCard \
@@ -459,27 +416,42 @@ proc userinfo::open {jid args} {
     bind $w <Control-Next> [list ifacetk::tab_move $tab 1]
 
     $tab raise $top_page
-    update idletasks
 
     wm deiconify $w
 }
 
+proc userinfo::reconfigure_photo {jid} {
+    set w [w_from_jid $jid]
+    set tab $w.frame.tab
+    set p [[$tab getframe photo].photo getframe]
+    set sw $p.sw
+    set sf $p.sf
+    set l [$sf getframe].photo
+
+    update
+
+    $sf configure -areawidth [max [winfo width $l] [winfo width $sw]] \
+		  -areaheight [max [winfo height $l] [winfo height $sw]]
+}
+
 proc userinfo::client_page {tab connid jid editable} {
+    if {$editable} return
+
     set client [$tab insert end client -text [::msgcat::mc "Client"]]
 
     set c [pack_frame $client.client [::msgcat::mc "Client"]]
     pack_entry $jid $c 0 clientname [::msgcat::mc "Client:"]
     pack_entry $jid $c 1 clientversion [::msgcat::mc "Version:"]
+    pack_entry $jid $c 2 os [::msgcat::mc "OS:"]
 
     set l [pack_frame $client.last [::msgcat::mc "Last Activity or Uptime"]]
     pack_entry $jid $l 0 lastseconds [::msgcat::mc "Interval:"]
     pack_entry $jid $l 1 lastdesc [::msgcat::mc "Description:"]
 
-    set o [pack_frame $client.computer [::msgcat::mc "Computer"]]
-    pack_entry $jid $o 0 os [::msgcat::mc "OS:"]
-    pack_entry $jid $o 1 time [::msgcat::mc "Time:"]
-    pack_entry $jid $o 2 tz   [::msgcat::mc "Time Zone:"]
-    pack_entry $jid $o 3 utc  [::msgcat::mc "UTC:"]
+    set o [pack_frame $client.computer [::msgcat::mc "Time"]]
+    pack_entry $jid $o 0 time [::msgcat::mc "Time:"]
+    pack_entry $jid $o 1 tz   [::msgcat::mc "Time Zone:"]
+    pack_entry $jid $o 2 utc  [::msgcat::mc "UTC:"]
 
     # FIX -to ...
     request_iq version $connid $jid
@@ -529,14 +501,35 @@ proc userinfo::load_photo {jid l} {
     variable userinfo
 
     set photo_img photo_$jid
-    set filename [tk_getOpenFile]
+    if {[catch { package require Img }]} {
+	set types [list [list [::msgcat::mc "GIF images"] {.gif}] \
+			[list [::msgcat::mc "All files"] {*}]]
+    } else {
+	set types [list [list [::msgcat::mc "JPEG images"] {.jpg .jpeg}] \
+			[list [::msgcat::mc "GIF images"] {.gif}] \
+			[list [::msgcat::mc "PNG images"] {.png}] \
+			[list [::msgcat::mc "All files"] {*}]]
+    }
+    set filename [tk_getOpenFile -filetypes $types]
     if {$filename != ""} {
-	if {[catch {image create photo $photo_img -file $filename}]} {
-	    # bla bla bla
+	if {[catch {image create photo $photo_img -file $filename} res]} {
+	    if {[winfo exists .load_photo_error]} {
+		destroy .load_photo_error
+	    }
+	    NonmodalMessageDlg .load_photo_error -aspect 50000 -icon error \
+	        -message [format [::msgcat::mc "Loading photo failed: %s."] \
+			         $res]
 	} else {
 	    set f [::open $filename]
 	    fconfigure $f -translation binary
 	    set userinfo(photo_binval,$jid) [read $f]
+	    binary scan $userinfo(photo_binval,$jid) H4 binsig
+	    switch -- $binsig {
+		ffd8 { set userinfo(photo_type,$jid) "image/jpeg" }
+		4749 { set userinfo(photo_type,$jid) "image/gif" }
+		8950 { set userinfo(photo_type,$jid) "image/png" }
+		default { set userinfo(photo_type,$jid) "image" }
+	    }
 	    close $f
 	    set userinfo(photo_use_binval,$jid) 1
 	}
@@ -668,12 +661,16 @@ proc userinfo::parse_vcard_photo_item {jid items} {
 		    set userinfo(photo_binval,$jid) [base64::decode $chdata]
 		    set userinfo(photo_use,$jid) image
 		    photo_$jid put $chdata
+		    catch { reconfigure_photo $jid }
 		}
 	    }
 	    EXTVAL {
 		set userinfo(photo_extval,$jid) $chdata
 		set userinfo(photo_use,$jid) url
 	    }
+	    TYPE {
+		set userinfo(photo_type,$jid) $chdata
+	    }
 	    default {debugmsg userinfo "Unknown vCard <PHOTO/> subtag $tag"}
 	}
     }
@@ -867,7 +864,7 @@ proc userinfo::parse_iqtime_item {jid items} {
 	    utc     {set userinfo(utc,$jid) $chdata}
 	    display {set userinfo(time,$jid) $chdata}
 	    tz      {set userinfo(tz,$jid) $chdata}
-	    default {debugmsg userinfo "Unknown iq:version tag '$tag'"}
+	    default {debugmsg userinfo "Unknown iq:time tag '$tag'"}
 	}
     }
 }
@@ -886,8 +883,12 @@ proc userinfo::parse_iqlast {jid res child} {
     jlib::wrapper:splitxml $child tag vars isempty chdata children
 
     if {[cequal [jlib::wrapper:getattr $vars xmlns] jabber:iq:last]} {
-	set userinfo(lastseconds,$jid) \
-		[format_time [jlib::wrapper:getattr $vars seconds]]
+	set seconds [jlib::wrapper:getattr $vars seconds]
+	if {[string is integer -strict $seconds]} {
+	    set userinfo(lastseconds,$jid) [format_time $seconds]
+	} else {
+	    set userinfo(lastseconds,$jid) ""
+	}
 	set userinfo(lastdesc,$jid) $chdata
     }
 }
@@ -976,7 +977,7 @@ proc userinfo::make_photo_item {jid} {
 	}
 	image {
 	    if {[info exists userinfo(photo_binval,$jid)]} {
-		append_tag tags TYPE image
+		append_tag tags TYPE $userinfo(photo_type,$jid)
 		append_tag tags \
 		    BINVAL [base64::encode $userinfo(photo_binval,$jid)]
 	    }
@@ -1033,6 +1034,20 @@ proc userinfo::make_org_item {jid} {
     return $tags
 }
 
+proc userinfo::add_menu_item {m connid jid} {
+    $m add command -label [::msgcat::mc "Show info"] \
+           -command [list userinfo::open $jid -connection $connid]
+}
+
+hook::add chat_create_user_menu_hook userinfo::add_menu_item 60
+hook::add chat_create_conference_menu_hook userinfo::add_menu_item 60
+hook::add roster_create_groupchat_user_menu_hook userinfo::add_menu_item 60
+hook::add roster_conference_popup_menu_hook userinfo::add_menu_item 60
+hook::add roster_service_popup_menu_hook userinfo::add_menu_item 60
+hook::add roster_jid_popup_menu_hook userinfo::add_menu_item 60
+hook::add message_dialog_menu_hook userinfo::add_menu_item 60
+hook::add search_popup_menu_hook userinfo::add_menu_item 60
+    
 
 hook::add postload_hook \
     [list browser::register_ns_handler vcard-temp userinfo::open \
