@@ -1,4 +1,4 @@
-# $Id: utils.tcl,v 1.25 2004/08/16 21:08:06 aleksey Exp $
+# $Id: utils.tcl,v 1.29 2006/03/28 19:12:17 aleksey Exp $
 
 proc user_from_jid {jid} {
     set user $jid
@@ -102,16 +102,15 @@ proc tag_to_jid {tag} {
 }
 
 proc double% {str} {
-    regsub -all % $str %% res
-    return $res
+    return [string map {% %%} $str]
 }
 
 proc error_type_condition {errmsg} {
-    return [lrange [jlib::error_to_list $errmsg] 0 1]
+    return [lrange [stanzaerror::error_to_list $errmsg] 0 1]
 }
 
 proc error_to_string {errmsg} {
-    return [lindex [jlib::error_to_list $errmsg] 2]
+    return [lindex [stanzaerror::error_to_list $errmsg] 2]
 }
 
 proc get_group_nick {jid fallback} {
@@ -154,15 +153,15 @@ proc format_time {t} {
 	}
 
 	set sec [expr {$t % 60}]
-	set secs [expr {($sec==1)?"[msgcat::mc second]":"[msgcat::mc seconds]"}]
+	set secs [expr {($sec==1)?"[::msgcat::mc second]":"[::msgcat::mc seconds]"}]
 	set t [expr {$t / 60}]
 	set min [expr {$t % 60}]
-	set mins [expr {($min==1)?"[msgcat::mc minute]":"[msgcat::mc minutes]"}]
+	set mins [expr {($min==1)?"[::msgcat::mc minute]":"[::msgcat::mc minutes]"}]
 	set t [expr {$t / 60}]
 	set hour [expr {$t % 24}]
-	set hours [expr {($hour==1)?"[msgcat::mc hour]":"[msgcat::mc hours]"}]
+	set hours [expr {($hour==1)?"[::msgcat::mc hour]":"[::msgcat::mc hours]"}]
 	set day [expr {$t / 24}]
-	set days [expr {($day==1)?"[msgcat::mc day]":"[msgcat::mc days]"}]
+	set days [expr {($day==1)?"[::msgcat::mc day]":"[::msgcat::mc days]"}]
 
 	set flag 0
 	set message ""
@@ -325,13 +324,14 @@ proc CbDialog {path title buttons var lnames lballoons args} {
 	catch { array unset result }
     }
 
+    set temp {}
     foreach idx [array names names] {
-	set temp($names($idx)) $idx
+	lappend temp [list $idx $names($idx)]
     }
 
     set i 0
-    foreach idxt [lsort -dictionary [array names temp]] {
-	set idx $temp($idxt)
+    foreach idxt [lsort -dictionary -index 1 $temp] {
+	set idx [lindex $idxt 0]
 	if {$radio} {
 	    set cb [radiobutton $sff.cb$i -variable $var \
 	                -text $names($idx) -value $idx]
